@@ -1,4 +1,4 @@
-/* 
+/*
  * ADIOS is freely available under the terms of the BSD license described
  * in the COPYING file in the top level directory of this source distribution.
  *
@@ -15,11 +15,21 @@
 #include "public/adios_mpi.h"
 #include "core/adios_transport_hooks.h"
 #include "core/adios_bp_v1.h"
+#include "core/qhashtbl.h"
+#include "public/adios_schema.h"
+
+// NCSU ALACRITY-ADIOS: Include needed for the transform spec struct
+#include "core/transforms/adios_transforms_specparse.h"
 
 #ifdef SKEL_TIMING
 #include "core/adios_timing.h"
 #endif
 
+/* Some cluster experience MPI_File_write() errors when trying to write INT32_MAX bytes (2GB) at once.
+ * Use this number to limit block sizes written with MPI-IO.
+ */
+#define MAX_MPIWRITE_SIZE 2130706432    /* 2GB - 16MB */
+
 enum ADIOS_METHOD_MODE {adios_mode_write  = 1
                        ,adios_mode_read   = 2
                        ,adios_mode_update = 3 // not supported yet
@@ -28,20 +38,18 @@ enum ADIOS_METHOD_MODE {adios_mode_write  = 1
 
 struct adios_dimension_struct;
 struct adios_var_struct;
-// ADIOS Schema: modifying mesh struct
-struct adios_mesh_struct;
 
 // NCSU - Generic data for statistics
-struct adios_stat_struct 
+struct adios_stat_struct
 {
-	void * data;
+    void * data;
 };
 
 
 struct adios_var_struct
 {
-    uint16_t id;
-    uint16_t parent_id;
+    uint32_t id;
+    struct adios_var_struct *parent_var; // copy_var_written links "written var" to "var definition"
 
     char * name;
     char * path;
@@ -57,17 +65,28 @@ struct adios_var_struct
     enum ADIOS_FLAG free_data;    // primarily used for writing
     void * data;                  // primarily used for reading
     uint64_t data_size;           // primarily used for reading
+    uint32_t write_count; // added to support multiple writes for transform layer.
+                          // Might needed for other things in the future.
 
-	// NCSU - Adding stat related variables
-	struct adios_stat_struct ** stats; // 2D array. Complex numbers can contain upto 3 parts
-	uint32_t bitmap;
+    // NCSU - Adding stat related variables
+    struct adios_stat_struct ** stats; // 2D array. Complex numbers can contain upto 3 parts
+    uint32_t bitmap;
+
+    // NCSU ALACRITY-ADIOS - Adding transform-related fields
+    uint8_t transform_type;
+    struct adios_transform_spec *transform_spec;
+
+    enum ADIOS_DATATYPES pre_transform_type;
+    struct adios_dimension_struct *pre_transform_dimensions;
+    uint16_t transform_metadata_len;
+    void *transform_metadata;
 
     struct adios_var_struct * next;
 };
 
 // NCSU - structure for histogram
 struct adios_hist_struct
-{   
+{
     double min; //minimum value of histogram ** for when we use complex variables
     double max; //maximum value of histogram
     uint32_t num_breaks; //number of break points for the histogram
@@ -77,7 +96,7 @@ struct adios_hist_struct
 
 struct adios_attribute_struct
 {
-    uint16_t id;
+    uint32_t id;
     char * name;
     char * path;
     enum ADIOS_DATATYPES type;
@@ -107,61 +126,22 @@ struct adios_method_list_struct
     struct adios_method_list_struct * next;
 };
 
-enum ADIOS_MESH_TYPE
-{
-     ADIOS_MESH_UNIFORM      = 1
-    ,ADIOS_MESH_STRUCTURED   = 2
-    ,ADIOS_MESH_RECTILINEAR  = 3
-    ,ADIOS_MESH_UNSTRUCTURED = 4
-};
-
-// ADIOS Schema: supported cell types
-enum ADIOS_CELL_TYPE
-{
-     ADIOS_CELL_PT         = 1
-    ,ADIOS_CELL_LINE       = 2
-    ,ADIOS_CELL_TRI        = 3
-    ,ADIOS_CELL_QUAD       = 4
-    ,ADIOS_CELL_HEX        = 5
-    ,ADIOS_CELL_PRI        = 6
-    ,ADIOS_CELL_TET        = 7
-    ,ADIOS_CELL_PYR        = 8
-};
-
-struct adios_mesh_uniform_struct;
-struct adios_mesh_rectilinear_struct;
-struct adios_mesh_structured_struct;
-struct adios_mesh_unstructured_struct;
-
-struct adios_mesh_struct
-{
-    // ADIOS Schema: adding mesh names 
-    // Groups can have multiple meshes
-    char * name;
-    enum ADIOS_FLAG time_varying;
-    enum ADIOS_MESH_TYPE type;
-    union 
-    {
-        struct adios_mesh_uniform_struct * uniform;
-        struct adios_mesh_rectilinear_struct * rectilinear;
-        struct adios_mesh_structured_struct * structured;
-        struct adios_mesh_unstructured_struct * unstructured;
-    };
-    struct adios_mesh_struct * next;
-};
 
 struct adios_group_struct
 {
     uint16_t id;
-    uint16_t member_count;
+    uint16_t member_count; // count of variables + attributes
     uint64_t group_offset;
 
     char * name;
-    int var_count;
+    uint32_t var_count;
     enum ADIOS_FLAG adios_host_language_fortran;
-    enum ADIOS_FLAG all_unique_var_names;
+    enum ADIOS_FLAG all_unique_var_names; // obsolete: remove
     struct adios_var_struct * vars;
+    struct adios_var_struct * vars_tail;  // last variable in the list 'vars'
+    qhashtbl_t *hashtbl_vars;
     struct adios_var_struct * vars_written;
+    struct adios_var_struct * vars_written_tail; // last variable in 'vars_written'
     struct adios_attribute_struct * attributes;
     char * group_comm;
     char * group_by;
@@ -175,7 +155,7 @@ struct adios_group_struct
     struct adios_mesh_struct * meshs;
     int mesh_count;
     enum ADIOS_FLAG all_unique_mesh_names;
-   
+
     int attrid_update_epoch; // ID of special attribute "/__adios__/update_time_epoch" to find it fast
 };
 
@@ -207,18 +187,19 @@ struct adios_file_struct
     uint64_t buffer_size;   // how big the buffer is currently
 
     uint64_t vars_start;    // offset for where to put the var/attr count
-    uint16_t vars_written;  // count of vars/attrs to write
+    uint32_t vars_written;  // count of vars/attrs to write
 
 #ifdef SKEL_TIMING
     struct adios_timing_struct * timing_obj;
 #endif
-
+    MPI_Comm comm;          // duplicate of comm received in adios_open()
 };
 
 struct adios_dimension_item_struct
 {
-    uint64_t rank;
-    uint16_t id;
+    uint64_t rank;                 // for numerical value
+    struct adios_var_struct * var; // for value stored in variable
+    struct adios_attribute_struct * attr; // for value stored in attribute
     enum ADIOS_FLAG time_index;
 };
 
@@ -230,6 +211,102 @@ struct adios_dimension_struct
     struct adios_dimension_struct * next;
 };
 
+//mesh structure used by write method
+// ADIOS Schema: modifying mesh struct
+struct adios_mesh_struct
+{
+    // ADIOS Schema: adding mesh names
+    // Groups can have multiple meshes
+    char * name;
+    enum ADIOS_FLAG time_varying;
+    enum ADIOS_MESH_TYPE type;
+/*    union
+    {
+        struct adios_mesh_uniform_struct * uniform;
+        struct adios_mesh_rectilinear_struct * rectilinear;
+        struct adios_mesh_structured_struct * structured;
+        struct adios_mesh_unstructured_struct * unstructured;
+    };*/
+    struct adios_mesh_struct * next;
+};
+
+////////////////////////
+// mesh support data structures
+////////////////////////
+struct adios_mesh_item_struct
+{
+    double rank;
+    struct adios_var_struct * var;
+};
+
+struct adios_mesh_item_list_struct
+{
+    struct adios_mesh_item_struct item;
+    struct adios_mesh_item_list_struct * next;
+};
+
+struct adios_mesh_var_list_struct
+{
+    struct adios_var_struct * var;
+    struct adios_mesh_var_list_struct * next;
+};
+
+struct adios_mesh_cell_list_struct
+{
+    enum ADIOS_FLAG cells_uniform;
+    struct adios_mesh_item_struct count;
+    struct adios_var_struct * data;
+    struct adios_mesh_item_struct type;
+};
+
+struct adios_mesh_cell_list_list_struct
+{
+    struct adios_mesh_cell_list_struct cell_list;
+    struct adios_mesh_cell_list_list_struct * next;
+};
+
+/*
+// Main mesh structs
+//////////////////////////////////////////////////////////////
+struct adios_mesh_uniform_struct
+{
+    struct adios_mesh_item_list_struct * dimensions;
+    struct adios_mesh_item_list_struct * origin;
+    struct adios_mesh_item_list_struct * spacing;
+    // ADIOS Schema: adding option to provide origin and maximum
+    // instead restricting users to origin and spacing
+    struct adios_mesh_item_list_struct * maximum;
+};
+
+struct adios_mesh_rectilinear_struct
+{
+    enum ADIOS_FLAG coordinates_single_var;
+    struct adios_mesh_item_list_struct * dimensions;
+    struct adios_mesh_var_list_struct * coordinates;
+};
+
+struct adios_mesh_structured_struct
+{
+    enum ADIOS_FLAG points_single_var;
+    struct adios_mesh_item_struct * nspace;
+    struct adios_mesh_item_list_struct * dimensions;
+    struct adios_mesh_var_list_struct * points;
+};
+
+struct adios_mesh_unstructured_struct
+{
+    // ADIOS Schema: adding single/multi points option
+    // adding nspace to allow 2D mesh in 3D for example,
+    // finally adding the concept of cellset/cellsetcount
+    enum ADIOS_FLAG points_single_var;
+    struct adios_mesh_item_struct * nspace;
+    struct adios_mesh_var_list_struct * points;
+    struct adios_mesh_item_struct * points_count;
+    struct adios_mesh_cell_list_list_struct * cell_list;
+    struct adios_mesh_item_struct * cell_set_count;
+};
+*/
+
 typedef void (* ADIOS_INIT_FN) (const PairStruct * parameters
                                ,struct adios_method_struct * method
                                );
@@ -271,6 +348,7 @@ typedef void (* ADIOS_STOP_CALCULATION_FN)
 
 struct adios_transport_struct
 {
+    char * method_name;
     ADIOS_INIT_FN adios_init_fn;
     ADIOS_OPEN_FN adios_open_fn;
     ADIOS_SHOULD_BUFFER_FN adios_should_buffer_fn;
@@ -298,81 +376,6 @@ struct adios_parse_buffer_struct
     void * buffer;
 };
 
-////////////////////////
-// mesh support data structures
-////////////////////////
-struct adios_mesh_item_struct
-{
-    double rank;
-    struct adios_var_struct * var;
-};
-
-struct adios_mesh_item_list_struct
-{
-    struct adios_mesh_item_struct item;
-    struct adios_mesh_item_list_struct * next;
-};
-
-struct adios_mesh_var_list_struct
-{
-    struct adios_var_struct * var;
-    struct adios_mesh_var_list_struct * next;
-};
-
-struct adios_mesh_cell_list_struct
-{
-    enum ADIOS_FLAG cells_uniform;
-    struct adios_mesh_item_struct count;
-    struct adios_var_struct * data;
-    struct adios_mesh_item_struct type;
-};
-
-struct adios_mesh_cell_list_list_struct
-{
-    struct adios_mesh_cell_list_struct cell_list;
-    struct adios_mesh_cell_list_list_struct * next;
-};
-
-//////////////////////////////////////////////////////////////
-// Main mesh structs
-//////////////////////////////////////////////////////////////
-struct adios_mesh_uniform_struct
-{
-    struct adios_mesh_item_list_struct * dimensions;
-    struct adios_mesh_item_list_struct * origin;
-    struct adios_mesh_item_list_struct * spacing;
-    // ADIOS Schema: adding option to provide origin and maximum
-    // instead restricting users to origin and spacing
-    struct adios_mesh_item_list_struct * maximum;
-};
-
-struct adios_mesh_rectilinear_struct
-{
-    enum ADIOS_FLAG coordinates_single_var;
-    struct adios_mesh_item_list_struct * dimensions;
-    struct adios_mesh_var_list_struct * coordinates;
-};
-
-struct adios_mesh_structured_struct
-{
-    enum ADIOS_FLAG points_single_var;
-    struct adios_mesh_item_struct * nspace;
-    struct adios_mesh_item_list_struct * dimensions;
-    struct adios_mesh_var_list_struct * points;
-};
-
-struct adios_mesh_unstructured_struct
-{
-    // ADIOS Schema: adding single/multi points option
-    // adding nspace to allow 2D mesh in 3D for example,
-    // finally adding the concept of cellset/cellsetcount
-    enum ADIOS_FLAG points_single_var;
-    struct adios_mesh_item_struct * nspace;
-    struct adios_mesh_var_list_struct * points;
-    struct adios_mesh_item_struct * points_count;
-    struct adios_mesh_cell_list_list_struct * cell_list;
-    struct adios_mesh_item_struct * cell_set_count;
-};
 
 //////////////////////////////////////////////////////////////////////////////
 // Function Delcarations
@@ -385,16 +388,14 @@ uint64_t adios_data_size (struct adios_group_struct * g);
 struct adios_method_list_struct * adios_get_methods (void);
 struct adios_group_list_struct * adios_get_groups (void);
 
-struct adios_var_struct * adios_find_var_by_name (struct adios_var_struct * root
-                                                 ,const char * name
-                                                 ,enum ADIOS_FLAG unique_names
-                                                 );
+struct adios_var_struct * adios_find_var_by_name (struct adios_group_struct * g,
+                                                  const char * fullpath);
 struct adios_var_struct * adios_find_var_by_id (struct adios_var_struct * root
-                                               ,uint16_t id
+                                               ,uint32_t id
                                                );
 struct adios_attribute_struct * adios_find_attribute_by_id
                                          (struct adios_attribute_struct * root
-                                         ,uint16_t id
+                                         ,uint32_t id
                                          );
 
 struct adios_attribute_struct * adios_find_attribute_var_by_name
@@ -402,10 +403,10 @@ struct adios_attribute_struct * adios_find_attribute_var_by_name
                                        ,const char * name
                                        );
 
-struct adios_attribute_struct * adios_find_attribute_var_by_id
+/*struct adios_attribute_struct * adios_find_attribute_var_by_id
                                        (struct adios_attribute_struct * root
-                                       ,uint16_t id
-                                       );
+                                       ,uint32_t id
+                                       );*/
 
 /*void adios_parse_buffer (struct adios_file_struct * fd, char * buffer
                         ,uint64_t len
@@ -441,11 +442,7 @@ void adios_add_method_to_group (struct adios_method_list_struct ** root
 
 void adios_append_group (struct adios_group_struct * group);
 
-// is the var name unique
-enum ADIOS_FLAG adios_append_var (struct adios_var_struct ** root
-                                 ,struct adios_var_struct * var
-                                 ,uint16_t id
-                                 );
+//void adios_append_var (struct adios_group_struct * g, struct adios_var_struct * var);
 
 void adios_append_dimension (struct adios_dimension_struct ** root
                             ,struct adios_dimension_struct * dimension
@@ -453,7 +450,7 @@ void adios_append_dimension (struct adios_dimension_struct ** root
 
 void adios_append_attribute (struct adios_attribute_struct ** root
                             ,struct adios_attribute_struct * attribute
-                            ,uint16_t id
+                            ,uint32_t id
                             );
 
 int adios_common_declare_group (int64_t * id, const char * name
@@ -471,18 +468,23 @@ int64_t adios_common_define_var (int64_t group_id, const char * name
                                 ,const char * local_offsets
                                 );
 
-int adios_common_define_var_characteristcs  (struct adios_group_struct * g, const char * var_name
-                            				,const char * bin_interval 
-                            				,const char * bin_min
-                            				,const char * bin_max
-                            				,const char * bin_count
-                            				);
+// set a transform method for a variable (=none if this function is never called)
+int adios_common_set_transform (int64_t var_id, const char *transform_type_str);
+
+int adios_common_define_var_characteristics  (struct adios_group_struct * g
+                                              ,const char * var_name
+                                              ,const char * bin_interval
+                                              ,const char * bin_min
+                                              ,const char * bin_max
+                                              ,const char * bin_count
+                                             );
 
 void adios_common_get_group (int64_t * group_id, const char * name);
 int adios_common_free_group (int64_t id);
 
 // ADIOS file format functions
 
+uint16_t adios_calc_var_characteristics_dims_overhead(struct adios_dimension_struct * d);
 uint16_t adios_calc_var_overhead_v1 (struct adios_var_struct * v);
 uint32_t adios_calc_attribute_overhead_v1 (struct adios_attribute_struct * a);
 uint64_t adios_calc_overhead_v1 (struct adios_file_struct * fd);
@@ -491,10 +493,18 @@ int adios_write_version_v1 (char ** buffer
                            ,uint64_t * buffer_size
                            ,uint64_t * buffer_offset
                            );
+int adios_write_version_flag_v1 (char ** buffer
+                           ,uint64_t * buffer_size
+                           ,uint64_t * buffer_offset
+                           ,uint32_t flag
+                           );
 int adios_write_process_group_header_v1 (struct adios_file_struct * fd
                                         ,uint64_t total_size
                                         );
 
+void adios_copy_var_written (struct adios_group_struct * g,
+                             struct adios_var_struct * var);
+
 // data is only there for sizing
 uint64_t adios_write_var_header_v1 (struct adios_file_struct * fd
                                    ,struct adios_var_struct * v
@@ -515,38 +525,47 @@ int adios_write_open_vars_v1 (struct adios_file_struct * fd);
 int adios_write_close_vars_v1 (struct adios_file_struct * fd);
 int adios_write_open_attributes_v1 (struct adios_file_struct * fd);
 int adios_write_close_attributes_v1 (struct adios_file_struct * fd);
+
+// allocate the adios_index_struct, freed in adios_free_index_v1
+struct adios_index_struct_v1 * adios_alloc_index_v1 (int alloc_hashtables);
+
 int adios_write_index_v1 (char ** buffer
                          ,uint64_t * buffer_size
                          ,uint64_t * buffer_offset
                          ,uint64_t index_start
-                         ,struct adios_index_process_group_struct_v1 * pg_root
-                         ,struct adios_index_var_struct_v1 * vars_root
-                         ,struct adios_index_attribute_struct_v1 * attrs_root
+                         ,struct adios_index_struct_v1 * index
                          );
 
 void adios_build_index_v1 (struct adios_file_struct * fd
-                       ,struct adios_index_process_group_struct_v1 ** pg_root
-                       ,struct adios_index_var_struct_v1 ** vars_root
-                       ,struct adios_index_attribute_struct_v1 ** attrs_root
+                         ,struct adios_index_struct_v1 * index
                        );
+
 void adios_merge_index_v1 (
-                   struct adios_index_process_group_struct_v1 ** main_pg_root
-                  ,struct adios_index_var_struct_v1 ** main_vars_root
-                  ,struct adios_index_attribute_struct_v1 ** main_attrs_root
+                   struct adios_index_struct_v1 * main_index
                   ,struct adios_index_process_group_struct_v1 * new_pg_root
                   ,struct adios_index_var_struct_v1 * new_vars_root
                   ,struct adios_index_attribute_struct_v1 * new_attrs_root
                   );
-void adios_clear_index_v1 (struct adios_index_process_group_struct_v1 * pg_root
-                          ,struct adios_index_var_struct_v1 * vars_root
-                          ,struct adios_index_attribute_struct_v1 * attrs_root
-                          );
+
+void adios_sort_index_v1 (struct adios_index_process_group_struct_v1 ** p1
+                         ,struct adios_index_var_struct_v1 ** v1
+                         ,struct adios_index_attribute_struct_v1 ** a1
+                         );
+ 
+void adios_clear_index_v1 (struct adios_index_struct_v1 * index); // in each adios_<method>_close()
+void adios_free_index_v1 (struct adios_index_struct_v1 * index);  // in adios_<method>_finalize()
+
+int adios_parse_scalar_string (enum ADIOS_DATATYPES type, char * value, void ** out);
+
+// NCSU ALACRITY-ADIOS - This function was static, but is now needed in adios_transforms_*.c
+uint8_t count_dimensions (const struct adios_dimension_struct * dimensions);
 
 uint64_t adios_get_type_size (enum ADIOS_DATATYPES type, void * var);
-uint64_t adios_get_var_size (struct adios_var_struct * var
-                            ,struct adios_group_struct * group, void * data
-                            );
-
+// NCSU ALACRITY-ADIOS - added this for use in the transform layer
+uint64_t adios_get_dimension_space_size (struct adios_var_struct * var
+                                        ,struct adios_dimension_struct * d);
+uint64_t adios_get_var_size (struct adios_var_struct * var, void * data);
+uint64_t adios_get_dim_value (struct adios_dimension_item_struct * dimension);
 uint64_t adios_get_stat_size (void * data, enum ADIOS_DATATYPES type, enum ADIOS_STAT stat_id);
 uint8_t adios_get_stat_set_count (enum ADIOS_DATATYPES type);
 
@@ -562,6 +581,52 @@ int adios_parse_method (const char * buf, enum ADIOS_IO_METHOD * method
 /* some internal functions that adios_internals.c and adios_internals_mxml.c share */
 int adios_int_is_var (const char * temp); // 1 == yes, 0 == no
 int adios_int_is_num (char * temp); // 1 == yes, 0 == no
+void adios_conca_mesh_numb_att_nam (char ** returnstr, const char * meshname, char * att_nam, char counterstr[5]);
+void adios_conca_mesh_att_nam (char ** returnstr, const char * meshname, char * att_nam);
+
+// No-XML API
+int adios_common_define_schema_version (struct adios_group_struct * new_group, char * schema_version);
+int adios_common_define_var_mesh (int64_t ptr_new_group, const char * varname, const char * meshname, const char * path);
+int adios_common_define_var_centering (int64_t ptr_new_group, const char * varname, const char * centering, const char * path);
+int adios_common_define_var_timesteps (const char * timesteps,struct adios_group_struct * new_group,const char * name, const char *path);
+int adios_common_define_var_timescale (const char * timescale,struct adios_group_struct * new_group,const char * name, const char *path);
+int adios_common_define_var_timeseriesformat (const char * timeseries,struct adios_group_struct * new_group,const char * name, const char *path);
+int adios_common_define_var_hyperslab ( const char * hyperslab,struct adios_group_struct * new_group,const char * name, const char *path);
+
+// defineMesh functions (missing mesh structs for now dueto problems checking accross groups
+int adios_common_define_mesh_file (int64_t ptr_new_group, char * name, char * file);
+int adios_common_define_mesh_group (int64_t ptr_new_group, const char * name, const char * group);
+int adios_common_define_mesh_timeVarying (const char * timevarying, int64_t group_id, const char * name);
+int adios_common_define_mesh_timeSeriesFormat (const char * timeseries, struct adios_group_struct * new_group, const char * name);
+int adios_common_define_mesh_timeScale (const char * timescale, struct adios_group_struct * new_group, const char * name);
+int adios_common_define_mesh_timeSteps (const char * timesteps, struct adios_group_struct * new_group, const char * name);
+
+int adios_define_mesh_nspace (const char * nspace,struct adios_group_struct * new_group,const char * name);
+int adios_common_define_mesh_rectilinear (char * dimensions, char * coordinates, char * nspace, const char * name, int64_t group_id);
+int adios_define_mesh_rectilinear_dimensions (const char * dimensions,struct adios_group_struct * new_group,const char * name);
+int adios_define_mesh_rectilinear_coordinatesSingleVar (const char * coordinates,struct adios_group_struct * new_group,const char * name);
+int adios_define_mesh_rectilinear_coordinatesMultiVar (const char * coordinates,struct adios_group_struct * new_group,const char * name);
+
+int adios_common_define_mesh_uniform (char * dimensions, char * origin, char * spacing, char * maximum, char * nspace, const char * name, int64_t group_id);
+int adios_define_mesh_uniform_dimensions (const char * dimensions, struct adios_group_struct * new_group, const char * name);
+int adios_define_mesh_uniform_origins (const char * origin ,struct adios_group_struct * new_group,const char * name);
+int adios_define_mesh_uniform_spacings (const char * spacing,struct adios_group_struct * new_group,const char * name);
+int adios_define_mesh_uniform_maximums (const char * maximum,struct adios_group_struct * new_group,const char * name);
+
+int adios_common_define_mesh_structured (char * dimensions, char * nspace, char * points, const char * name, int64_t group_id);
+int adios_define_mesh_structured_dimensions (const char * dimensions,struct adios_group_struct * new_group,const char * name);
+//int adios_define_mesh_structured_nspace (const char * nspace,struct adios_group_struct * new_group,const char * name);
+int adios_define_mesh_structured_pointsSingleVar (const char * points,struct adios_group_struct * new_group,const char * name);
+int adios_define_mesh_structured_pointsMultiVar (const char * points,struct adios_group_struct * new_group,const char * name);
+
+int adios_common_define_mesh_unstructured (char * points, char * data, char * count, char * type, char * nspace, char * npoints, const char * name, int64_t group_id);
+int adios_define_mesh_unstructured_npoints (const char * npoints,struct adios_group_struct * new_group ,const char * name);
+//int adios_define_mesh_unstructured_nspace (const char * nspace,struct adios_group_struct * new_group,const char * name);
+int adios_define_mesh_unstructured_pointsSingleVar (const char * points,struct adios_group_struct * new_group,const char * name);
+int adios_define_mesh_unstructured_pointsMultiVar (const char * points,struct adios_group_struct * new_group ,const char * name);
+int adios_define_mesh_unstructured_uniformCells (const char * count,const char * data,const char * type,struct adios_group_struct * new_group,const char * name);
+int adios_define_mesh_unstructured_mixedCells (const char * count,const char * data,const char * types,struct adios_group_struct * new_group,const char * name);
+
 
 // queue code for adaptive message passing
 #ifdef __cplusplus
