@@ -47,13 +47,14 @@
 #undef adios_type_to_string
 #undef adios_type_size
 #undef adios_print_groupinfo
-//#undef adios_print_fileinfo
+#undef adios_print_fileinfo
 
 
 #include "core/common_read.h"
 
 
 #include "public/adios_error.h"
+#include "core/bp_utils.h"
 #define BYTE_ALIGN 8
 
 static enum ADIOS_READ_METHOD lastmethod = ADIOS_READ_METHOD_BP;
@@ -224,8 +225,7 @@ static ADIOS_VARINFO_V1 * adios_varinfo_to_v1 (ADIOS_GROUP_V1 *gp, ADIOS_VARINFO
         v->value = vi->value; 
              
         /* TIME dimension should be emulated here !!! */
-        int timed = common_read_is_var_timed((ADIOS_FILE *)gp->fp->internal_data, vi->varid);
-        if (timed) {
+        if (vi->nsteps > 1) {
             v->ndim = vi->ndim + 1;
             v->timedim = 0;
         } else {
@@ -234,7 +234,7 @@ static ADIOS_VARINFO_V1 * adios_varinfo_to_v1 (ADIOS_GROUP_V1 *gp, ADIOS_VARINFO
         }
         int tidx = 0, i;
         v->dims = (uint64_t *) malloc (sizeof(uint64_t) * (v->ndim));
-        if (timed) {
+        if (vi->nsteps > 1) {
             v->dims[0] = vi->nsteps;
             tidx=1;
         }
@@ -250,7 +250,7 @@ static ADIOS_VARINFO_V1 * adios_varinfo_to_v1 (ADIOS_GROUP_V1 *gp, ADIOS_VARINFO
         }
 
         if (stat) {
-            v->characteristics_count;// = stat->characteristics_count; FIXME
+            v->characteristics_count=0;// = stat->characteristics_count; FIXME
             v->gmin = stat->min; 
             v->gmax = stat->max; 
             v->gavg = stat->avg; 
@@ -348,7 +348,7 @@ int64_t adios_read_var_byid_v1 (ADIOS_GROUP_V1 * gp,
 
     ADIOS_SELECTION * sel = adios_selection_boundingbox (ndim, start+tidx, count+tidx);
 
-    common_read_schedule_read_byid (f, sel, varid, from_step, nsteps, data);
+    common_read_schedule_read_byid (f, sel, varid, from_step, nsteps, NULL, data); // NCSU ALACRITY-ADIOS
     int ret = common_read_perform_reads (f, 1);
     int64_t rbytes;
     if (ret == err_no_error) {
@@ -425,7 +425,7 @@ int64_t adios_read_local_var_v1 (ADIOS_GROUP_V1 * gp,
 
     ADIOS_SELECTION * sel = adios_selection_writeblock (idx_in_step);
 
-    common_read_schedule_read_byid (f, sel, vi->varid, step, 1, data);
+    common_read_schedule_read_byid (f, sel, vi->varid, step, 1, NULL, data); // NCSU ALACRITY-ADIOS
     int ret = common_read_perform_reads (f, 1);
     int64_t rbytes;
     if (ret == err_no_error) {
@@ -472,7 +472,7 @@ void adios_print_fileinfo_v1 (ADIOS_FILE_V1 *fp)
 // NCSU - Timer series analysis, correlation
 double adios_stat_cor_v1 (ADIOS_VARINFO_V1 * vix, ADIOS_VARINFO_V1 * viy, char * characteristic, uint32_t time_start, uint32_t time_end, uint32_t lag)
 {
-    int i,j;
+    int i;
 
     double avg_x = 0.0, avg_y = 0.0, avg_lag = 0.0;
     double var_x = 0.0, var_y = 0.0, var_lag = 0.0;
@@ -480,7 +480,7 @@ double adios_stat_cor_v1 (ADIOS_VARINFO_V1 * vix, ADIOS_VARINFO_V1 * viy, char *
 
     if (vix == NULL)
     {
-	adios_error(err_invalid_argument, "Variable not defined\n");
+        adios_error(err_invalid_argument, "Variable not defined\n");
         return 0;
     }
 
@@ -680,7 +680,7 @@ double adios_stat_cor_v1 (ADIOS_VARINFO_V1 * vix, ADIOS_VARINFO_V1 * viy, char *
 //covariance(x,y) = sum(i=1,..N) [(x_1 - x_mean)(y_i - y_mean)]/N
 double adios_stat_cov_v1 (ADIOS_VARINFO_V1 * vix, ADIOS_VARINFO_V1 * viy, char * characteristic, uint32_t time_start, uint32_t time_end, uint32_t lag)
 {
-    int i,j;
+    int i;
 
     double avg_x = 0.0, avg_y = 0.0, avg_lag = 0.0;
     double cov = 0;
