@@ -18,6 +18,7 @@
 #include "core/globals.h"
 #include "public/adios_error.h"
 #include "core/adios_logger.h"
+#include "core/adios_timing.h"
 
 #ifdef __cplusplus
 extern "C"  /* prevent C++ name mangling */
@@ -34,7 +35,7 @@ extern int adios_errno;
 void FC_FUNC_(adios_set_application_id, ADIOS_SET_APPLICATION_ID) (int *id, int * err)
 {
     globals_adios_set_application_id (*id);
-    if (err != 0) *err = 0;
+    if (err != 0) *err = err_no_error;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -48,7 +49,7 @@ void FC_FUNC_(adios_init, ADIOS_INIT) (const char * config, MPI_Fint * comm, int
         *err = common_adios_init (buf1, c_comm);
         free (buf1);
     } else {
-        *err = -adios_errno;
+        *err = adios_errno;
     }
 }
 
@@ -94,7 +95,7 @@ void FC_FUNC_(adios_open, ADIOS_OPEN)
         free (buf2);
         free (buf3);
     } else {
-        *err = -adios_errno;
+        *err = adios_errno;
     }
 }
 
@@ -120,7 +121,7 @@ void FC_FUNC_(adios_write_byid, ADIOS_WRITE_BYID)
     if (!fd)
     {
         adios_error (err_invalid_file_pointer, "Invalid handle passed to adios_write\n");
-        *err = 1;
+        *err = adios_errno;
         return;
     }
 
@@ -128,7 +129,7 @@ void FC_FUNC_(adios_write_byid, ADIOS_WRITE_BYID)
     if (!v)
     {
         adios_error (err_invalid_varid, "Invalid id passed to adios_write_byid\n");
-        *err = 1;
+        *err = adios_errno;
         return;
     }
 
@@ -137,14 +138,14 @@ void FC_FUNC_(adios_write_byid, ADIOS_WRITE_BYID)
     if (m && m->next == NULL && m->method->m == ADIOS_METHOD_NULL)
     {
         // nothing to do so just return
-        *err = 0;
+        *err = err_no_error;
         return;
     }
 
     if (!var)
     {
         adios_error (err_invalid_data, "Invalid data (NULL pointer) passed to write\n");
-        *err = 1;
+        *err = adios_errno;
         return;
     }
 
@@ -180,7 +181,7 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
     if (!fd)
     {
         adios_error (err_invalid_file_pointer, "Invalid handle passed to adios_write\n");
-        *err = 1;
+        *err = adios_errno;
         return;
     }
 
@@ -190,7 +191,7 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
     if (m && m->next == NULL && m->method->m == ADIOS_METHOD_NULL)
     {
         // nothing to do so just return
-        *err = 0;
+        *err = err_no_error;
         return;
     }
 
@@ -200,16 +201,16 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
     //printf("  -- adios_write: name=[%s] var size = %d\n", buf1, var_size);
 
     if (!buf1) {
-        *err = -adios_errno;
+        *err = adios_errno;
         return;
     }
 
-    v = adios_find_var_by_name (v, buf1, fd->group->all_unique_var_names);
+    v = adios_find_var_by_name (fd->group, buf1);
 
     if (!v)
     {
         adios_error (err_invalid_varname, "Bad var name (ignored): '%s'\n", buf1);
-        *err = 1;
+        *err = adios_errno;
         free (buf1);
         return;
     }
@@ -221,7 +222,7 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
            )
         {
             adios_error (err_invalid_file_mode, "write attempted on %s in %s.  This was opened for read\n" ,buf1 , fd->name);
-            *err = 1;
+            *err = adios_errno;
             free (buf1);
             return;
         }
@@ -230,7 +231,7 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
     if (!var)
     {
         adios_error (err_invalid_data, "Invalid data (NULL pointer) passed to write for variable %s\n", buf1);
-        *err = 1;
+        *err = adios_errno;
         free (buf1);
         return;
     }
@@ -242,6 +243,7 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
     }
 
     // Q.L. 10-2010. To fix a memory leak problem.
+    // NCSU - Clear stats
     if (v->stats)
     {   
         int j, idx;
@@ -307,7 +309,7 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
                     adios_error (err_no_memory, 
                                  "In adios_write, cannot allocate %lld bytes to copy scalar %s\n",
                                  element_size, v->name);
-                    *err = 1;
+                    *err = adios_errno;
                     free (buf1);
                     return;
                 }
@@ -321,7 +323,7 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
                     adios_error (err_no_memory, 
                                  "In adios_write, cannot allocate %lld bytes to copy string %s\n",
                                  element_size, v->name);
-                    *err = 1;
+                    *err = adios_errno;
                     free (buf1);
                     return;
                 }
@@ -336,7 +338,7 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
     *err = common_adios_write (fd, v, var);
     if (fd->mode == adios_mode_write || fd->mode == adios_mode_append)
     {
-        adios_copy_var_written (&fd->group->vars_written, v, fd);
+        adios_copy_var_written (fd->group, v);
     }
 
     free (buf1);
@@ -368,7 +370,7 @@ void FC_FUNC_(adios_get_write_buffer, ADIOS_GET_WRITE_BUFFER)
         *err = common_adios_get_write_buffer (*fd_p, buf1, (uint64_t *) size, buffer);
         free (buf1);
     } else {
-        *err = -adios_errno;
+        *err = adios_errno;
     }
 }
 
@@ -386,7 +388,7 @@ void FC_FUNC_(adios_read, ADIOS_READ)
         *err = common_adios_read (*fd_p, buf1, buffer, *buffer_size);
         free (buf1);
     } else {
-        *err = -adios_errno;
+        *err = adios_errno;
     }
 }
 
@@ -409,7 +411,7 @@ void FC_FUNC_(adios_set_path, ADIOS_SET_PATH)
         *err = common_adios_set_path (*fd_p, buf1);
         free (buf1);
     } else {
-        *err = -adios_errno;
+        *err = adios_errno;
     }
 }
 
@@ -428,7 +430,7 @@ void FC_FUNC_(adios_set_path_var, ADIOS_SET_PATH_VAR)
         free (buf1);
         free (buf2);
     } else {
-        *err = -adios_errno;
+        *err = adios_errno;
     }
 }
 
@@ -488,20 +490,20 @@ void FC_FUNC_(adios_declare_group, ADIOS_DECLARE_GROUP)
     char * buf1 = 0;
     char * buf2 = 0;
 
+    adios_errno = err_no_error;
     buf1 = futils_fstr_to_cstr (name, name_size);
     buf2 = futils_fstr_to_cstr (time_index, time_index_size);
 
     if (buf1 != 0 && buf2 != 0) {
-        *err = adios_common_declare_group (id, buf1, adios_flag_yes, "", "", buf2, *stats);
+        int ret = adios_common_declare_group (id, buf1, adios_flag_yes, "", "", buf2, *stats);
         free (buf1);
         free (buf2);
-        if (*err == 1) {
+        if (ret == 1) {
             struct adios_group_struct * g = (struct adios_group_struct *) *id;
             g->all_unique_var_names = adios_flag_no;
         }
-    } else {
-        *err = -adios_errno;
     }
+    *err = adios_errno;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -523,6 +525,7 @@ void FC_FUNC_(adios_define_var, ADIOS_DEFINE_VAR)
     char * buf4 = 0;
     char * buf5 = 0;
 
+    adios_errno = err_no_error;
     buf1 = futils_fstr_to_cstr (name, name_size);
     buf2 = futils_fstr_to_cstr (path, path_size);
     buf3 = futils_fstr_to_cstr (dimensions, dimensions_size);
@@ -532,8 +535,7 @@ void FC_FUNC_(adios_define_var, ADIOS_DEFINE_VAR)
     if (buf1 != 0 && buf2 != 0) {
         *id = adios_common_define_var (*group_id, buf1, buf2
                                        ,(enum ADIOS_DATATYPES) *type
-                                       ,buf3, buf4, buf5
-                                       );
+                                       ,buf3, buf4, buf5);
 
         free (buf1);
         free (buf2);
@@ -545,6 +547,23 @@ void FC_FUNC_(adios_define_var, ADIOS_DEFINE_VAR)
     }
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// adios_common_set_transform is in adios_internals.c
+// set the transform method for the selected variable (default is "none")
+void FC_FUNC_(adios_set_transform, ADIOS_SET_TRANSFORM)
+    (int64_t * var_id, const char *transform_type_str, int *err, 
+     int str_size)
+{
+    char * buf1 = 0;
+    adios_errno = err_no_error;
+    buf1 = futils_fstr_to_cstr (transform_type_str, str_size);
+    if (buf1 != 0) {
+        *err = adios_common_set_transform (*var_id, buf1);
+        free (buf1);
+    } else {
+        *err = adios_errno;
+    }
+}
 ///////////////////////////////////////////////////////////////////////////////
 // adios_common_define_attribute is in adios_internals.c
 void FC_FUNC_(adios_define_attribute, ADIOS_DEFINE_ATTRIBUTE) 
@@ -560,24 +579,23 @@ void FC_FUNC_(adios_define_attribute, ADIOS_DEFINE_ATTRIBUTE)
     char * buf3 = 0;
     char * buf4 = 0;
 
+    adios_errno = err_no_error;
     buf1 = futils_fstr_to_cstr (name, name_size);
     buf2 = futils_fstr_to_cstr (path, path_size);
     buf3 = futils_fstr_to_cstr (value, value_size);
     buf4 = futils_fstr_to_cstr (var, var_size);
 
     if (buf1 != 0 && buf2 != 0 && buf3 != 0 && buf4 != 0) {
-        *err = adios_common_define_attribute (*group, buf1, buf2
-                                             ,(enum ADIOS_DATATYPES) *type, buf3
-                                             ,buf4
-                                             );
-
+        adios_common_define_attribute (*group, buf1, buf2
+                                      ,(enum ADIOS_DATATYPES) *type, buf3
+                                      ,buf4
+                                      );
         free (buf1);
         free (buf2);
         free (buf3);
         free (buf4);
-    } else {
-        *err = -adios_errno;
     }
+    *err = adios_errno;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -592,20 +610,21 @@ void FC_FUNC_(adios_select_method, ADIOS_SELECT_METHOD)
     char * buf1 = 0;
     char * buf2 = 0;
     char * buf3 = 0;
+
+    adios_errno = err_no_error;
     buf1 = futils_fstr_to_cstr (method, method_size);
     buf2 = futils_fstr_to_cstr (parameters, parameters_size);
     buf3 = futils_fstr_to_cstr (base_path, base_path_size);
 
     if (buf1 != 0 && buf2 != 0 && buf3 != 0) {
         struct adios_group_struct * g = (struct adios_group_struct *) (* group);
-        *err = adios_common_select_method (0, buf1, buf2, g->name, buf3, 0);
+        adios_common_select_method (0, buf1, buf2, g->name, buf3, 0);
 
         free (buf1);
         free (buf2);
         free (buf3);
-    } else {
-        *err = -adios_errno;
     }
+    *err = adios_errno;
 }
 
 
