@@ -11,26 +11,30 @@
 #ifndef __ADIOS_READ_VER2_H__
 #define __ADIOS_READ_VER2_H__
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+#include <stdint.h>
 
 #include "adios_mpi.h"
 #include "adios_types.h"
 #include "adios_selection.h"
+#include "adios_schema.h"
+#include "adios_read_v2_fwd.h"
 
-#include <stdint.h>
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /*************************/
 /* Types used in the API */
 /*************************/
 
-typedef struct {
+struct _ADIOS_FILE {
         uint64_t fh;                /* File handler                                                   */
         int      nvars;             /* Number of variables in all groups (with full path)             */
         char     ** var_namelist;   /* Variable names in a char* array                                */
         int      nattrs;            /* Number of attributes in all groups                             */
         char     ** attr_namelist;  /* Attribute names in a char* array                               */
+        int      nmeshes;            /* Number of attributes in all groups                            */
+        char     ** mesh_namelist;  /* Attribute names in a char* array                               */
 
         /* Stream step information */
         int      current_step;      /* The current step in a stream. For a file, it is always 0.      */
@@ -45,9 +49,9 @@ typedef struct {
 
         /* Internals */
         void     * internal_data;   /* Data for internal use                                          */
-} ADIOS_FILE;
+};
 
-typedef struct {
+struct _ADIOS_VARSTAT {
         void     * min;            /* minimum value in an array variable, = value for a scalar       */
         void     * max;            /* maximum value of an array variable (over all steps)            */
         double   * avg;            /* average value of an array variable (over all steps)            */
@@ -78,17 +82,25 @@ typedef struct {
             uint32_t ** frequencies;
             uint32_t *  gfrequencies;
         } *histogram;
+};
 
-} ADIOS_VARSTAT;
-
-
-typedef struct {
+struct _ADIOS_VARBLOCK {
     uint64_t * start;      /* offset start point in global array ('ndim' elements)         */
     uint64_t * count;      /* local sizes in global array ('ndim' elements)                */
-} ADIOS_VARBLOCK;
+};
 
+enum var_centering
+{
+    point = 1,            // unstructured mesh point centering
+    cell = 2              // unstructured mesh cell centering
+};
 
-typedef struct {
+struct _ADIOS_VARMESH {
+    int meshid;
+    enum var_centering centering;
+};
+
+struct _ADIOS_VARINFO {
         int        varid;           /* variable index (0..ADIOS_FILE.nvars-1)                         */
         enum ADIOS_DATATYPES type;  /* type of variable                                               */
         int        ndim;            /* number of dimensions, 0 for scalars                            */
@@ -109,16 +121,20 @@ typedef struct {
         ADIOS_VARBLOCK *blockinfo;  /* Spatial arrangement of written blocks, 
                                        retrieved in separate call: adios_inq_var_blockinfo()       
                                        It is an array of 'sum_nblocks' elements                       */
-} ADIOS_VARINFO;
+        ADIOS_VARMESH *meshinfo;    /* structure in this file,
+                                       retrieved in separate call: adios_inq_var_meshinfo()          */ 
+};
 
-
-typedef struct {
+struct _ADIOS_VARCHUNK {
         int                   varid;    /* variable index (0..ADIOS_FILE.nvars-1)              */
         enum ADIOS_DATATYPES  type;     /* type of variable                                    */
+        // NCSU ALACRITY-ADIOS - Added timestep information into varchunks
+        int                   from_steps; /* the first timestep in the returned data             */
+        int                   nsteps;     /* the number of timesteps in the returned data        */
         ADIOS_SELECTION     * sel;      /* sub-selection of requested selection                */
         void                * data;     /* pointer to data, at next adios_read_check() memory 
                                            will likely be overwritten                          */
-} ADIOS_VARCHUNK;
+};
 
 /* The list of the available read methods */
 enum ADIOS_READ_METHOD {
@@ -167,7 +183,7 @@ const char *adios_errmsg();
  *                  List of parameters is documented for each method 
  *                  separately. 
  *                  
- *  RETURN:       0 if accepted, <0 on error
+ *  RETURN:       0 if accepted, !=0 on error (adios_errno value)
  *  Initialization is required for the staging methods, where init/finalize 
  *  perform the connection/disconnection to the staging server once.
  *  The ADIOS_READ_METHOD_BP does not need to be initialized/finalized.
@@ -252,7 +268,7 @@ ADIOS_FILE * adios_read_open_file (const char * fname,
 /** Close an adios file.
  *  It will free the content of the underlying data structures and the fp pointer itself.
  *  IN:   fp       pointer to an ADIOS_FILE struct
- *  RETURN: 0 OK, !=0 on error (also sets adios_errno)
+ *  RETURN: 0 OK, !=0 on error (adios_errno value)
  */
 int adios_read_close (ADIOS_FILE *fp);
 
@@ -272,7 +288,7 @@ int adios_read_close (ADIOS_FILE *fp);
  *                            for max 'timeout_sec' seconds.
  *                            0.0 means return immediately if step is not available
  *                 <0.0: block forever if necessary
- *  RETURN: 0 OK, !=0 on error (also sets adios_errno)
+ *  RETURN: 0 OK, !=0 on error (adios_errno value)
  *      
  *  Possible errors (adios_errno values):
  *       err_end_of_stream    Stream has ended, no more steps should be expected
@@ -312,13 +328,13 @@ void adios_release_step (ADIOS_FILE *fp);
  *       varname  name of the variable
  *  RETURN:       pointer to and ADIOS_VARINFO struct, NULL on error (sets adios_errno)
  */
-ADIOS_VARINFO * adios_inq_var (ADIOS_FILE *fp, const char * varname);
+ADIOS_VARINFO * adios_inq_var (ADIOS_FILE *fp, const char * varname);  
 
 /** Inquiry a variable by index
  *       varid    index of variable (0..fp->nvars-1)
  *                in fp->vars_namelist of ADIOS_FILE struct
  */
-ADIOS_VARINFO * adios_inq_var_byid (ADIOS_FILE *fp, int varid);
+ADIOS_VARINFO * adios_inq_var_byid (ADIOS_FILE *fp, int varid);   
 
 /** Free memory used by an ADIOS_VARINFO struct */
 void adios_free_varinfo (ADIOS_VARINFO *cp);
@@ -334,7 +350,7 @@ void adios_free_varinfo (ADIOS_VARINFO *cp);
  *       varinfo        result of adios_inq_var() 
  *       per_step_stat  !=0: return statistics also per step
  *       per_block_stat !=0: return statistics also per writer block 
- *  RETURN: 0 OK, !=0 on error (also sets adios_errno)
+ *  RETURN: 0 OK, !=0 on error (adios_errno value)
  */
 int adios_inq_var_stat (ADIOS_FILE *fp, ADIOS_VARINFO * varinfo,
                         int per_step_stat, int per_block_stat);
@@ -349,10 +365,32 @@ int adios_inq_var_stat (ADIOS_FILE *fp, ADIOS_VARINFO * varinfo,
  *
  *  IN:  fp       pointer to an (opened) ADIOS_FILE struct
  *       varinfo  result of adios_inq_var() 
- *  RETURN: 0 OK, !=0 on error (also sets adios_errno)
+ *  RETURN: 0 OK, !=0 on error (adios_errno value)
  */
 int adios_inq_var_blockinfo (ADIOS_FILE *fp, ADIOS_VARINFO * varinfo);
 
+/** Inquiry a mesh by index
+*       meshid   index of mesh (0..fp->nmeshes-1)
+*                in fp->mesh_namelist of ADIOS_FILE struct
+*/
+ADIOS_MESH * adios_inq_mesh_byid (ADIOS_FILE *fp, int meshid);
+
+/** fill in the complete mesh structure if mesh struct is stored in external file  
+    call adios_inq_mesh_byid() first to determine if external file is NULL
+*/
+int adios_complete_meshinfo (ADIOS_FILE *datafile, ADIOS_FILE *meshfile, ADIOS_MESH *meshinfo);
+
+/** Free memory used by an ADIOS_MESH struct */
+void adios_free_meshinfo (ADIOS_MESH *meshinfo);
+
+/** Get the mesh information of the variable about how it is stored in
+ * ADIOS_MESH structure
+ * IN:  fp       pointer to an (opened) ADIOS_FILE struct
+ *      varinfo  result of adios_inq_var() 
+ * RETURN: 0 OK, !=0 on error
+ */
+int adios_inq_var_meshinfo (ADIOS_FILE *fp, ADIOS_VARINFO * varinfo);
+
 /** Schedule reading a variable (slice) from the file.
  *  You need to call adios_perform_reads() to do the reading of
  *  variables.
@@ -375,7 +413,7 @@ int adios_inq_var_blockinfo (ADIOS_FILE *fp, ADIOS_VARINFO * varinfo);
  *                  already available of a variable (in ADIOS own memory)
  *                  and the application has to rearrange the data. The user
  *                  has to process/copy the data before getting new chunks.
- *  RETURN: 0 OK, !=0 on error, sets adios_errno too
+ *  RETURN: 0 OK, !=0 on error (adios_errno value)
  */
 int adios_schedule_read (const ADIOS_FILE * fp,
                          const ADIOS_SELECTION * sel,
@@ -395,6 +433,23 @@ int adios_schedule_read_byid (const ADIOS_FILE * fp,
                               int                     nsteps,
                               void                  * data);
 
+// NCSU ALACRITY-ADIOS: Support for those transforms that can change reading behavior (e.g., level-of-detail)
+int adios_schedule_read_param (const ADIOS_FILE * fp,
+                               const ADIOS_SELECTION * sel,
+                               const char            * varname,
+                               int                     from_steps,
+                               int                     nsteps,
+                               const char            * param,
+                               void                  * data);
+
+int adios_schedule_read_byid_param (const ADIOS_FILE * fp,
+                                    const ADIOS_SELECTION * sel,
+                                    int                     varid,
+                                    int                     from_steps,
+                                    int                     nsteps,
+                                    const char            * param,
+                                    void                  * data);
+
 
 /** Let ADIOS perform the scheduled reads 
  *  IN:  blocking  If non-zero, return only when all reads are completed.
@@ -417,7 +472,7 @@ int adios_perform_reads (const ADIOS_FILE *fp, int blocking);
  *  RETURN:         0: all chunks have been returned previously, 
  *                     no need to call again (chunk is NULL, too)
  *                  1: some chunks are/will be available, call again
- *                  <0 on error, sets adios_errno too
+ *                  <0 on error (adios_errno value)
  */
 int adios_check_reads (const ADIOS_FILE  * fp, 
                        ADIOS_VARCHUNK   ** chunk);
@@ -441,7 +496,7 @@ void adios_free_chunk (ADIOS_VARCHUNK *chunk);
  *  OUT: type     adios type of attribute (see enum ADIOS_DATATYPES in adios_types.h)
  *       size     memory size of value (n+1 for a string of n characters)
  *       data     pointer to the value. You need to cast it afterward according to the type.
- *  RETURN: 0 OK, error: set and return adios_errno
+ *  RETURN: 0 OK, !=0 on error (adios_errno value)
  */
 int adios_get_attr (ADIOS_FILE            * fp,
                     const char            * attrname,
@@ -478,8 +533,7 @@ int adios_type_size(enum ADIOS_DATATYPES type, void *data);
  *  IN:   fp              pointer to an (opened) ADIOS_FILE struct
  *  OUT:  group_namelist  list of strings,
  *                        Note: one should pass a pointer to char** list
- *  RETURN:               number of groups, <0 on error 
- *                        (sets adios_errno too)
+ *  RETURN:               number of groups, <0 on error (adios_errno value)
  *  
  */
 int adios_get_grouplist (ADIOS_FILE  *fp, char ***group_namelist);
@@ -494,8 +548,7 @@ int adios_get_grouplist (ADIOS_FILE  *fp, char ***group_namelist);
  *  IN/OUT:   fp         pointer to an (opened) ADIOS_FILE struct
  *                       nvars, var_namelist, nattrs, and
  *                       attr_namelist will be modified.
- *  RETURN:              0 on success, adios_errno otherwise
- *                        (sets adios_errno too)
+ *  RETURN:   0 OK, !=0 on error (adios_errno value)
  *  Note: a stream does not have groups. Only a file can have
  *  multiple groups (from separate adios_open/adios_close operations)
  */
@@ -514,6 +567,9 @@ double adios_stat_cov (ADIOS_VARINFO * vix, ADIOS_VARINFO * viy, char * characte
  */
 void adios_reset_dimension_order (ADIOS_FILE *fp, int is_fortran);
 
+/** Test function to print basic info about the file to stdout */
+void adios_print_fileinfo (ADIOS_FILE *fp);
+
 #endif  /*__INCLUDED_FROM_FORTRAN_API__*/
 
 #ifdef __cplusplus
