@@ -18,6 +18,7 @@
 #include <pthread.h>
 #include <assert.h>
 #include <errno.h>
+#include <sys/time.h>
 #include "public/adios_types.h"
 #include "public/adios_read.h"
 #include "public/adios_error.h"
@@ -178,7 +179,7 @@ static void _buffer_read (char * buffer, uint64_t * buffer_offset
 
 static int calc_data_size (BP_PROC * p)
 {
-    int i, size = 0;
+    int size = 0;
     read_request * h = p->local_read_request_list;
 
     // count
@@ -207,16 +208,15 @@ static void get_data_addr (const ADIOS_FILE * fp, int varid,
     BP_PROC * p = (BP_PROC *) fp->fh;
     BP_FILE * fh = (BP_FILE *) p->fh;
     struct adios_index_var_struct_v1 * v;
-    int i, j, k, idx, t;
-    int start_time, stop_time, start_idx, stop_idx, f_idx;
-    int ndim, time, has_subfile, file_is_fortran;
-    uint64_t size, * dims = 0;
+    int j, idx, t;
+    int start_idx, stop_idx;
+    int ndim, time, file_is_fortran;
+    uint64_t * dims = 0;
     uint64_t ldims[32], gdims[32], offsets[32];
     uint64_t * start, * count;
     int nsteps, is_global = 0, flag;
 
     file_is_fortran = is_fortran_file (fh);
-    has_subfile = has_subfiles (fh);
     v = bp_find_var_byid (fh, varid);
 
     /* Get dimensions and flip if caller != writer language */
@@ -395,12 +395,10 @@ static void sort_read_requests (BP_PROC * p)
     pvt->split_read_request_list = n;
 }
 
+/*
 static void send_read_data1 (BP_PROC * p)
 {
     bp_proc_pvt_struct * pvt = (bp_proc_pvt_struct *) p->priv;
-    void * data = 0;
-    uint64_t ds;
-    int i, counter = 0;
     read_request * r = p->local_read_request_list;
     rr_pvt_struct * rr_pvt;
 
@@ -419,6 +417,7 @@ static void send_read_data1 (BP_PROC * p)
         r = r->next;
     }
 }
+*/
 
 static void send_read_data (BP_PROC * p)
 {
@@ -485,6 +484,7 @@ e group
     free (offsets);
 }
 
+/*
 static void get_read_data1 (BP_PROC * p)
 {
     bp_proc_pvt_struct * pvt = (bp_proc_pvt_struct *) p->priv;
@@ -504,6 +504,7 @@ static void get_read_data1 (BP_PROC * p)
         }
     }
 }
+*/
 
 /* Receive read data from aggregator */
 static void get_read_data (BP_PROC * p)
@@ -512,7 +513,6 @@ static void get_read_data (BP_PROC * p)
     char * b = 0, * recv_buff = 0;
     int * sizes = 0, * offsets = 0;
     int size = 0;
-    MPI_Status status;
     read_request * r = p->local_read_request_list;
 
     if (pvt->rank == pvt->aggregator_rank)
@@ -554,9 +554,7 @@ static void get_read_data (BP_PROC * p)
 
 static void parse_buffer (BP_PROC * p, char * b, int src)
 {
-    read_request * h = p->local_read_request_list;
-    int i, j, type, count, varid, ndims, size = calc_data_size (p);
-    void * buf;
+    int i, count;
     read_request * r;
     rr_pvt_struct * rr_pvt;
 
@@ -629,24 +627,21 @@ static read_request * split_read_requests (const ADIOS_FILE * fp, read_request *
     BP_PROC * p = (BP_PROC *) fp->fh;
     BP_FILE * fh = (BP_FILE *) p->fh;
     struct adios_index_var_struct_v1 * v;
-    int i, j, k, idx, t, varid, nsteps, time, dummy;
-    int start_time, stop_time, start_idx, stop_idx, f_idx;
-    int ndim, has_subfile, file_is_fortran;
-    uint64_t size, * dims = 0;
+    int i, j, idx, t, varid, nsteps, time, dummy;
+    int start_idx, stop_idx;
+    int ndim, file_is_fortran;
+    uint64_t * dims = 0;
     uint64_t ldims[32], gdims[32], offsets[32];
     uint64_t * start, * count;
     int is_global = 0, flag;
-    ADIOS_SELECTION * sel;
     read_request * h = 0;
     rr_pvt_struct * rr_pvt = (rr_pvt_struct *) r->priv;
     assert (rr_pvt);
 
     varid = r->varid;
-    sel = r->sel;
     start = r->sel->u.bb.start;
     count = r->sel->u.bb.count;
     file_is_fortran = is_fortran_file (fh);
-    has_subfile = has_subfiles (fh);
     v = bp_find_var_byid (fh, varid);
 
     bp_get_and_swap_dimensions (fh, v, file_is_fortran,
@@ -812,8 +807,6 @@ static read_request * split_read_requests (const ADIOS_FILE * fp, read_request *
             else if (hole_break == 0)
             {
                 int isize;
-                uint64_t size_in_dset = 0;
-                uint64_t offset_in_dset = 0;
 
                 isize = offsets[0] + ldims[0];
                 if (start[0] >= offsets[0])
@@ -838,7 +831,6 @@ static read_request * split_read_requests (const ADIOS_FILE * fp, read_request *
                     if (isize < start[0] + count[0])
                     {
                         n->sel->u.bb.count[0] = ldims[0];
-                    //    size_in_dset = ldims[0];
                     }
                     else
                     {
@@ -846,13 +838,11 @@ static read_request * split_read_requests (const ADIOS_FILE * fp, read_request *
                         n->sel->u.bb.count[0] = count[0] + start[0] - offsets[0];
                     }
                     n->sel->u.bb.start[0] = offsets[0];
-                    //offset_in_dset = 0;
                 }
 
             }
             else
             {
-                uint64_t stride_offset = 0;
                 int isize;
                 uint64_t size_in_dset[10];
                 uint64_t offset_in_dset[10];
@@ -957,8 +947,8 @@ static void read_chunk (const ADIOS_FILE * fp,
     MPI_Status status;
     int has_subfile;
 
-struct timeval t0, t1;
-gettimeofday (&t0, NULL);
+    //struct timeval t0, t1;
+    //gettimeofday (&t0, NULL);
 
     has_subfile = has_subfiles (fh);
 
@@ -978,7 +968,7 @@ gettimeofday (&t0, NULL);
 
             new_h->file_index = file_idx;
             new_h->next = 0;
-            if (ch = strrchr (fh->fname, '/'))
+            if ( (ch = strrchr (fh->fname, '/')))
             {
                 name_no_path = malloc (strlen (ch + 1) + 1);
                 strcpy (name_no_path, ch + 1);
@@ -1038,9 +1028,8 @@ gettimeofday (&t0, NULL);
 
     fh->b->offset = 0;
 
-    gettimeofday (&t1, NULL);
-
-//    printf ("read chunk time = %f \n", t1.tv_sec - t0.tv_sec + (double)(t1.tv_usec - t0.tv_usec)/1000000 );
+    //gettimeofday (&t1, NULL);
+    //printf ("read chunk time = %f \n", t1.tv_sec - t0.tv_sec + (double)(t1.tv_usec - t0.tv_usec)/1000000 );
 }
 
 /* 
@@ -1049,22 +1038,19 @@ The read request link list needs to be sorted beforehand
 static void do_read (const ADIOS_FILE * fp)
 {
     BP_PROC * p = (BP_PROC *) fp->fh;
-    BP_FILE * fh = (BP_FILE *) p->fh;
     bp_proc_pvt_struct * pvt = (bp_proc_pvt_struct *) p->priv;
-    void * data = 0;
-    int i, counter = 0;
     int file_idx;
     uint64_t offset, payload_size;
-struct timeval t0;
-struct timeval t1;
-double t2, t3, t4, t5;
+    //struct timeval t0;
+    //struct timeval t1;
+    //double t2, t3, t4, t5;
 
     read_request * s = pvt->split_read_request_list, * f_start = s, * f_end = s;
     read_request * o_start = s, * o_end = s, * o_prev_end = 0, * parent = 0;
 
-    gettimeofday (&t0, NULL);
+    //gettimeofday (&t0, NULL);
 
-    t2 = MPI_Wtime();
+    //t2 = MPI_Wtime();
 
     while (f_start)
     {
@@ -1102,7 +1088,7 @@ double t2, t3, t4, t5;
 //printf ("o_start.offset = %llu\n", o_start->ra->offset);
 //printf ("o_prev_end.offset = %llu\n", o_prev_end->ra->offset);
 
-            t4 = MPI_Wtime ();
+            //t4 = MPI_Wtime ();
             // read a chunk from file into internal buffer
             read_chunk (fp, 
                         ((rr_pvt_struct *) o_start->priv)->file_idx, 
@@ -1111,7 +1097,7 @@ double t2, t3, t4, t5;
                          - ((rr_pvt_struct *) o_start->priv)->offset + payload_size
                        );
 
-            t5 = MPI_Wtime ();
+            //t5 = MPI_Wtime ();
 //    printf ("read chunk = %f \n", t5 - t4);
 
             s = o_start;
@@ -1131,11 +1117,11 @@ double t2, t3, t4, t5;
         f_start = f_end;
     }
 
-    gettimeofday (&t1, NULL);
-    t3 = MPI_Wtime ();
+    //gettimeofday (&t1, NULL);
+    //t3 = MPI_Wtime ();
 
-//    printf ("while time = %f \n", t1.tv_sec - t0.tv_sec + (double)(t1.tv_usec - t0.tv_usec)/1000000 );
-//    printf ("while time = %f \n", t3 - t2);
+    //printf ("while time = %f \n", t1.tv_sec - t0.tv_sec + (double)(t1.tv_usec - t0.tv_usec)/1000000 );
+    //printf ("while time = %f \n", t3 - t2);
 }
 
 /* r - the original read rquest
@@ -1153,8 +1139,8 @@ static void read_buffer (const ADIOS_FILE * fp,
     struct adios_index_var_struct_v1 * v;
     uint64_t * s_start, * s_count;
     uint64_t * start, * count;
-    int i, j, k, idx, t, time, ndim, dummy;
-    int varid, start_idx, stop_idx, has_subfile, file_is_fortran;
+    int i, j, idx, t, time, ndim, dummy;
+    int varid, start_idx, stop_idx, file_is_fortran;
     uint64_t ldims[MAX_DIMS], gdims[MAX_DIMS], offsets[MAX_DIMS];
     uint64_t datasize, dset_stride, var_stride, total_size = 0, items_read;
     int is_global = 0, size_unit, break_dim, idx_check1, idx_check2;
@@ -1169,7 +1155,6 @@ static void read_buffer (const ADIOS_FILE * fp,
     s_start = s->sel->u.bb.start;
     s_count = s->sel->u.bb.count;
     file_is_fortran = is_fortran_file (fh);
-    has_subfile = has_subfiles (fh);
     ndim = r->sel->u.bb.ndim;
     size_unit = bp_get_type_size (v->type,
                                   v->characteristics [0].value
@@ -1410,7 +1395,6 @@ static void read_buffer (const ADIOS_FILE * fp,
             }
             else
             {
-                uint64_t stride_offset = 0;
                 int isize;
                 uint64_t size_in_dset[MAX_DIMS];
                 uint64_t offset_in_dset[MAX_DIMS];
@@ -1530,7 +1514,6 @@ static void broadcast_fh_buffer (ADIOS_FILE * fp)
     BP_FILE * fh = (BP_FILE *) p->fh;
     struct bp_index_pg_struct_v1 * pgs_root = fh->pgs_root, * pg;
     struct adios_index_var_struct_v1 * vars_root = fh->vars_root, * v;
-    struct adios_index_attribute_struct_v1 * attrs_root = fh->attrs_root;
     char * buffer;
     uint64_t buffer_size, buffer_offset = 0;
     int i, j, nsteps;
@@ -1554,9 +1537,9 @@ static void broadcast_fh_buffer (ADIOS_FILE * fp)
         _buffer_write (&buffer, &buffer_size, &buffer_offset,
                        &fh->mfooter.pgs_count, 8); //vars_count 
         _buffer_write (&buffer, &buffer_size, &buffer_offset, 
-                       &fh->mfooter.vars_count, 2); //vars_count 
+                       &fh->mfooter.vars_count, 4); //vars_count 
         _buffer_write (&buffer, &buffer_size, &buffer_offset,
-                       &fh->mfooter.attrs_count, 2); //attrs_count 
+                       &fh->mfooter.attrs_count, 4); //attrs_count 
 
         for (i = 0; i < fh->gvar_h->group_count; i++)
         {
@@ -1566,7 +1549,7 @@ static void broadcast_fh_buffer (ADIOS_FILE * fp)
             _buffer_write (&buffer, &buffer_size, &buffer_offset,
                            fh->gvar_h->namelist[i], len); // namelist
             _buffer_write (&buffer, &buffer_size, &buffer_offset, 
-                           &fh->gvar_h->var_counts_per_group[i], 2); // var_counts_per_group
+                           &fh->gvar_h->var_counts_per_group[i], sizeof(uint32_t)); // var_counts_per_group
         }
 
         _buffer_write (&buffer, &buffer_size, &buffer_offset,
@@ -1574,7 +1557,7 @@ static void broadcast_fh_buffer (ADIOS_FILE * fp)
         for (i = 0; i < fh->gattr_h->group_count; i++)
         {
             _buffer_write (&buffer, &buffer_size, &buffer_offset, 
-                           &fh->gattr_h->attr_counts_per_group[i], 2); // attr_counts_per_group
+                           &fh->gattr_h->attr_counts_per_group[i], sizeof(uint32_t)); // attr_counts_per_group
         }
 
         for (i = 0; i < fh->mfooter.vars_count; i++)
@@ -1626,9 +1609,8 @@ static void broadcast_fh_buffer (ADIOS_FILE * fp)
         vars_root = fh->vars_root;
         while (vars_root)
         {
-uint64_t bo = buffer_offset;
             _buffer_write (&buffer, &buffer_size, &buffer_offset, 
-                           &vars_root->id, 2); // id
+                           &vars_root->id, 4); // id
 
             len = strlen (vars_root->group_name);
             _buffer_write (&buffer, &buffer_size, &buffer_offset, 
@@ -1708,7 +1690,7 @@ fprintf (stderr, "bc %s bo 1 = %llu, bo 2 = %llu, len = %d\n", vars_root->var_na
 
     if (!isAggregator (p))
     {
-        uint16_t len, group_count, var_counts_per_group;
+        uint16_t len; 
 
         buffer_offset = 0;
 
@@ -1722,11 +1704,11 @@ fprintf (stderr, "bc %s bo 1 = %llu, bo 2 = %llu, len = %d\n", vars_root->var_na
 
         _buffer_read (buffer, &buffer_offset, &fh->gvar_h->group_count, 2); //group_count 
         _buffer_read (buffer, &buffer_offset, &fh->mfooter.pgs_count, 8); //pgs_count 
-        _buffer_read (buffer, &buffer_offset, &fh->mfooter.vars_count, 2); //vars_count 
-        _buffer_read (buffer, &buffer_offset, &fh->mfooter.attrs_count, 2); //attrs_count 
+        _buffer_read (buffer, &buffer_offset, &fh->mfooter.vars_count, 4); //vars_count 
+        _buffer_read (buffer, &buffer_offset, &fh->mfooter.attrs_count, 4); //attrs_count 
 
         fh->gvar_h->namelist = (char **) malloc (fh->gvar_h->group_count * sizeof (char *));
-        fh->gvar_h->var_counts_per_group = (uint16_t *) malloc (fh->gvar_h->group_count * 2);
+        fh->gvar_h->var_counts_per_group = (uint32_t *) malloc (fh->gvar_h->group_count * sizeof(uint32_t));
 
         for (i = 0; i < fh->gvar_h->group_count; i++)
         {
@@ -1735,7 +1717,7 @@ fprintf (stderr, "bc %s bo 1 = %llu, bo 2 = %llu, len = %d\n", vars_root->var_na
             _buffer_read (buffer, &buffer_offset, fh->gvar_h->namelist[i], len); // namelist
             fh->gvar_h->namelist[i][len] = '\0';
 
-            _buffer_read (buffer, &buffer_offset, &fh->gvar_h->var_counts_per_group[i], 2); // var_counts_per_group
+            _buffer_read (buffer, &buffer_offset, &fh->gvar_h->var_counts_per_group[i], sizeof(uint32_t)); // var_counts_per_group
         }
 
         fh->gattr_h = (struct BP_GROUP_ATTR *) malloc (sizeof (struct BP_GROUP_ATTR));
@@ -1744,12 +1726,12 @@ fprintf (stderr, "bc %s bo 1 = %llu, bo 2 = %llu, len = %d\n", vars_root->var_na
 
         _buffer_read (buffer, &buffer_offset, &fh->gattr_h->group_count, 2); //group_count 
 
-        fh->gattr_h->attr_counts_per_group = (uint16_t *) malloc (fh->gattr_h->group_count * 2);
+        fh->gattr_h->attr_counts_per_group = (uint32_t *) malloc (fh->gattr_h->group_count * sizeof(uint32_t));
         fh->gattr_h->namelist = fh->gvar_h->namelist;
 
         for (i = 0; i < fh->gattr_h->group_count; i++)
         {
-            _buffer_read (buffer, &buffer_offset, &fh->gattr_h->attr_counts_per_group[i], 2); // attr_counts_per_group
+            _buffer_read (buffer, &buffer_offset, &fh->gattr_h->attr_counts_per_group[i], sizeof(uint32_t)); // attr_counts_per_group
         }
 
         fh->gvar_h->var_namelist = (char **) malloc (fh->mfooter.vars_count * sizeof (char *));
@@ -1809,13 +1791,16 @@ fprintf (stderr, "bc %s bo 1 = %llu, bo 2 = %llu, len = %d\n", vars_root->var_na
             }
         }
 
+        fh->vars_table = (struct adios_index_var_struct_v1 **) malloc (fh->mfooter.vars_count * 8);
+
         vars_root = 0;
         for (i = 0; i < fh->mfooter.vars_count; i++)
         {
             v = (struct adios_index_var_struct_v1 *) malloc (sizeof (struct adios_index_var_struct_v1));
             assert (v);
-uint64_t bo = buffer_offset;
-            _buffer_read (buffer, &buffer_offset, &v->id, 2);
+            fh->vars_table[i] = v;
+
+            _buffer_read (buffer, &buffer_offset, &v->id, 4);
 
             _buffer_read (buffer, &buffer_offset, &len, 2);
             v->group_name = (char *) malloc (len + 1);
@@ -2017,7 +2002,6 @@ ADIOS_FILE * adios_read_bp_staged_open (const char * fname, MPI_Comm comm, enum
 static void free_proc_struct (BP_PROC * p)
 {
     bp_proc_pvt_struct * pvt = (bp_proc_pvt_struct *) p->priv;
-    read_request * h = p->local_read_request_list, * n;
 
     /* We need to free split_read_request_list cause
      * people might do two back-to-back sets of
@@ -2147,7 +2131,7 @@ static void init_read (BP_PROC * p)
 
 ADIOS_FILE * adios_read_bp_staged_open_file (const char * fname, MPI_Comm comm)
 {
-    int i, rank;
+    int rank;
     struct BP_PROC * p;
     BP_FILE * fh;
     ADIOS_FILE * fp;
@@ -2167,6 +2151,7 @@ ADIOS_FILE * adios_read_bp_staged_open_file (const char * fname, MPI_Comm comm)
     fh->pgs_root = 0;
     fh->vars_root = 0;
     fh->attrs_root = 0;
+    fh->vars_table = 0;
     fh->b = malloc (sizeof (struct adios_bp_buffer_struct_v1));
     assert (fh->b);
     adios_buffer_struct_init (fh->b);
@@ -2190,7 +2175,7 @@ ADIOS_FILE * adios_read_bp_staged_open_file (const char * fname, MPI_Comm comm)
     {
         if (bp_open (fname, pvt->new_comm2, fh) < 0)
         {
-            adios_error (err_file_open_error, "File open failed: %s", fname);
+            adios_error (err_file_open_error, "File open failed: %s\n", fname);
             return 0;
         }
     }
@@ -2216,7 +2201,7 @@ ently.
 
     fp->path = strdup (fh->fname);
     fp->endianness = bp_get_endianness (fh->mfooter.change_endianness);
-    fp->version = fh->mfooter.version;
+    fp->version = fh->mfooter.version & ADIOS_VERSION_NUM_MASK;
     fp->file_size = fh->mfooter.file_size;
 
     return fp;
@@ -2325,11 +2310,9 @@ int adios_read_bp_staged_schedule_read_byid (const ADIOS_FILE * fp,
 int adios_read_bp_staged_perform_reads (const ADIOS_FILE *fp, int blocking)
 {
     BP_PROC * p = (BP_PROC *) fp->fh;
-    BP_FILE * fh = (BP_FILE *) p->fh;
     bp_proc_pvt_struct * pvt = (bp_proc_pvt_struct *) p->priv;
     read_request * r, * h;
     int i, count, varid, ndims, total_size, size;
-    rr_pvt_struct * rrs;
     void * buf;
 
     // First populate the read request private struct for
@@ -2484,18 +2467,18 @@ void adios_read_bp_staged_reset_dimension_order (const ADIOS_FILE *fp, int is_fo
     adios_read_bp_reset_dimension_order (fp, is_fortran);
 }
 
-void adios_read_bp_staged_get_groupinfo (const ADIOS_FILE *fp, int *ngroups, char ***group_namelist, int **nvars_per_group, int **nattrs_per_group)
+void adios_read_bp_staged_get_groupinfo (const ADIOS_FILE *fp, int *ngroups, char ***group_namelist, uint32_t **nvars_per_group, uint32_t **nattrs_per_group)
 {
     BP_PROC * p;
     BP_FILE * fh;
-    int i, j, k, offset;
+    int i, j, offset;
 
     p = (BP_PROC *) fp->fh;
     fh = (BP_FILE *) p->fh;
 
     * ngroups = fh->gvar_h->group_count;
 
-    alloc_namelist (group_namelist, fh->gvar_h->group_count);
+    *group_namelist = (char **) malloc (sizeof (char *) * fh->gvar_h->group_count);
     for (i = 0; i < fh->gvar_h->group_count; i++)
     {
         (*group_namelist)[i] = malloc (strlen (fh->gvar_h->namelist[i]) + 1);
@@ -2505,7 +2488,7 @@ void adios_read_bp_staged_get_groupinfo (const ADIOS_FILE *fp, int *ngroups, cha
 namelist[i]) + 1);
     }
 
-    * nvars_per_group = (int *) malloc (fh->gvar_h->group_count * sizeof (int));
+    * nvars_per_group = (uint32_t *) malloc (fh->gvar_h->group_count * sizeof (uint32_t));
     assert (* nvars_per_group);
 
     for (i = 0; i < fh->gvar_h->group_count; i++)
@@ -2513,7 +2496,7 @@ namelist[i]) + 1);
         (* nvars_per_group)[i] = fh->gvar_h->var_counts_per_group[i];
     }
 
-    * nattrs_per_group = (int *) malloc (fh->gattr_h->group_count * sizeof (int));
+    * nattrs_per_group = (uint32_t *) malloc (fh->gattr_h->group_count * sizeof (uint32_t));
     assert (* nattrs_per_group);
 
     for (i = 0; i < fh->gvar_h->group_count; i++)
@@ -2547,7 +2530,7 @@ int adios_read_bp_staged_is_var_timed (const ADIOS_FILE *fp, int varid)
     BP_FILE * fh;
     struct adios_index_var_struct_v1 * v;
     struct adios_index_characteristic_struct_v1 ch;
-    int retval = 0, ndim, k, dummy, file_is_fortran;
+    int retval = 0, ndim, k;
     uint64_t gdims[32];
 
     p = (BP_PROC *) fp->fh;
