@@ -11,10 +11,10 @@
 #include <math.h>
 #include <string.h>
 #if defined(__APPLE__)
-#	include <sys/param.h>
-#	include <sys/mount.h>
+#    include <sys/param.h>
+#    include <sys/mount.h>
 #else
-#	include <sys/vfs.h>
+#    include <sys/vfs.h>
 #endif
 #include <sys/ioctl.h>
 #include <assert.h>
@@ -51,9 +51,7 @@ struct adios_MPI_data_struct
 
     struct adios_bp_buffer_struct_v1 b;
 
-    struct adios_index_process_group_struct_v1 * old_pg_root;
-    struct adios_index_var_struct_v1 * old_vars_root;
-    struct adios_index_attribute_struct_v1 * old_attrs_root;
+    struct adios_index_struct_v1 * index;
 
     uint64_t vars_start;
     uint64_t vars_header_size;
@@ -292,9 +290,7 @@ void adios_mpi_init (const PairStruct * parameters
     md->rank = 0;
     md->size = 0;
     md->group_comm = method->init_comm; // unused here, adios_open will set the current comm
-    md->old_pg_root = 0;
-    md->old_vars_root = 0;
-    md->old_attrs_root = 0;
+    md->index = adios_alloc_index_v1(1); // with hashtables
     md->vars_start = 0;
     md->vars_header_size = 0;
     md->storage_targets = 0;
@@ -344,9 +340,10 @@ int adios_mpi_open (struct adios_file_struct * fd
 static
 void build_offsets (struct adios_bp_buffer_struct_v1 * b
                    ,MPI_Offset * offsets, int size, char * group_name
-                   ,struct adios_index_process_group_struct_v1 * pg_root
+                   ,struct adios_index_struct_v1 * index
                    )
 {
+    struct adios_index_process_group_struct_v1 * pg_root = index->pg_root;
     while (pg_root)
     {
         if (!strcasecmp (pg_root->group_name, group_name))
@@ -546,7 +543,6 @@ enum ADIOS_FLAG adios_mpi_should_buffer (struct adios_file_struct * fd
                                         ,struct adios_method_struct * method
                                         )
 {
-    int i;
     struct adios_MPI_data_struct * md = (struct adios_MPI_data_struct *)
                                                       method->method_data;
     char * name;
@@ -620,9 +616,7 @@ enum ADIOS_FLAG adios_mpi_should_buffer (struct adios_file_struct * fd
                 MPI_File_read (md->fh, md->b.buff, md->b.pg_size, MPI_BYTE
                               ,&md->status
                               );
-                adios_parse_process_group_index_v1 (&md->b
-                                                   ,&md->old_pg_root
-                                                   );
+                adios_parse_process_group_index_v1 (&md->b, &md->index->pg_root);
 
 #if 1
                 adios_init_buffer_read_vars_index (&md->b);
@@ -632,7 +626,9 @@ enum ADIOS_FLAG adios_mpi_should_buffer (struct adios_file_struct * fd
                 MPI_File_read (md->fh, md->b.buff, md->b.vars_size, MPI_BYTE
                               ,&md->status
                               );
-                adios_parse_vars_index_v1 (&md->b, &md->old_vars_root);
+                adios_parse_vars_index_v1 (&md->b, &md->index->vars_root, 
+                                           md->index->hashtbl_vars,
+                                           &md->index->vars_tail);
 
                 adios_init_buffer_read_attributes_index (&md->b);
                 MPI_File_seek (md->fh, md->b.attrs_index_offset
@@ -641,7 +637,7 @@ enum ADIOS_FLAG adios_mpi_should_buffer (struct adios_file_struct * fd
                 MPI_File_read (md->fh, md->b.buff, md->b.attrs_size, MPI_BYTE
                               ,&md->status
                               );
-                adios_parse_attributes_index_v1 (&md->b, &md->old_attrs_root);
+                adios_parse_attributes_index_v1 (&md->b, &md->index->attrs_root);
 #endif
 
                 fd->base_offset = md->b.end_of_pgs;
@@ -660,7 +656,7 @@ enum ADIOS_FLAG adios_mpi_should_buffer (struct adios_file_struct * fd
 
                     // go through the pg index to build the offsets array
                     build_offsets (&md->b, offsets, md->size
-                                  ,fd->group->name, md->old_pg_root
+                                  ,fd->group->name, md->index
                                   );
                     MPI_Scatter (offsets, 3, MPI_LONG_LONG
                                 ,MPI_IN_PLACE, 3, MPI_LONG_LONG
@@ -882,14 +878,12 @@ enum ADIOS_FLAG adios_mpi_should_buffer (struct adios_file_struct * fd
                                   ,&md->status
                                   );
 
-                    adios_parse_process_group_index_v1 (&md->b
-                                                       ,&md->old_pg_root
-                                                       );
+                    adios_parse_process_group_index_v1 (&md->b, &md->index->pg_root);
 
                     // find the largest time index so we can append properly
                     struct adios_index_process_group_struct_v1 * p;
                     uint32_t max_time_index = 0;
-                    p = md->old_pg_root;
+                    p = md->index->pg_root;
                     while (p)
                     {
                         if (p->time_index > max_time_index)
@@ -908,7 +902,9 @@ enum ADIOS_FLAG adios_mpi_should_buffer (struct adios_file_struct * fd
                     MPI_File_read (md->fh, md->b.buff, md->b.vars_size, MPI_BYTE
                                   ,&md->status
                                   );
-                    adios_parse_vars_index_v1 (&md->b, &md->old_vars_root);
+                    adios_parse_vars_index_v1 (&md->b, &md->index->vars_root, 
+                                               md->index->hashtbl_vars,
+                                               &md->index->vars_tail);
 
                     adios_init_buffer_read_attributes_index (&md->b);
                     MPI_File_seek (md->fh, md->b.attrs_index_offset
@@ -917,9 +913,7 @@ enum ADIOS_FLAG adios_mpi_should_buffer (struct adios_file_struct * fd
                     MPI_File_read (md->fh, md->b.buff, md->b.attrs_size
                                   ,MPI_BYTE, &md->status
                                   );
-                    adios_parse_attributes_index_v1 (&md->b
-                                                    ,&md->old_attrs_root
-                                                    );
+                    adios_parse_attributes_index_v1 (&md->b, &md->index->attrs_root);
 
                     fd->base_offset = md->b.end_of_pgs;
                     fd->pg_start_in_file = fd->base_offset;
@@ -1037,7 +1031,7 @@ enum ADIOS_FLAG adios_mpi_should_buffer (struct adios_file_struct * fd
             char * buf_ptr = fd->buffer;
             while (total_written < fd->bytes_written)
             {
-                write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
                 err = MPI_File_write (md->fh, buf_ptr, write_len, MPI_BYTE, &md->status);
                 MPI_Get_count(&md->status, MPI_BYTE, &count);
                 if (count != write_len)
@@ -1070,7 +1064,7 @@ enum ADIOS_FLAG adios_mpi_should_buffer (struct adios_file_struct * fd
         MPI_Get_count (&md->status, MPI_BYTE, &count);
         if (count != fd->bytes_written)
         {
-            log_warn ("a:MPI method tried to write %llu, only wrote %llu\n",
+            log_warn ("a:MPI method tried to write %llu, only wrote %d\n",
                       fd->bytes_written, count);
         }
         fd->base_offset += count;
@@ -1140,7 +1134,7 @@ void adios_mpi_write (struct adios_file_struct * fd
             char * buf_ptr = fd->buffer;
             while (total_written < fd->bytes_written)
             {
-                write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
                 err = MPI_File_write (md->fh, buf_ptr, write_len, MPI_BYTE, &md->status);
                 MPI_Get_count(&md->status, MPI_BYTE, &count);
                 if (count != write_len)
@@ -1181,7 +1175,7 @@ void adios_mpi_write (struct adios_file_struct * fd
 
         // write payload
         // adios_write_var_payload_v1 (fd, v);
-        uint64_t var_size = adios_get_var_size (v, fd->group, v->data);
+        uint64_t var_size = adios_get_var_size (v, v->data);
 
         if (fd->base_offset + var_size > fd->pg_start_in_file + fd->write_size_bytes)
             adios_error (err_out_of_bound, 
@@ -1202,7 +1196,7 @@ void adios_mpi_write (struct adios_file_struct * fd
             char * buf_ptr = v->data;
             while (total_written < var_size)
             {
-                write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
                 err = MPI_File_write (md->fh, buf_ptr, write_len, MPI_BYTE, &md->status);
                 MPI_Get_count(&md->status, MPI_BYTE, &count);
                 if (count != write_len)
@@ -1333,15 +1327,11 @@ static void adios_mpi_do_read (struct adios_file_struct * fd
                                                       method->method_data;
     struct adios_var_struct * v = fd->group->vars;
 
-    struct adios_parse_buffer_struct data;
-
-    data.vars = v;
-    data.buffer = 0;
-    data.buffer_len = 0;
-
-    switch (md->b.version & ADIOS_VERSION_NUM_MASK)
+    uint32_t version = md->b.version & ADIOS_VERSION_NUM_MASK;
+    switch (version)
     {
         case 1:
+        case 2:
         {
             // the three section headers
             struct adios_process_group_header_struct_v1 pg_header;
@@ -1424,7 +1414,7 @@ static void adios_mpi_do_read (struct adios_file_struct * fd
         default:
             adios_error (err_invalid_file_version, 
                          "MPI method read: ADIOS file version unknown: %u\n",
-                         md->b.version);
+                         version);
             return;
     }
 
@@ -1497,7 +1487,7 @@ void adios_mpi_close (struct adios_file_struct * fd
                     char * buf_ptr = fd->buffer;
                     while (total_written < md->vars_header_size)
                     {
-                        write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                        write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
                         err = MPI_File_write (md->fh, buf_ptr, write_len, MPI_BYTE, &md->status);
                         MPI_Get_count(&md->status, MPI_BYTE, &count);
                         if (count != write_len)
@@ -1570,7 +1560,7 @@ void adios_mpi_close (struct adios_file_struct * fd
                             char * buf_ptr = fd->buffer;
                             while (total_written < fd->bytes_written)
                             {
-                                write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                                write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
                                 err = MPI_File_write (md->fh, buf_ptr, write_len, MPI_BYTE, &md->status);
                                 MPI_Get_count(&md->status, MPI_BYTE, &count);
                                 if (count != write_len)
@@ -1632,7 +1622,7 @@ void adios_mpi_close (struct adios_file_struct * fd
                     char * buf_ptr = fd->buffer;
                     while (total_written < md->vars_header_size)
                     {
-                        write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                        write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
                         err = MPI_File_write (md->fh, buf_ptr, write_len, MPI_BYTE, &md->status);
                         MPI_Get_count(&md->status, MPI_BYTE, &count);
                         if (count != write_len)
@@ -1674,9 +1664,7 @@ void adios_mpi_close (struct adios_file_struct * fd
             gettimeofday (&timing.t12, NULL);
 #endif
             // build index appending to any existing index
-            adios_build_index_v1 (fd, &md->old_pg_root, &md->old_vars_root
-                                 ,&md->old_attrs_root
-                                 );
+            adios_build_index_v1 (fd, md->index);
             // if collective, gather the indexes from the rest and call
             if (md->group_comm != MPI_COMM_NULL)
             {
@@ -1720,19 +1708,15 @@ void adios_mpi_close (struct adios_file_struct * fd
                         adios_parse_process_group_index_v1 (&md->b
                                                            ,&new_pg_root
                                                            );
-                        adios_parse_vars_index_v1 (&md->b, &new_vars_root);
+                        adios_parse_vars_index_v1 (&md->b, &new_vars_root, NULL, NULL);
                         // do not merge attributes from other processes from 1.4
                         /*
                         adios_parse_attributes_index_v1 (&md->b
                                                         ,&new_attrs_root
                                                         );
                         */
-                        adios_merge_index_v1 (&md->old_pg_root
-                                             ,&md->old_vars_root
-                                             ,&md->old_attrs_root
-                                             ,new_pg_root, new_vars_root
-                                             ,new_attrs_root
-                                             );
+                        adios_merge_index_v1 (md->index, new_pg_root, 
+                                              new_vars_root, new_attrs_root);
                         new_pg_root = 0;
                         new_vars_root = 0;
                         new_attrs_root = 0;
@@ -1748,15 +1732,13 @@ void adios_mpi_close (struct adios_file_struct * fd
                 else
                 {
                     adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                         ,0, md->old_pg_root
-                                         ,md->old_vars_root
-                                         ,md->old_attrs_root
-                                         );
-                    uint32_t tmp_buffer_size = (uint32_t) buffer_size;
+                                         ,0, md->index);
+
+                    uint32_t tmp_buffer_size = (uint32_t) buffer_offset;
                     MPI_Gather (&tmp_buffer_size, 1, MPI_INT, 0, 0, MPI_INT
                                ,0, md->group_comm
                                );
-                    MPI_Gatherv (buffer, buffer_size, MPI_BYTE
+                    MPI_Gatherv (buffer, buffer_offset, MPI_BYTE
                                 ,0, 0, 0, MPI_BYTE
                                 ,0, md->group_comm
                                 );
@@ -1769,12 +1751,12 @@ void adios_mpi_close (struct adios_file_struct * fd
             if (fd->shared_buffer == adios_flag_yes)
             {
                 // if we need to write > 2 GB, need to do it in parts
-                // since count is limited to INT32_MAX (signed 32-bit max).
+                // since count is limited to MAX_MPIWRITE_SIZE (signed 32-bit max).
                 uint64_t bytes_written = 0;
                 int32_t to_write = 0;
-                if (fd->bytes_written > INT32_MAX)
+                if (fd->bytes_written > MAX_MPIWRITE_SIZE)
                 {
-                    to_write = INT32_MAX;
+                    to_write = MAX_MPIWRITE_SIZE;
                 }
                 else
                 {
@@ -1818,9 +1800,9 @@ void adios_mpi_close (struct adios_file_struct * fd
                     bytes_written += to_write;
                     if (fd->bytes_written > bytes_written)
                     {
-                        if (fd->bytes_written - bytes_written > INT32_MAX)
+                        if (fd->bytes_written - bytes_written > MAX_MPIWRITE_SIZE)
                         {
-                            to_write = INT32_MAX;
+                            to_write = MAX_MPIWRITE_SIZE;
                         }
                         else
                         {
@@ -1833,10 +1815,7 @@ void adios_mpi_close (struct adios_file_struct * fd
             if (md->rank == 0)
             {
                 adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                     ,index_start, md->old_pg_root
-                                     ,md->old_vars_root
-                                     ,md->old_attrs_root
-                                     );
+                                     ,index_start, md->index);
                 adios_write_version_v1 (&buffer, &buffer_size, &buffer_offset);
 
                 MPI_File_seek (md->fh, md->b.pg_index_offset, MPI_SEEK_SET);
@@ -1853,7 +1832,7 @@ void adios_mpi_close (struct adios_file_struct * fd
                     char * buf_ptr = buffer;
                     while (total_written < buffer_offset)
                     {
-                        write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                        write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
 #if COLLECT_METRICS
 struct timeval a, b;
 gettimeofday (&a, NULL);
@@ -1903,16 +1882,6 @@ timeval_subtract (&timing.t8, &b, &a);
                 buffer_offset = 0;
             }
 
-            adios_clear_index_v1 (new_pg_root, new_vars_root, new_attrs_root);
-            adios_clear_index_v1 (md->old_pg_root, md->old_vars_root
-                                 ,md->old_attrs_root
-                                 );
-            new_pg_root = 0;
-            new_vars_root = 0;
-            new_attrs_root = 0;
-            md->old_pg_root = 0;
-            md->old_vars_root = 0;
-            md->old_attrs_root = 0;
 #if COLLECT_METRICS
             gettimeofday (&timing.t11, NULL);
 #endif
@@ -1953,7 +1922,7 @@ timeval_subtract (&timing.t8, &b, &a);
                     char * buf_ptr = fd->buffer;
                     while (total_written < md->vars_header_size)
                     {
-                        write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                        write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
                         err = MPI_File_write (md->fh, buf_ptr, write_len, MPI_BYTE, &md->status);
                         MPI_Get_count(&md->status, MPI_BYTE, &count);
                         if (count != write_len)
@@ -2026,7 +1995,7 @@ timeval_subtract (&timing.t8, &b, &a);
                             char * buf_ptr = fd->buffer;
                             while (total_written < fd->bytes_written)
                             {
-                                write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                                write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
                                 err = MPI_File_write (md->fh, buf_ptr, write_len, MPI_BYTE, &md->status);
                                 MPI_Get_count(&md->status, MPI_BYTE, &count);
                                 if (count != write_len)
@@ -2088,7 +2057,7 @@ timeval_subtract (&timing.t8, &b, &a);
                     char * buf_ptr = fd->buffer;
                     while (total_written < md->vars_header_size)
                     {
-                        write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                        write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
                         err = MPI_File_write (md->fh, buf_ptr, write_len, MPI_BYTE, &md->status);
                         MPI_Get_count(&md->status, MPI_BYTE, &count);
                         if (count != write_len)
@@ -2126,9 +2095,7 @@ timeval_subtract (&timing.t8, &b, &a);
             }
 
             // build index appending to any existing index
-            adios_build_index_v1 (fd, &md->old_pg_root, &md->old_vars_root
-                                 ,&md->old_attrs_root
-                                 );
+            adios_build_index_v1 (fd, md->index);
             // if collective, gather the indexes from the rest and call
             if (md->group_comm != MPI_COMM_NULL)
             {
@@ -2172,19 +2139,15 @@ timeval_subtract (&timing.t8, &b, &a);
                         adios_parse_process_group_index_v1 (&md->b
                                                            ,&new_pg_root
                                                            );
-                        adios_parse_vars_index_v1 (&md->b, &new_vars_root);
+                        adios_parse_vars_index_v1 (&md->b, &new_vars_root, NULL, NULL);
                         // do not merge attributes from other processes from 1.4
                         /*
                         adios_parse_attributes_index_v1 (&md->b
                                                         ,&new_attrs_root
                                                         );
-                         */
-                        adios_merge_index_v1 (&md->old_pg_root
-                                             ,&md->old_vars_root
-                                             ,&md->old_attrs_root
-                                             ,new_pg_root, new_vars_root
-                                             ,new_attrs_root
-                                             );
+                        */
+                        adios_merge_index_v1 (md->index, new_pg_root, 
+                                              new_vars_root, new_attrs_root);
                         new_pg_root = 0;
                         new_vars_root = 0;
                         new_attrs_root = 0;
@@ -2200,17 +2163,14 @@ timeval_subtract (&timing.t8, &b, &a);
                 else
                 {
                     adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                         ,0, md->old_pg_root
-                                         ,md->old_vars_root
-                                         ,md->old_attrs_root
-                                         );
+                                         ,0, md->index);
 
-                    int _buffer_size = buffer_size;
+                    int _buffer_size = buffer_offset;
 
                     MPI_Gather (&_buffer_size, 1, MPI_INT, 0, 0, MPI_INT
                                ,0, md->group_comm
                                );
-                    MPI_Gatherv (buffer, buffer_size, MPI_BYTE
+                    MPI_Gatherv (buffer, buffer_offset, MPI_BYTE
                                 ,0, 0, 0, MPI_BYTE
                                 ,0, md->group_comm
                                 );
@@ -2245,7 +2205,7 @@ timeval_subtract (&timing.t8, &b, &a);
                     char * buf_ptr = fd->buffer;
                     while (total_written < fd->bytes_written)
                     {
-                        write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                        write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
                         err = MPI_File_write (md->fh, buf_ptr, write_len, MPI_BYTE, &md->status);
                         MPI_Get_count(&md->status, MPI_BYTE, &count);
                         if (count != write_len)
@@ -2275,10 +2235,7 @@ timeval_subtract (&timing.t8, &b, &a);
             if (md->rank == 0)
             {
                 adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                     ,index_start, md->old_pg_root
-                                     ,md->old_vars_root
-                                     ,md->old_attrs_root
-                                     );
+                                     ,index_start, md->index);
                 adios_write_version_v1 (&buffer, &buffer_size, &buffer_offset);
 
                 MPI_File_seek (md->fh, md->b.pg_index_offset, MPI_SEEK_SET);
@@ -2295,7 +2252,7 @@ timeval_subtract (&timing.t8, &b, &a);
                     char * buf_ptr = buffer;
                     while (total_written < buffer_offset)
                     {
-                        write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+                        write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
                         err = MPI_File_write (md->fh, buf_ptr, write_len, MPI_BYTE, &md->status);
                         MPI_Get_count(&md->status, MPI_BYTE, &count);
                         if (count != write_len)
@@ -2327,17 +2284,6 @@ timeval_subtract (&timing.t8, &b, &a);
 
             free (buffer);
 
-            adios_clear_index_v1 (new_pg_root, new_vars_root, new_attrs_root);
-            adios_clear_index_v1 (md->old_pg_root, md->old_vars_root
-                                 ,md->old_attrs_root
-                                 );
-            new_pg_root = 0;
-            new_vars_root = 0;
-            new_attrs_root = 0;
-            md->old_pg_root = 0;
-            md->old_vars_root = 0;
-            md->old_attrs_root = 0;
-
             break;
         }
 
@@ -2374,12 +2320,7 @@ timeval_subtract (&timing.t8, &b, &a);
     memset (&md->status, 0, sizeof (MPI_Status));
     md->group_comm = MPI_COMM_NULL;
 
-    adios_clear_index_v1 (md->old_pg_root, md->old_vars_root
-                         ,md->old_attrs_root
-                         );
-    md->old_pg_root = 0;
-    md->old_vars_root = 0;
-    md->old_attrs_root = 0;
+    adios_clear_index_v1 (md->index);
 }
 
 void adios_mpi_finalize (int mype, struct adios_method_struct * method)
@@ -2391,6 +2332,7 @@ void adios_mpi_finalize (int mype, struct adios_method_struct * method)
         adios_mpi_initialized = 0;
         MPI_Info_free (&md->info);
     }
+    adios_free_index_v1 (md->index);
 }
 
 void adios_mpi_end_iteration (struct adios_method_struct * method)
