@@ -10,6 +10,8 @@
 #include <math.h>
 #include <string.h>
 #include <errno.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 // xml parser
 #include <mxml.h>
@@ -48,9 +50,7 @@ struct adios_MPI_data_struct
 
     struct adios_bp_buffer_struct_v1 b;
 
-    struct adios_index_process_group_struct_v1 * old_pg_root;
-    struct adios_index_var_struct_v1 * old_vars_root;
-    struct adios_index_attribute_struct_v1 * old_attrs_root;
+    struct adios_index_struct_v1 * index;
 
     uint64_t vars_start;
     uint64_t vars_header_size;
@@ -165,10 +165,10 @@ void print_metrics (struct adios_MPI_data_struct * md, int iteration)
 #endif
 
 #if defined(__APPLE__)
-#	include <sys/param.h>
-#	include <sys/mount.h>
+#    include <sys/param.h>
+#    include <sys/mount.h>
 #else
-#	include <sys/statfs.h>
+#    include <sys/statfs.h>
 #endif
 
 // this should be determined at configure time
@@ -224,20 +224,16 @@ static void trim_spaces (char * str)
 static void
 adios_mpi_lustre_set_striping_unit(char *filename, char *parameters, struct adios_MPI_data_struct * md)
 {
-    MPI_File fh = md->fh;
-    int nproc = md->size;
-    struct statfs fsbuf;
-    int err = 0, flag;
+    int err = 0;
 //    uint64_t striping_unit = 0;
-    uint64_t block_unit = 0;
     uint16_t striping_count = 0;
     uint16_t stripe_offset = -1;
-    char     value[64], *temp_string, *p_count,*p_size;
-    MPI_Info info_used;
+    char     *temp_string, *p_count,*p_size;
 
-    int fd, old_mask, perm, num_ost, rc;
-    struct lov_user_md lum;
-    struct obd_uuid uuids[1024], * uuidp;
+    int fd, old_mask, perm;
+#ifdef HAVE_LUSTRE
+    int num_ost;
+#endif
 
     old_mask = umask(022);
     umask(old_mask);
@@ -249,6 +245,8 @@ adios_mpi_lustre_set_striping_unit(char *filename, char *parameters, struct adio
 
 #ifdef HAVE_LUSTRE
     // To get the number of ost's in the system
+    struct obd_uuid uuids[1024];
+    int rc;
     num_ost = 1024;
     rc = llapi_lov_get_uuids(fd, uuids, &num_ost);
     if (rc != 0)
@@ -258,14 +256,14 @@ adios_mpi_lustre_set_striping_unit(char *filename, char *parameters, struct adio
                 );
     }
 #else
-    num_ost = 0;
+    //num_ost = 0;
 #endif
 
     temp_string = (char *) malloc (strlen (parameters) + 1);
     strcpy (temp_string, parameters);
     trim_spaces (temp_string);
 
-    if (p_count = strstr (temp_string, "stripe_count"))
+    if ( (p_count = strstr (temp_string, "stripe_count")) )
     {
         char * p = strchr (p_count, '=');
         char * q = strtok (p, ",");
@@ -277,6 +275,7 @@ adios_mpi_lustre_set_striping_unit(char *filename, char *parameters, struct adio
     else
     {
 #ifdef HAVE_LUSTRE
+        int nproc = md->size;
         striping_count = (nproc > num_ost ? -1 : nproc);
 #else
         striping_count = 4;
@@ -286,7 +285,7 @@ adios_mpi_lustre_set_striping_unit(char *filename, char *parameters, struct adio
     strcpy (temp_string, parameters);
     trim_spaces (temp_string);
 
-    if (p_size = strstr (temp_string, "stripe_size"))
+    if ( (p_size = strstr (temp_string, "stripe_size")))
     {
         char * p = strchr (p_size, '=');
         char * q = strtok (p, ",");
@@ -304,7 +303,7 @@ adios_mpi_lustre_set_striping_unit(char *filename, char *parameters, struct adio
     strcpy (temp_string, parameters);
     trim_spaces (temp_string);
 
-    if (p_size = strstr (temp_string, "stripe_offset"))
+    if ( (p_size = strstr (temp_string, "stripe_offset")) )
     {
         char * p = strchr (p_size, '=');
         char * q = strtok (p, ",");
@@ -319,23 +318,6 @@ adios_mpi_lustre_set_striping_unit(char *filename, char *parameters, struct adio
         stripe_offset = -1;
     }
 
-    strcpy (temp_string, parameters);
-    trim_spaces (temp_string);
-
-    if (p_size = strstr (temp_string, "block_size"))
-    {
-        char * p = strchr (p_size, '=');
-        char * q = strtok (p, ",");
-        if (!q)
-            block_unit = atoi(q + 1);
-        else
-            block_unit = atoi(p + 1);
-    }
-    else
-    {
-        // set block_unit to 0 to make one large write
-        block_unit = 0;
-    }
 
     free (temp_string);
 
@@ -363,13 +345,13 @@ adios_mpi_lustre_set_striping_unit(char *filename, char *parameters, struct adio
 static void
 adios_mpi_lustre_set_block_unit(uint64_t *block_unit, char *parameters)
 {
-    char *temp_string, *p_count,*p_size;
+    char *temp_string, *p_size;
 
     temp_string = (char *) malloc (strlen (parameters) + 1);
     strcpy (temp_string, parameters);
     trim_spaces (temp_string);
 
-    if (p_size = strstr (temp_string, "block_size"))
+    if ( (p_size = strstr (temp_string, "block_size")) )
     {
         char * p = strchr (p_size, '=');
         char * q = strtok (p, ",");
@@ -491,7 +473,7 @@ printf("adios_mpi_lustre_striping_unit_write offset=%12lld len=%12d\n",offset,wr
         char * buf_ptr = buf;
         while (total_written < len)
         {
-            write_len = (to_write > INT32_MAX) ? INT32_MAX : to_write;
+            write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
             MPI_File_write (fh, buf_ptr, write_len, MPI_BYTE, &status);
             MPI_Get_count(&status, MPI_BYTE, &count);
             if (count != write_len)
@@ -527,9 +509,7 @@ void adios_mpi_lustre_init (const PairStruct * parameters
     md->rank = 0;
     md->size = 0;
     md->group_comm = method->init_comm;//unused, adios_open sets current comm
-    md->old_pg_root = 0;
-    md->old_vars_root = 0;
-    md->old_attrs_root = 0;
+    md->index = adios_alloc_index_v1(1); // with hashtables
     md->vars_start = 0;
     md->vars_header_size = 0;
     md->striping_unit = 0;
@@ -591,9 +571,10 @@ int adios_mpi_lustre_open (struct adios_file_struct * fd
 static
 void build_offsets (struct adios_bp_buffer_struct_v1 * b
                    ,MPI_Offset * offsets, uint64_t size, char * group_name
-                   ,struct adios_index_process_group_struct_v1 * pg_root
+                   ,struct adios_index_struct_v1 * index
                    )
 {
+    struct adios_index_process_group_struct_v1 * pg_root = index->pg_root;
     while (pg_root)
     {
         if (!strcasecmp (pg_root->group_name, group_name))
@@ -637,7 +618,7 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
     gettimeofday (&t21, NULL);
 #endif
 
-	START_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_SHOULD_BUFFER);
+    START_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_SHOULD_BUFFER);
 
 
     name = malloc (strlen (method->base_path) + strlen (fd->name) + 1);
@@ -701,9 +682,7 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
                 MPI_File_read (md->fh, md->b.buff, md->b.pg_size, MPI_BYTE
                               ,&md->status
                               );
-                adios_parse_process_group_index_v1 (&md->b
-                                                   ,&md->old_pg_root
-                                                   );
+                adios_parse_process_group_index_v1 (&md->b, &md->index->pg_root);
 
 #if 1
                 adios_init_buffer_read_vars_index (&md->b);
@@ -713,7 +692,9 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
                 MPI_File_read (md->fh, md->b.buff, md->b.vars_size, MPI_BYTE
                               ,&md->status
                               );
-                adios_parse_vars_index_v1 (&md->b, &md->old_vars_root);
+                adios_parse_vars_index_v1 (&md->b, &md->index->vars_root, 
+                                           md->index->hashtbl_vars,
+                                           &md->index->vars_tail);
 
                 adios_init_buffer_read_attributes_index (&md->b);
                 MPI_File_seek (md->fh, md->b.attrs_index_offset
@@ -722,7 +703,7 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
                 MPI_File_read (md->fh, md->b.buff, md->b.attrs_size, MPI_BYTE
                               ,&md->status
                               );
-                adios_parse_attributes_index_v1 (&md->b, &md->old_attrs_root);
+                adios_parse_attributes_index_v1 (&md->b, &md->index->attrs_root);
 #endif
 
                 fd->base_offset = md->b.end_of_pgs;
@@ -741,7 +722,7 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
 
                     // go through the pg index to build the offsets array
                     build_offsets (&md->b, offsets, md->size
-                                  ,fd->group->name, md->old_pg_root
+                                  ,fd->group->name, md->index
                                   );
                     MPI_Scatter (offsets, 3, MPI_LONG_LONG
                                 ,MPI_IN_PLACE, 3, MPI_LONG_LONG
@@ -853,9 +834,9 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
                     }
                     // How to handle that each processor has varying amount of data??
                     md->striping_unit = offsets[1] - offsets[0];
-                    if (md->striping_unit > 4 * 1024 * 1024 * 1024L)
+                    if (md->striping_unit > 4 * 1024 * (uint64_t) (1024 * 1024L))
                     {
-                        md->striping_unit = 4 * 1024 * 1024 * 1024L;
+                        md->striping_unit = 4 * 1024 * (uint64_t) (1024 * 1024L);
                     }
 
                     md->b.pg_index_offset =   offsets [md->size - 1]
@@ -1032,9 +1013,7 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
                     MPI_File_read (md->fh, md->b.buff, md->b.pg_size, MPI_BYTE
                                   ,&md->status
                                   );
-                    adios_parse_process_group_index_v1 (&md->b
-                                                       ,&md->old_pg_root
-                                                       );
+                    adios_parse_process_group_index_v1 (&md->b ,&md->index->pg_root);
 
                     adios_init_buffer_read_vars_index (&md->b);
                     MPI_File_seek (md->fh, md->b.vars_index_offset
@@ -1043,7 +1022,10 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
                     MPI_File_read (md->fh, md->b.buff, md->b.vars_size, MPI_BYTE
                                   ,&md->status
                                   );
-                    adios_parse_vars_index_v1 (&md->b, &md->old_vars_root);
+                    adios_parse_vars_index_v1 (&md->b, &md->index->vars_root, 
+                                               md->index->hashtbl_vars,
+                                               &md->index->vars_tail);
+
 
                     adios_init_buffer_read_attributes_index (&md->b);
                     MPI_File_seek (md->fh, md->b.attrs_index_offset
@@ -1053,7 +1035,7 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
                                   ,MPI_BYTE, &md->status
                                   );
                     adios_parse_attributes_index_v1 (&md->b
-                                                    ,&md->old_attrs_root
+                                                    ,&md->index->attrs_root
                                                     );
 
                     fd->base_offset = md->b.end_of_pgs;
@@ -1295,7 +1277,7 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
         adios_shared_buffer_free (&md->b);
     }
 
-	STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_SHOULD_BUFFER);
+    STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_SHOULD_BUFFER);
 
 #if COLLECT_METRICS
     gettimeofday (&t22, NULL);
@@ -1312,7 +1294,7 @@ void adios_mpi_lustre_write (struct adios_file_struct * fd
     struct adios_MPI_data_struct * md = (struct adios_MPI_data_struct *)
                                                       method->method_data;
 
-	START_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_WRITE);
+    START_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_WRITE);
 
 
     if (v->got_buffer == adios_flag_yes)
@@ -1359,7 +1341,7 @@ void adios_mpi_lustre_write (struct adios_file_struct * fd
 
         // write payload
         // adios_write_var_payload_v1 (fd, v);
-        uint64_t var_size = adios_get_var_size (v, fd->group, v->data);
+        uint64_t var_size = adios_get_var_size (v, v->data);
         if (fd->base_offset + var_size > fd->pg_start_in_file + fd->write_size_bytes) 
             fprintf (stderr, "adios_mpi_write exceeds pg bound. File is corrupted. "
                              "Need to enlarge group size. \n");
@@ -1383,7 +1365,7 @@ void adios_mpi_lustre_write (struct adios_file_struct * fd
         adios_shared_buffer_free (&md->b);
     }
 
-	STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_WRITE);
+    STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_WRITE);
 
 
 #if COLLECT_METRICS
@@ -1473,15 +1455,11 @@ static void adios_mpi_lustre_do_read (struct adios_file_struct * fd
                                                       method->method_data;
     struct adios_var_struct * v = fd->group->vars;
 
-    struct adios_parse_buffer_struct data;
-
-    data.vars = v;
-    data.buffer = 0;
-    data.buffer_len = 0;
-
-    switch (md->b.version & ADIOS_VERSION_NUM_MASK)
+    uint32_t version = md->b.version & ADIOS_VERSION_NUM_MASK;
+    switch (version)
     {
         case 1:
+        case 2:
         {
             // the three section headers
             struct adios_process_group_header_struct_v1 pg_header;
@@ -1563,9 +1541,7 @@ static void adios_mpi_lustre_do_read (struct adios_file_struct * fd
         }
 
         default:
-            fprintf (stderr, "MPI read: file version unknown: %u\n"
-                    ,md->b.version
-                    );
+            fprintf (stderr, "MPI read: file version unknown: %u\n", version);
             return;
     }
 
@@ -1588,7 +1564,7 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
     static int iteration = 0;
 #endif
 
-	START_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_CLOSE);
+    START_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_CLOSE);
 
     switch (fd->mode)
     {
@@ -1624,19 +1600,18 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                 adios_write_close_vars_v1 (fd);
                 // fd->vars_start gets updated with the size written
                 uint64_t count;
-                int retlen;
-				START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
                 count = adios_mpi_lustre_striping_unit_write(
                                   md->fh,
                                   md->vars_start,
                                   fd->buffer,
                                   md->vars_header_size,
                                   md->block_unit);
-				STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
                 if (count != md->vars_header_size)
                 {
                     fprintf (stderr, "d:MPI method tried to write %llu, "
-                                     "only wrote %d\n"
+                                     "only wrote %llu\n"
                             ,md->vars_header_size
                             ,count
                             );
@@ -1691,7 +1666,7 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                 fd->offset = fd->base_offset - md->vars_start;
                 fd->vars_start = 0;
                 fd->buffer_size = 0;
-				START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
                 adios_write_close_attributes_v1 (fd);
                 // fd->vars_start gets updated with the size written
                 count = adios_mpi_lustre_striping_unit_write(
@@ -1700,7 +1675,7 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                                   fd->buffer,
                                   md->vars_header_size,
                                   md->block_unit);
-				STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
                 if (count != md->vars_header_size)
                 {
                     fprintf (stderr, "f:MPI method tried to write %llu, "
@@ -1723,9 +1698,7 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
             gettimeofday (&t12, NULL);
 #endif
             // build index appending to any existing index
-            adios_build_index_v1 (fd, &md->old_pg_root, &md->old_vars_root
-                                 ,&md->old_attrs_root
-                                 );
+            adios_build_index_v1 (fd, md->index);
             // if collective, gather the indexes from the rest and call
             if (md->group_comm != MPI_COMM_NULL)
             {
@@ -1738,12 +1711,12 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                     uint32_t total_size = 0;
                     int i;
 
-					START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
                     MPI_Gather (&size, 1, MPI_INT
                                ,index_sizes, 1, MPI_INT
                                ,0, md->group_comm
                                );
-					STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
 
                     for (i = 0; i < md->size; i++)
                     {
@@ -1753,12 +1726,12 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
 
                     recv_buffer = malloc (total_size);
 
-					START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
                     MPI_Gatherv (&size, 0, MPI_BYTE
                                 ,recv_buffer, index_sizes, index_offsets
                                 ,MPI_BYTE, 0, md->group_comm
                                 );
-					STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
 
                     char * buffer_save = md->b.buff;
                     uint64_t buffer_size_save = md->b.length;
@@ -1773,19 +1746,15 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                         adios_parse_process_group_index_v1 (&md->b
                                                            ,&new_pg_root
                                                            );
-                        adios_parse_vars_index_v1 (&md->b, &new_vars_root);
+                        adios_parse_vars_index_v1 (&md->b, &new_vars_root, NULL, NULL);
                         // do not merge attributes from other processes from 1.4
                         /*
                         adios_parse_attributes_index_v1 (&md->b
                                                         ,&new_attrs_root
                                                         );
                          */
-                        adios_merge_index_v1 (&md->old_pg_root
-                                             ,&md->old_vars_root
-                                             ,&md->old_attrs_root
-                                             ,new_pg_root, new_vars_root
-                                             ,new_attrs_root
-                                             );
+                        adios_merge_index_v1 (md->index, new_pg_root, 
+                                              new_vars_root, new_attrs_root);
                         new_pg_root = 0;
                         new_vars_root = 0;
                         new_attrs_root = 0;
@@ -1801,20 +1770,17 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                 else
                 {
                     adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                         ,0, md->old_pg_root
-                                         ,md->old_vars_root
-                                         ,md->old_attrs_root
-                                         );
+                                         ,0, md->index);
 
-					START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
-                    MPI_Gather (&buffer_size, 1, MPI_INT, 0, 0, MPI_INT
+                    START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    MPI_Gather (&buffer_offset, 1, MPI_INT, 0, 0, MPI_INT
                                ,0, md->group_comm
                                );
-                    MPI_Gatherv (buffer, buffer_size, MPI_BYTE
+                    MPI_Gatherv (buffer, buffer_offset, MPI_BYTE
                                 ,0, 0, 0, MPI_BYTE
                                 ,0, md->group_comm
                                 );
-					STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
                 }
             }
 
@@ -1828,33 +1794,30 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                     fprintf (stderr, "adios_mpi_write exceeds pg bound. File is corrupted. "
                              "Need to enlarge group size. \n");
 
-				START_TIMER (ADIOS_TIMER_MPI_LUSTRE_IO);
+                START_TIMER (ADIOS_TIMER_MPI_LUSTRE_IO);
                 adios_mpi_lustre_striping_unit_write(
                                   md->fh,
                                   fd->base_offset,
                                   fd->buffer,
                                   fd->bytes_written,
                                   md->block_unit);
-				STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_IO);
+                STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_IO);
             }
 
             if (md->rank == 0)
             {
                 adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                     ,index_start, md->old_pg_root
-                                     ,md->old_vars_root
-                                     ,md->old_attrs_root
-                                     );
+                                     ,index_start, md->index);
                 adios_write_version_v1 (&buffer, &buffer_size, &buffer_offset);
 
-				START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
                 adios_mpi_lustre_striping_unit_write(
                                   md->fh,
                                   md->b.pg_index_offset,
                                   buffer,
                                   buffer_offset,
                                   md->block_unit);
-				STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
             }
 #if COLLECT_METRICS
             gettimeofday (&t8, NULL);
@@ -1874,16 +1837,6 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                 buffer_offset = 0;
             }
 
-            adios_clear_index_v1 (new_pg_root, new_vars_root, new_attrs_root);
-            adios_clear_index_v1 (md->old_pg_root, md->old_vars_root
-                                 ,md->old_attrs_root
-                                 );
-            new_pg_root = 0;
-            new_vars_root = 0;
-            new_attrs_root = 0;
-            md->old_pg_root = 0;
-            md->old_vars_root = 0;
-            md->old_attrs_root = 0;
 #if COLLECT_METRICS
             gettimeofday (&t11, NULL);
             t15.tv_sec = t11.tv_sec;
@@ -1911,14 +1864,14 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                 adios_write_close_vars_v1 (fd);
                 // fd->vars_start gets updated with the size written
                 uint64_t count;
-				START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
                 count = adios_mpi_lustre_striping_unit_write(
                                   md->fh,
                                   md->vars_start,
                                   fd->buffer,
                                   md->vars_header_size,
                                   md->block_unit);
-				STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
                 if (count != md->vars_header_size)
                 {
                     fprintf (stderr, "d:MPI method tried to write %llu, "
@@ -1976,14 +1929,14 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                 fd->buffer_size = 0;
                 adios_write_close_attributes_v1 (fd);
                 // fd->vars_start gets updated with the size written
-				START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
                 count = adios_mpi_lustre_striping_unit_write(
                                   md->fh,
                                   md->vars_start,
                                   fd->buffer,
                                   md->vars_header_size,
                                   md->block_unit);
-				STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
                 if (count != md->vars_header_size)
                 {
                     fprintf (stderr, "f:MPI method tried to write %llu, "
@@ -1997,9 +1950,7 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
             }
 
             // build index appending to any existing index
-            adios_build_index_v1 (fd, &md->old_pg_root, &md->old_vars_root
-                                 ,&md->old_attrs_root
-                                 );
+            adios_build_index_v1 (fd, md->index);
             // if collective, gather the indexes from the rest and call
             if (md->group_comm != MPI_COMM_NULL)
             {
@@ -2012,12 +1963,12 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                     uint32_t total_size = 0;
                     int i;
 
-					START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
                     MPI_Gather (&size, 1, MPI_INT
                                ,index_sizes, 1, MPI_INT
                                ,0, md->group_comm
                                );
-					STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
 
                     for (i = 0; i < md->size; i++)
                     {
@@ -2027,12 +1978,12 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
 
                     recv_buffer = malloc (total_size);
 
-					START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
                     MPI_Gatherv (&size, 0, MPI_BYTE
                                 ,recv_buffer, index_sizes, index_offsets
                                 ,MPI_BYTE, 0, md->group_comm
                                 );
-					STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
 
                     char * buffer_save = md->b.buff;
                     uint64_t buffer_size_save = md->b.length;
@@ -2047,19 +1998,15 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                         adios_parse_process_group_index_v1 (&md->b
                                                            ,&new_pg_root
                                                            );
-                        adios_parse_vars_index_v1 (&md->b, &new_vars_root);
+                        adios_parse_vars_index_v1 (&md->b, &new_vars_root, NULL, NULL);
                         // do not merge attributes from other processes from 1.4
                         /*
                         adios_parse_attributes_index_v1 (&md->b
                                                         ,&new_attrs_root
                                                         );
                          */
-                        adios_merge_index_v1 (&md->old_pg_root
-                                             ,&md->old_vars_root
-                                             ,&md->old_attrs_root
-                                             ,new_pg_root, new_vars_root
-                                             ,new_attrs_root
-                                             );
+                        adios_merge_index_v1 (md->index, new_pg_root, 
+                                              new_vars_root, new_attrs_root);
                         new_pg_root = 0;
                         new_vars_root = 0;
                         new_attrs_root = 0;
@@ -2075,68 +2022,51 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                 else
                 {
                     adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                         ,0, md->old_pg_root
-                                         ,md->old_vars_root
-                                         ,md->old_attrs_root
-                                         );
+                                         ,0, md->index);
 
-					START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
-                    MPI_Gather (&buffer_size, 1, MPI_INT, 0, 0, MPI_INT
+                    START_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    MPI_Gather (&buffer_offset, 1, MPI_INT, 0, 0, MPI_INT
                                ,0, md->group_comm
                                );
-                    MPI_Gatherv (buffer, buffer_size, MPI_BYTE
+                    MPI_Gatherv (buffer, buffer_offset, MPI_BYTE
                                 ,0, 0, 0, MPI_BYTE
                                 ,0, md->group_comm
                                 );
-					STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
+                    STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_COMM);
                 }
             }
 
             if (fd->shared_buffer == adios_flag_yes)
             {
                 // everyone writes their data
-				START_TIMER (ADIOS_TIMER_MPI_LUSTRE_IO);
+                START_TIMER (ADIOS_TIMER_MPI_LUSTRE_IO);
                 adios_mpi_lustre_striping_unit_write(
                                   md->fh,
                                   fd->base_offset,
                                   fd->buffer,
                                   fd->bytes_written,
                                   md->block_unit);
-				STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_IO);
+                STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_IO);
             }
 
             if (md->rank == 0)
             {
                 adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                     ,index_start, md->old_pg_root
-                                     ,md->old_vars_root
-                                     ,md->old_attrs_root
-                                     );
+                                     ,index_start, md->index);
                 adios_write_version_v1 (&buffer, &buffer_size, &buffer_offset);
 
-				START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                START_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
                 adios_mpi_lustre_striping_unit_write(
                                   md->fh,
                                   md->b.pg_index_offset,
                                   buffer,
                                   buffer_offset,
                                   md->block_unit);
-				STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
+                STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_MD);
             }
 
             free (buffer);
 
-            adios_clear_index_v1 (new_pg_root, new_vars_root, new_attrs_root);
-            adios_clear_index_v1 (md->old_pg_root, md->old_vars_root
-                                 ,md->old_attrs_root
-                                 );
-            new_pg_root = 0;
-            new_vars_root = 0;
-            new_attrs_root = 0;
-            md->old_pg_root = 0;
-            md->old_vars_root = 0;
-            md->old_attrs_root = 0;
-
             break;
         }
 
@@ -2162,14 +2092,9 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
     memset (&md->status, 0, sizeof (MPI_Status));
     md->group_comm = MPI_COMM_NULL;
 
-    adios_clear_index_v1 (md->old_pg_root, md->old_vars_root
-                         ,md->old_attrs_root
-                         );
-    md->old_pg_root = 0;
-    md->old_vars_root = 0;
-    md->old_attrs_root = 0;
+    adios_clear_index_v1 (md->index);
 
-	STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_CLOSE);
+    STOP_TIMER (ADIOS_TIMER_MPI_LUSTRE_AD_CLOSE);
 
 #if COLLECT_METRICS
     print_metrics (md, iteration++);
@@ -2178,7 +2103,10 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
 
 void adios_mpi_lustre_finalize (int mype, struct adios_method_struct * method)
 {
-// nothing to do here
+    struct adios_MPI_data_struct * md = (struct adios_MPI_data_struct *)
+                                                 method->method_data;
+    adios_free_index_v1 (md->index);
+
     if (adios_mpi_lustre_initialized)
         adios_mpi_lustre_initialized = 0;
 }
