@@ -53,9 +53,7 @@ struct adios_POSIX_data_struct
     struct adios_bp_buffer_struct_v1 b;
 
     // old index structs we read in and have to be merged in
-    struct adios_index_process_group_struct_v1 * old_pg_root;
-    struct adios_index_var_struct_v1 * old_vars_root;
-    struct adios_index_attribute_struct_v1 * old_attrs_root;
+    struct adios_index_struct_v1 * index;
 
     uint64_t vars_start;
     uint64_t vars_header_size;
@@ -83,9 +81,7 @@ void adios_posix_init (const PairStruct * parameters
     method->method_data = malloc (sizeof (struct adios_POSIX_data_struct));
     p = (struct adios_POSIX_data_struct *) method->method_data;
     adios_buffer_struct_init (&p->b);
-    p->old_pg_root = 0;
-    p->old_vars_root = 0;
-    p->old_attrs_root = 0;
+    p->index = adios_alloc_index_v1(1); // with hashtables
     p->vars_start = 0;
     p->vars_header_size = 0;
 #ifdef HAVE_MPI
@@ -114,7 +110,6 @@ int adios_posix_open (struct adios_file_struct * fd
 {
     char * subfile_name;
     char * mdfile_name;
-    int rank;
     char * name_with_rank, rank_string[16];
     struct adios_POSIX_data_struct * p = (struct adios_POSIX_data_struct *)
                                                           method->method_data;
@@ -386,19 +381,18 @@ START_TIMER (ADIOS_TIMER_POSIX_AD_OPEN);
                 switch (version & ADIOS_VERSION_NUM_MASK)
                 {
                     case 1:
+                    case 2:
                         // read the old stuff and set the base offset
                         adios_posix_read_index_offsets (&p->b);
                         adios_parse_index_offsets_v1 (&p->b);
 
                         adios_posix_read_process_group_index (&p->b);
-                        adios_parse_process_group_index_v1 (&p->b
-                                                           ,&p->old_pg_root
-                                                           );
+                        adios_parse_process_group_index_v1 (&p->b, &p->index->pg_root);
 
                         // find the largest time index so we can append properly
                         struct adios_index_process_group_struct_v1 * pg;
                         uint32_t max_time_index = 0;
-                        pg = p->old_pg_root;
+                        pg = p->index->pg_root;
                         while (pg)
                         {
                             if (pg->time_index > max_time_index)
@@ -408,12 +402,12 @@ START_TIMER (ADIOS_TIMER_POSIX_AD_OPEN);
                         fd->group->time_index = ++max_time_index;
 
                         adios_posix_read_vars_index (&p->b);
-                        adios_parse_vars_index_v1 (&p->b, &p->old_vars_root);
+                        adios_parse_vars_index_v1 (&p->b, &p->index->vars_root, 
+                                                   p->index->hashtbl_vars,
+                                                   &p->index->vars_tail);
 
                         adios_posix_read_attributes_index (&p->b);
-                        adios_parse_attributes_index_v1 (&p->b
-                                                        ,&p->old_attrs_root
-                                                        );
+                        adios_parse_attributes_index_v1 (&p->b, &p->index->attrs_root);
 
                         fd->base_offset = p->b.end_of_pgs;
                         fd->pg_start_in_file = p->b.end_of_pgs;
@@ -448,8 +442,8 @@ START_TIMER (ADIOS_TIMER_POSIX_AD_OPEN);
 
     free (subfile_name);
     free (mdfile_name);
-	
-	STOP_TIMER (ADIOS_TIMER_POSIX_AD_OPEN);
+    
+    STOP_TIMER (ADIOS_TIMER_POSIX_AD_OPEN);
 
     return 1;
 }
@@ -461,7 +455,7 @@ enum ADIOS_FLAG adios_posix_should_buffer (struct adios_file_struct * fd
     struct adios_POSIX_data_struct * p = (struct adios_POSIX_data_struct *)
                                                           method->method_data;
 
-	START_TIMER (ADIOS_TIMER_POSIX_AD_SHOULD_BUFFER);
+    START_TIMER (ADIOS_TIMER_POSIX_AD_SHOULD_BUFFER);
 
     if (fd->shared_buffer == adios_flag_no && fd->mode != adios_mode_read)
     {
@@ -469,15 +463,15 @@ enum ADIOS_FLAG adios_posix_should_buffer (struct adios_file_struct * fd
         adios_write_process_group_header_v1 (fd, fd->write_size_bytes);
 
         lseek (p->b.f, fd->base_offset, SEEK_SET);
-		START_TIMER (ADIOS_TIMER_POSIX_MD);
+        START_TIMER (ADIOS_TIMER_POSIX_MD);
         ssize_t s = write (p->b.f, fd->buffer, fd->bytes_written);
-		STOP_TIMER (ADIOS_TIMER_POSIX_MD);
+        STOP_TIMER (ADIOS_TIMER_POSIX_MD);
         if (s != fd->bytes_written)
         {
             fprintf (stderr, "POSIX method tried to write %llu, "
-                             "only wrote %llu\n"
+                             "only wrote %lld\n"
                     ,fd->bytes_written
-                    ,s
+                    ,(int64_t)s
                     );
         }
         fd->base_offset += s;
@@ -496,7 +490,7 @@ enum ADIOS_FLAG adios_posix_should_buffer (struct adios_file_struct * fd
         adios_shared_buffer_free (&p->b);
     }
 
-	STOP_TIMER (ADIOS_TIMER_POSIX_AD_SHOULD_BUFFER);
+    STOP_TIMER (ADIOS_TIMER_POSIX_AD_SHOULD_BUFFER);
 
     return fd->shared_buffer;   // buffer if there is space
 }
@@ -510,7 +504,7 @@ void adios_posix_write (struct adios_file_struct * fd
     struct adios_POSIX_data_struct * p = (struct adios_POSIX_data_struct *)
                                                           method->method_data;
 
-	START_TIMER (ADIOS_TIMER_POSIX_AD_WRITE);
+    START_TIMER (ADIOS_TIMER_POSIX_AD_WRITE);
 
     if (v->got_buffer == adios_flag_yes)
     {
@@ -533,15 +527,15 @@ void adios_posix_write (struct adios_file_struct * fd
     {
         // var payload sent for sizing information
         adios_write_var_header_v1 (fd, v);
-		START_TIMER (ADIOS_TIMER_POSIX_MD);
+        START_TIMER (ADIOS_TIMER_POSIX_MD);
         ssize_t s = write (p->b.f, fd->buffer, fd->bytes_written);
-		STOP_TIMER (ADIOS_TIMER_POSIX_MD);
+        STOP_TIMER (ADIOS_TIMER_POSIX_MD);
         if (s != fd->bytes_written)
         {
             fprintf (stderr, "POSIX method tried to write %llu, "
-                             "only wrote %llu\n"
+                             "only wrote %lld\n"
                     ,fd->bytes_written
-                    ,s
+                    ,(int64_t)s
                     );
         }
         fd->base_offset += s;
@@ -551,16 +545,16 @@ void adios_posix_write (struct adios_file_struct * fd
 
         // write payload
         adios_write_var_payload_v1 (fd, v);
-        uint64_t var_size = adios_get_var_size (v, fd->group, v->data);
+        uint64_t var_size = adios_get_var_size (v, v->data);
         if (fd->base_offset + var_size > fd->pg_start_in_file + fd->write_size_bytes)
             fprintf (stderr, "adios_posix_write exceeds pg bound. File is corrupted. "
                              "Need to enlarge group size. \n"); 
 
         int32_t to_write;
         uint64_t bytes_written = 0;
-        if (var_size > INT32_MAX)
+        if (var_size > MAX_MPIWRITE_SIZE)
         {
-            to_write = INT32_MAX;
+            to_write = MAX_MPIWRITE_SIZE;
         }
         else
         {
@@ -569,14 +563,14 @@ void adios_posix_write (struct adios_file_struct * fd
 
         while (bytes_written < var_size)
         {
-			START_TIMER (ADIOS_TIMER_POSIX_IO);
+            START_TIMER (ADIOS_TIMER_POSIX_IO);
             bytes_written += write (p->b.f, v->data + bytes_written, to_write);
-			STOP_TIMER (ADIOS_TIMER_POSIX_IO);
+            STOP_TIMER (ADIOS_TIMER_POSIX_IO);
             if (var_size > bytes_written)
             {
-                if (var_size - bytes_written > INT32_MAX)
+                if (var_size - bytes_written > MAX_MPIWRITE_SIZE)
                 {
-                    to_write = INT32_MAX;
+                    to_write = MAX_MPIWRITE_SIZE;
                 }
                 else
                 {
@@ -585,16 +579,16 @@ void adios_posix_write (struct adios_file_struct * fd
             }
         }
 
-		START_TIMER (ADIOS_TIMER_POSIX_IO);
+        START_TIMER (ADIOS_TIMER_POSIX_IO);
         s = write (p->b.f, v->data, var_size);
-		STOP_TIMER (ADIOS_TIMER_POSIX_IO);
+        STOP_TIMER (ADIOS_TIMER_POSIX_IO);
         s = bytes_written;
         if (s != var_size)
         {
             fprintf (stderr, "POSIX method tried to write %llu, "
-                             "only wrote %llu\n"
+                             "only wrote %lld\n"
                     ,var_size
-                    ,s
+                    ,(int64_t)s
                     );
         }
         fd->base_offset += s;
@@ -603,7 +597,7 @@ void adios_posix_write (struct adios_file_struct * fd
         adios_shared_buffer_free (&p->b);
     }
 
-	STOP_TIMER (ADIOS_TIMER_POSIX_AD_WRITE);
+    STOP_TIMER (ADIOS_TIMER_POSIX_AD_WRITE);
 }
 
 void adios_posix_get_write_buffer (struct adios_file_struct * fd
@@ -695,9 +689,9 @@ static void adios_posix_do_write (struct adios_file_struct * fd
             fprintf (stderr, "adios_posix_write exceeds pg bound. File is corrupted. "
                              "Need to enlarge group size. \n");
 
-        if (fd->bytes_written > INT32_MAX)
+        if (fd->bytes_written > MAX_MPIWRITE_SIZE)
         {
-            to_write = INT32_MAX;
+            to_write = MAX_MPIWRITE_SIZE;
         }
         else
         {
@@ -710,9 +704,9 @@ static void adios_posix_do_write (struct adios_file_struct * fd
             bytes_written += to_write;
             if (fd->bytes_written > bytes_written)
             {
-                if (fd->bytes_written - bytes_written > INT32_MAX)
+                if (fd->bytes_written - bytes_written > MAX_MPIWRITE_SIZE)
                 {
-                    to_write = INT32_MAX;
+                    to_write = MAX_MPIWRITE_SIZE;
                 }
                 else
                 {
@@ -739,25 +733,20 @@ static void adios_posix_do_read (struct adios_file_struct * fd
                                                           method->method_data;
     struct adios_var_struct * v = fd->group->vars;
 
-    struct adios_parse_buffer_struct data;
-
-    data.vars = v;
-    data.buffer = 0;
-    data.buffer_len = 0;
-
     uint32_t version = 0;
 
     adios_posix_read_version (&p->b);
     adios_parse_version (&p->b, &version);
+    version &= ADIOS_VERSION_NUM_MASK;
 
-    switch (version & ADIOS_VERSION_NUM_MASK)
+    switch (version)
     {
         case 1:
+        case 2:
         {
-            struct adios_index_process_group_struct_v1 * pg_root = 0;
+            struct adios_index_struct_v1 * index = adios_alloc_index_v1(0); // no hashtables
+            struct adios_index_process_group_struct_v1 * pg_root = index->pg_root;
             struct adios_index_process_group_struct_v1 * pg_root_temp = 0;
-            struct adios_index_var_struct_v1 * vars_root = 0;
-            struct adios_index_attribute_struct_v1 * attrs_root = 0;
 
             adios_posix_read_index_offsets (&p->b);
             adios_parse_index_offsets_v1 (&p->b);
@@ -766,10 +755,10 @@ static void adios_posix_do_read (struct adios_file_struct * fd
             adios_parse_process_group_index_v1 (&p->b, &pg_root);
 #if 1
             adios_posix_read_vars_index (&p->b);
-            adios_parse_vars_index_v1 (&p->b, &vars_root);
+            adios_parse_vars_index_v1 (&p->b, &index->vars_root, NULL, NULL);
 
             adios_posix_read_attributes_index (&p->b);
-            adios_parse_attributes_index_v1 (&p->b, &attrs_root);
+            adios_parse_attributes_index_v1 (&p->b, &index->attrs_root);
 #endif
 
             // the three section headers
@@ -852,14 +841,12 @@ static void adios_posix_do_read (struct adios_file_struct * fd
             }
 #endif
             adios_clear_process_group_header_v1 (&pg_header);
-            adios_clear_index_v1 (pg_root, vars_root, attrs_root);
+            adios_clear_index_v1 (index);
             break;
         }
 
         default:
-            fprintf (stderr, "POSIX read: file version unknown: %u\n"
-                    ,version
-                    );
+            fprintf (stderr, "POSIX read: file version unknown: %u\n", version);
             return;
     }
 
@@ -878,7 +865,7 @@ void adios_posix_close (struct adios_file_struct * fd
     struct adios_index_var_struct_v1 * new_vars_root = 0;
     struct adios_index_attribute_struct_v1 * new_attrs_root = 0;
 
-	START_TIMER (ADIOS_TIMER_POSIX_AD_CLOSE);
+    START_TIMER (ADIOS_TIMER_POSIX_AD_CLOSE);
 
     switch (fd->mode)
     {
@@ -895,15 +882,15 @@ void adios_posix_close (struct adios_file_struct * fd
                 adios_write_close_vars_v1 (fd);
                 // fd->vars_start gets updated with the size written
                 fd->offset = lseek (p->b.f, p->vars_start, SEEK_SET);
-				START_TIMER (ADIOS_TIMER_POSIX_IO);
+                START_TIMER (ADIOS_TIMER_POSIX_IO);
                 ssize_t s = write (p->b.f, fd->buffer, p->vars_header_size);
-				STOP_TIMER (ADIOS_TIMER_POSIX_IO);
+                STOP_TIMER (ADIOS_TIMER_POSIX_IO);
                 if (s != fd->vars_start)
                 {
                     fprintf (stderr, "POSIX method tried to write %llu, "
-                                     "only wrote %llu\n"
+                                     "only wrote %lld\n"
                             ,fd->vars_start
-                            ,s
+                            ,(int64_t)s
                             );
                 }
                 fd->offset = 0;
@@ -932,9 +919,9 @@ void adios_posix_close (struct adios_file_struct * fd
                         if (s != fd->bytes_written)
                         {
                             fprintf (stderr, "POSIX method tried to write %llu, "
-                                    "only wrote %llu\n"
+                                    "only wrote %lld\n"
                                     ,fd->bytes_written
-                                    ,s
+                                    ,(int64_t)s
                                     );
                         }
                         fd->base_offset += s;
@@ -953,15 +940,15 @@ void adios_posix_close (struct adios_file_struct * fd
                 adios_write_close_attributes_v1 (fd);
                 fd->offset = lseek (p->b.f, p->vars_start, SEEK_SET);
                 // fd->vars_start gets updated with the size written
-				START_TIMER (ADIOS_TIMER_POSIX_MD);
+                START_TIMER (ADIOS_TIMER_POSIX_MD);
                 s = write (p->b.f, fd->buffer, p->vars_header_size);
-				STOP_TIMER (ADIOS_TIMER_POSIX_MD);
+                STOP_TIMER (ADIOS_TIMER_POSIX_MD);
                 if (s != p->vars_header_size)
                 {
                     fprintf (stderr, "POSIX method tried to write %llu, "
-                                     "only wrote %llu\n"
+                                     "only wrote %lld\n"
                             ,p->vars_header_size
-                            ,s
+                            ,(int64_t)s
                             );
                 }
                 fd->offset = 0;
@@ -975,19 +962,15 @@ void adios_posix_close (struct adios_file_struct * fd
             uint64_t index_start = fd->base_offset + fd->offset;
 
             // build index
-            adios_build_index_v1 (fd, &p->old_pg_root, &p->old_vars_root
-                                 ,&p->old_attrs_root
-                                 );
+            adios_build_index_v1 (fd, p->index);
             // if collective, gather the indexes from the rest and call
             // adios_merge_index_v1 (&new_pg_root, &new_vars_root, pg, vars);
             adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                 ,index_start, p->old_pg_root, p->old_vars_root
-                                 ,p->old_attrs_root
-                                 );
+                                 ,index_start, p->index);
             adios_write_version_v1 (&buffer, &buffer_size, &buffer_offset);
-			START_TIMER (ADIOS_TIMER_POSIX_IO);
+            START_TIMER (ADIOS_TIMER_POSIX_IO);
             adios_posix_do_write (fd, method, buffer, buffer_offset); // Buffered vars written here
-			STOP_TIMER (ADIOS_TIMER_POSIX_IO);
+            STOP_TIMER (ADIOS_TIMER_POSIX_IO);
 #ifdef HAVE_MPI
             if (p->group_comm != MPI_COMM_SELF)
             {
@@ -999,12 +982,12 @@ void adios_posix_close (struct adios_file_struct * fd
                     int i;
                     uint32_t size = 0, total_size = 0;
 
-					START_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    START_TIMER (ADIOS_TIMER_POSIX_COMM);
                     MPI_Gather (&size, 1, MPI_INT
                                ,index_sizes, 1, MPI_INT
                                ,0, p->group_comm
                                );
-					STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
 
                     for (i = 0; i < p->size; i++)
                     {
@@ -1013,12 +996,12 @@ void adios_posix_close (struct adios_file_struct * fd
                     }
 
                     recv_buffer = malloc (total_size);
-					START_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    START_TIMER (ADIOS_TIMER_POSIX_COMM);
                     MPI_Gatherv (&size, 0, MPI_BYTE
                                 ,recv_buffer, index_sizes, index_offsets
                                 ,MPI_BYTE, 0, p->group_comm
                                 );
-					STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
 
                     char * buffer_save = p->b.buff;
                     uint64_t buffer_size_save = p->b.length;
@@ -1033,7 +1016,7 @@ void adios_posix_close (struct adios_file_struct * fd
                         adios_parse_process_group_index_v1 (&p->b
                                                            ,&new_pg_root
                                                            );
-                        adios_parse_vars_index_v1 (&p->b, &new_vars_root);
+                        adios_parse_vars_index_v1 (&p->b, &new_vars_root, NULL, NULL);
                         // do not merge attributes from other processes from 1.4
                         /*
                         adios_parse_attributes_index_v1 (&p->b
@@ -1041,12 +1024,8 @@ void adios_posix_close (struct adios_file_struct * fd
                                                         );
                         */
 
-                        adios_merge_index_v1 (&p->old_pg_root
-                                             ,&p->old_vars_root
-                                             ,&p->old_attrs_root
-                                             ,new_pg_root, new_vars_root
-                                             ,new_attrs_root
-                                             );
+                        adios_merge_index_v1 (p->index, new_pg_root, 
+                                              new_vars_root, new_attrs_root);
                         new_pg_root = 0;
                         new_vars_root = 0;
                         new_attrs_root = 0;
@@ -1068,8 +1047,7 @@ void adios_posix_close (struct adios_file_struct * fd
 
                     adios_write_index_v1 (&global_index_buffer, &global_index_buffer_size
                                          ,&global_index_buffer_offset, global_index_start
-                                         ,p->old_pg_root, p->old_vars_root, p->old_attrs_root
-                                         );
+                                         ,p->index);
 
                     flag |= ADIOS_VERSION_HAVE_SUBFILE;
 
@@ -1078,15 +1056,15 @@ void adios_posix_close (struct adios_file_struct * fd
                                                 ,&global_index_buffer_offset
                                                 ,flag
                                                 );
-					START_TIMER (ADIOS_TIMER_POSIX_MD);
+                    START_TIMER (ADIOS_TIMER_POSIX_MD);
                     ssize_t s = write (p->mf, global_index_buffer, global_index_buffer_offset);
-					STOP_TIMER (ADIOS_TIMER_POSIX_MD);
+                    STOP_TIMER (ADIOS_TIMER_POSIX_MD);
                     if (s != global_index_buffer_offset)
                     {
                         fprintf (stderr, "POSIX method tried to write %llu, "
-                                         "only wrote %llu\n"
+                                         "only wrote %lld\n"
                                          ,fd->bytes_written
-                                         ,s
+                                         ,(int64_t)s
                                 );
                     }
 
@@ -1094,9 +1072,9 @@ void adios_posix_close (struct adios_file_struct * fd
                 }
                 else
                 {
-					START_TIMER (ADIOS_TIMER_POSIX_COMM);
-					// Added this explicit cast to avoid truncation of low-order bytes on BGP
-					int i_buffer_size = (int) buffer_size;
+                    START_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    // Added this explicit cast to avoid truncation of low-order bytes on BGP
+                    int i_buffer_size = (int) buffer_size;
                     MPI_Gather (&i_buffer_size, 1, MPI_INT
                                ,0, 0, MPI_INT
                                ,0, p->group_comm
@@ -1106,14 +1084,12 @@ void adios_posix_close (struct adios_file_struct * fd
                                 ,0, 0, 0, MPI_BYTE
                                 ,0, p->group_comm
                                 );
-					STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
                 }
             }
 #endif
             free (buffer);
 
-            adios_clear_index_v1 (new_pg_root, new_vars_root, new_attrs_root);
-
             break;
         }
 
@@ -1131,15 +1107,15 @@ void adios_posix_close (struct adios_file_struct * fd
                 adios_write_close_vars_v1 (fd);
                 // fd->vars_start gets updated with the size written
                 fd->offset = lseek (p->b.f, p->vars_start, SEEK_SET);
-				START_TIMER (ADIOS_TIMER_POSIX_IO);
+                START_TIMER (ADIOS_TIMER_POSIX_IO);
                 ssize_t s = write (p->b.f, fd->buffer, p->vars_header_size);
-				STOP_TIMER (ADIOS_TIMER_POSIX_IO);
+                STOP_TIMER (ADIOS_TIMER_POSIX_IO);
                 if (s != fd->vars_start)
                 {
                     fprintf (stderr, "POSIX method tried to write %llu, "
-                                     "only wrote %llu\n"
+                                     "only wrote %lld\n"
                             ,fd->vars_start
-                            ,s
+                            ,(int64_t)s
                             );
                 }
                 fd->offset = 0;
@@ -1165,9 +1141,9 @@ void adios_posix_close (struct adios_file_struct * fd
                         if (s != fd->bytes_written)
                         {
                             fprintf (stderr, "POSIX method tried to write %llu, "
-                                    "only wrote %llu\n"
+                                    "only wrote %lld\n"
                                     ,fd->bytes_written
-                                    ,s
+                                    ,(int64_t)s
                                     );
                         }
                         fd->base_offset += s;
@@ -1186,15 +1162,15 @@ void adios_posix_close (struct adios_file_struct * fd
                 adios_write_close_attributes_v1 (fd);
                 fd->offset = lseek (p->b.f, p->vars_start, SEEK_SET);
                 // fd->vars_start gets updated with the size written
-				START_TIMER (ADIOS_TIMER_POSIX_MD);
+                START_TIMER (ADIOS_TIMER_POSIX_MD);
                 s = write (p->b.f, fd->buffer, p->vars_header_size);
-				STOP_TIMER (ADIOS_TIMER_POSIX_MD);
+                STOP_TIMER (ADIOS_TIMER_POSIX_MD);
                 if (s != p->vars_header_size)
                 {
                     fprintf (stderr, "POSIX method tried to write %llu, "
-                                     "only wrote %llu\n"
+                                     "only wrote %lld\n"
                             ,p->vars_header_size
-                            ,s
+                            ,(int64_t)s
                             );
                 }
                 fd->offset = 0;
@@ -1207,13 +1183,9 @@ void adios_posix_close (struct adios_file_struct * fd
             uint64_t index_start = fd->base_offset + fd->offset;
 
             // build index
-            adios_build_index_v1 (fd, &p->old_pg_root, &p->old_vars_root
-                                 ,&p->old_attrs_root
-                                 );
+            adios_build_index_v1 (fd, p->index);
             adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                 ,index_start, p->old_pg_root, p->old_vars_root
-                                 ,p->old_attrs_root
-                                 );
+                                 ,index_start, p->index);
 #ifdef HAVE_MPI
             if (p->group_comm != MPI_COMM_SELF)
             {
@@ -1225,12 +1197,12 @@ void adios_posix_close (struct adios_file_struct * fd
                     int i;
                     uint32_t size = 0, total_size = 0;
 
-					START_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    START_TIMER (ADIOS_TIMER_POSIX_COMM);
                     MPI_Gather (&size, 1, MPI_INT
                                ,index_sizes, 1, MPI_INT
                                ,0, p->group_comm
                                );
-					STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
 
                     for (i = 0; i < p->size; i++)
                     {
@@ -1240,12 +1212,12 @@ void adios_posix_close (struct adios_file_struct * fd
 
                     recv_buffer = malloc (total_size);
 
-					START_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    START_TIMER (ADIOS_TIMER_POSIX_COMM);
                     MPI_Gatherv (&size, 0, MPI_BYTE
                                 ,recv_buffer, index_sizes, index_offsets
                                 ,MPI_BYTE, 0, p->group_comm
                                 );
-					STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
 
                     char * buffer_save = p->b.buff;
                     uint64_t buffer_size_save = p->b.length;
@@ -1260,7 +1232,7 @@ void adios_posix_close (struct adios_file_struct * fd
                         adios_parse_process_group_index_v1 (&p->b
                                                            ,&new_pg_root
                                                            );
-                        adios_parse_vars_index_v1 (&p->b, &new_vars_root);
+                        adios_parse_vars_index_v1 (&p->b, &new_vars_root, NULL, NULL);
                         // do not merge attributes from other processes from 1.4
                         /*
                         adios_parse_attributes_index_v1 (&p->b
@@ -1268,21 +1240,17 @@ void adios_posix_close (struct adios_file_struct * fd
                                                         );
                          */
 
-                        adios_merge_index_v1 (&p->old_pg_root
-                                             ,&p->old_vars_root
-                                             ,&p->old_attrs_root
-                                             ,new_pg_root, new_vars_root
-                                             ,new_attrs_root
-                                             );
+                        adios_merge_index_v1 (p->index,new_pg_root, 
+                                              new_vars_root, new_attrs_root);
                     
                         new_pg_root = 0;
                         new_vars_root = 0;
                         new_attrs_root = 0;
                     }
 
-                    adios_sort_index_v1 (&p->old_pg_root
-                                        ,&p->old_vars_root
-                                        ,&p->old_attrs_root
+                    adios_sort_index_v1 (&p->index->pg_root
+                                        ,&p->index->vars_root
+                                        ,&p->index->attrs_root
                                         );
 
                     p->b.buff = buffer_save;
@@ -1301,8 +1269,7 @@ void adios_posix_close (struct adios_file_struct * fd
 
                     adios_write_index_v1 (&global_index_buffer, &global_index_buffer_size
                                          ,&global_index_buffer_offset, global_index_start
-                                         ,p->old_pg_root, p->old_vars_root, p->old_attrs_root
-                                         );
+                                         ,p->index);
 
                     flag |= ADIOS_VERSION_HAVE_SUBFILE;
 
@@ -1312,15 +1279,15 @@ void adios_posix_close (struct adios_file_struct * fd
                                                 ,flag
                                                 );
 
-					START_TIMER (ADIOS_TIMER_POSIX_MD);
+                    START_TIMER (ADIOS_TIMER_POSIX_MD);
                     ssize_t s = write (p->mf, global_index_buffer, global_index_buffer_offset);
-					STOP_TIMER (ADIOS_TIMER_POSIX_MD);
+                    STOP_TIMER (ADIOS_TIMER_POSIX_MD);
                     if (s != global_index_buffer_offset)
                     {
                         fprintf (stderr, "POSIX method tried to write %llu, "
-                                         "only wrote %llu, Mode: a\n"
+                                         "only wrote %lld, Mode: a\n"
                                          ,global_index_buffer_offset
-                                         ,s
+                                         ,(int64_t)s
                                 );
                     }
 
@@ -1330,7 +1297,7 @@ void adios_posix_close (struct adios_file_struct * fd
                 }
                 else
                 {
-					START_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    START_TIMER (ADIOS_TIMER_POSIX_COMM);
                     MPI_Gather (&buffer_size, 1, MPI_INT
                                ,0, 0, MPI_INT
                                ,0, p->group_comm
@@ -1340,14 +1307,14 @@ void adios_posix_close (struct adios_file_struct * fd
                                 ,0, 0, 0, MPI_BYTE
                                 ,0, p->group_comm
                                 );
-					STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
+                    STOP_TIMER (ADIOS_TIMER_POSIX_COMM);
                 }
             }
 #endif
             adios_write_version_v1 (&buffer, &buffer_size, &buffer_offset);
-			START_TIMER (ADIOS_TIMER_POSIX_MD);
+            START_TIMER (ADIOS_TIMER_POSIX_MD);
             adios_posix_do_write (fd, method, buffer, buffer_offset);
-			STOP_TIMER (ADIOS_TIMER_POSIX_MD);
+            STOP_TIMER (ADIOS_TIMER_POSIX_MD);
 
             free (buffer);
 
@@ -1377,18 +1344,17 @@ void adios_posix_close (struct adios_file_struct * fd
     }
 
     adios_posix_close_internal (&p->b);
-    adios_clear_index_v1 (p->old_pg_root, p->old_vars_root, p->old_attrs_root);
-    p->old_pg_root = 0;
-    p->old_vars_root = 0;
-    p->old_attrs_root = 0;
+    adios_clear_index_v1 (p->index);
 
-	STOP_TIMER (ADIOS_TIMER_POSIX_AD_CLOSE);
+    STOP_TIMER (ADIOS_TIMER_POSIX_AD_CLOSE);
 
 }
 
 void adios_posix_finalize (int mype, struct adios_method_struct * method)
 {
-// nothing to do here
+    struct adios_POSIX_data_struct * p = (struct adios_POSIX_data_struct *)
+                                                          method->method_data;
+    adios_free_index_v1 (p->index);
     if (adios_posix_initialized)
         adios_posix_initialized = 0;
 }
