@@ -35,9 +35,7 @@ struct adios_POSIX1_data_struct
     struct adios_bp_buffer_struct_v1 b;
 
     // old index structs we read in and have to be merged in
-    struct adios_index_process_group_struct_v1 * old_pg_root;
-    struct adios_index_var_struct_v1 * old_vars_root;
-    struct adios_index_attribute_struct_v1 * old_attrs_root;
+    struct adios_index_struct_v1 * index;
 
     uint64_t vars_start;
     uint64_t vars_header_size;
@@ -56,9 +54,7 @@ void adios_posix1_init (const PairStruct * parameters
     method->method_data = malloc (sizeof (struct adios_POSIX1_data_struct));
     p = (struct adios_POSIX1_data_struct *) method->method_data;
     adios_buffer_struct_init (&p->b);
-    p->old_pg_root = 0;
-    p->old_vars_root = 0;
-    p->old_attrs_root = 0;
+    p->index = adios_alloc_index_v1(1); // with hashtables
     p->vars_start = 0;
     p->vars_header_size = 0;
 }
@@ -172,19 +168,20 @@ int adios_posix1_open (struct adios_file_struct * fd
                 switch (version & ADIOS_VERSION_NUM_MASK)
                 {
                     case 1:
+                    case 2:
                         // read the old stuff and set the base offset
                         adios_posix_read_index_offsets (&p->b);
                         adios_parse_index_offsets_v1 (&p->b);
 
                         adios_posix_read_process_group_index (&p->b);
                         adios_parse_process_group_index_v1 (&p->b
-                                                           ,&p->old_pg_root
+                                                           ,&p->index->pg_root
                                                            );
 
                         // find the largest time index so we can append properly
                         struct adios_index_process_group_struct_v1 * pg;
                         uint32_t max_time_index = 0;
-                        pg = p->old_pg_root;
+                        pg = p->index->pg_root;
                         while (pg)
                         {
                             if (pg->time_index > max_time_index)
@@ -194,11 +191,13 @@ int adios_posix1_open (struct adios_file_struct * fd
                         fd->group->time_index = ++max_time_index;
 
                         adios_posix_read_vars_index (&p->b);
-                        adios_parse_vars_index_v1 (&p->b, &p->old_vars_root);
+                        adios_parse_vars_index_v1 (&p->b, &p->index->vars_root, 
+                                                   p->index->hashtbl_vars,
+                                                   &p->index->vars_tail);
 
                         adios_posix_read_attributes_index (&p->b);
                         adios_parse_attributes_index_v1 (&p->b
-                                                        ,&p->old_attrs_root
+                                                        ,&p->index->attrs_root
                                                         );
 
                         fd->base_offset = p->b.end_of_pgs;
@@ -252,9 +251,9 @@ enum ADIOS_FLAG adios_posix1_should_buffer (struct adios_file_struct * fd
         if (s != fd->bytes_written)
         {
             fprintf (stderr, "POSIX1 method tried to write %llu, "
-                             "only wrote %llu\n"
+                             "only wrote %lld\n"
                     ,fd->bytes_written
-                    ,s
+                    ,(int64_t)s
                     );
         }
         fd->base_offset += s;
@@ -310,9 +309,9 @@ void adios_posix1_write (struct adios_file_struct * fd
         if (s != fd->bytes_written)
         {
             fprintf (stderr, "POSIX1 method tried to write %llu, "
-                             "only wrote %llu\n"
+                             "only wrote %lld\n"
                     ,fd->bytes_written
-                    ,s
+                    ,(int64_t)s
                     );
         }
         fd->base_offset += s;
@@ -322,16 +321,16 @@ void adios_posix1_write (struct adios_file_struct * fd
 
         // write payload
         // adios_write_var_payload_v1 (fd, v);
-        uint64_t var_size = adios_get_var_size (v, fd->group, v->data);
+        uint64_t var_size = adios_get_var_size (v, v->data);
         if (fd->base_offset + var_size > fd->pg_start_in_file + fd->write_size_bytes)
             fprintf (stderr, "adios_posix1_write exceeds pg bound. File is corrupted. "
                              "Need to enlarge group size. \n"); 
 
         int32_t to_write;
         uint64_t bytes_written = 0;
-        if (var_size > INT32_MAX)
+        if (var_size > MAX_MPIWRITE_SIZE)
         {
-            to_write = INT32_MAX;
+            to_write = MAX_MPIWRITE_SIZE;
         }
         else
         {
@@ -343,9 +342,9 @@ void adios_posix1_write (struct adios_file_struct * fd
             bytes_written += write (p->b.f, v->data + bytes_written, to_write);
             if (var_size > bytes_written)
             {
-                if (var_size - bytes_written > INT32_MAX)
+                if (var_size - bytes_written > MAX_MPIWRITE_SIZE)
                 {
-                    to_write = INT32_MAX;
+                    to_write = MAX_MPIWRITE_SIZE;
                 }
                 else
                 {
@@ -359,9 +358,9 @@ void adios_posix1_write (struct adios_file_struct * fd
         if (s != var_size)
         {
             fprintf (stderr, "POSIX1 method tried to write %llu, "
-                             "only wrote %llu\n"
+                             "only wrote %lld\n"
                     ,var_size
-                    ,s
+                    ,(int64_t)s
                     );
         }
         fd->base_offset += s;
@@ -460,9 +459,9 @@ static void adios_posix1_do_write (struct adios_file_struct * fd
             fprintf (stderr, "adios_posix1_write exceeds pg bound. File is corrupted. "
                              "Need to enlarge group size. \n");
 
-        if (fd->bytes_written > INT32_MAX)
+        if (fd->bytes_written > MAX_MPIWRITE_SIZE)
         {
-            to_write = INT32_MAX;
+            to_write = MAX_MPIWRITE_SIZE;
         }
         else
         {
@@ -475,9 +474,9 @@ static void adios_posix1_do_write (struct adios_file_struct * fd
             bytes_written += to_write;
             if (fd->bytes_written > bytes_written)
             {
-                if (fd->bytes_written - bytes_written > INT32_MAX)
+                if (fd->bytes_written - bytes_written > MAX_MPIWRITE_SIZE)
                 {
-                    to_write = INT32_MAX;
+                    to_write = MAX_MPIWRITE_SIZE;
                 }
                 else
                 {
@@ -503,25 +502,20 @@ static void adios_posix1_do_read (struct adios_file_struct * fd
                                                           method->method_data;
     struct adios_var_struct * v = fd->group->vars;
 
-    struct adios_parse_buffer_struct data;
-
-    data.vars = v;
-    data.buffer = 0;
-    data.buffer_len = 0;
-
     uint32_t version = 0;
 
     adios_posix_read_version (&p->b);
     adios_parse_version (&p->b, &version);
+    version &= ADIOS_VERSION_NUM_MASK;
 
-    switch (version & ADIOS_VERSION_NUM_MASK)
+    switch (version)
     {
         case 1:
+        case 2:
         {
-            struct adios_index_process_group_struct_v1 * pg_root = 0;
+            struct adios_index_struct_v1 * index = adios_alloc_index_v1(0); // no hashtables
+            struct adios_index_process_group_struct_v1 * pg_root = index->pg_root;
             struct adios_index_process_group_struct_v1 * pg_root_temp = 0;
-            struct adios_index_var_struct_v1 * vars_root = 0;
-            struct adios_index_attribute_struct_v1 * attrs_root = 0;
 
             adios_posix_read_index_offsets (&p->b);
             adios_parse_index_offsets_v1 (&p->b);
@@ -530,10 +524,10 @@ static void adios_posix1_do_read (struct adios_file_struct * fd
             adios_parse_process_group_index_v1 (&p->b, &pg_root);
 #if 1
             adios_posix_read_vars_index (&p->b);
-            adios_parse_vars_index_v1 (&p->b, &vars_root);
+            adios_parse_vars_index_v1 (&p->b, &index->vars_root, NULL, NULL);
 
             adios_posix_read_attributes_index (&p->b);
-            adios_parse_attributes_index_v1 (&p->b, &attrs_root);
+            adios_parse_attributes_index_v1 (&p->b, &index->attrs_root);
 #endif
 
             // the three section headers
@@ -616,14 +610,12 @@ static void adios_posix1_do_read (struct adios_file_struct * fd
             }
 #endif
             adios_clear_process_group_header_v1 (&pg_header);
-            adios_clear_index_v1 (pg_root, vars_root, attrs_root);
+            adios_clear_index_v1 (index);
             break;
         }
 
         default:
-            fprintf (stderr, "POSIX1 read: file version unknown: %u\n"
-                    ,version
-                    );
+            fprintf (stderr, "POSIX1 read: file version unknown: %u\n", version);
             return;
     }
 
@@ -638,10 +630,6 @@ void adios_posix1_close (struct adios_file_struct * fd
                                                           method->method_data;
     struct adios_attribute_struct * a = fd->group->attributes;
 
-    struct adios_index_process_group_struct_v1 * new_pg_root = 0;
-    struct adios_index_var_struct_v1 * new_vars_root = 0;
-    struct adios_index_attribute_struct_v1 * new_attrs_root = 0;
-
     switch (fd->mode)
     {
         case adios_mode_write:
@@ -661,9 +649,9 @@ void adios_posix1_close (struct adios_file_struct * fd
                 if (s != fd->vars_start)
                 {
                     fprintf (stderr, "POSIX1 method tried to write %llu, "
-                                     "only wrote %llu\n"
+                                     "only wrote %lld\n"
                             ,fd->vars_start
-                            ,s
+                            ,(int64_t)s
                             );
                 }
                 fd->offset = 0;
@@ -689,9 +677,9 @@ void adios_posix1_close (struct adios_file_struct * fd
                     if (s != fd->bytes_written)
                     {
                         fprintf (stderr, "POSIX1 method tried to write %llu, "
-                                         "only wrote %llu\n"
+                                         "only wrote %lld\n"
                                 ,fd->bytes_written
-                                ,s
+                                ,(int64_t)s
                                 );
                     }
                     fd->base_offset += s;
@@ -713,9 +701,9 @@ void adios_posix1_close (struct adios_file_struct * fd
                 if (s != p->vars_header_size)
                 {
                     fprintf (stderr, "POSIX1 method tried to write %llu, "
-                                     "only wrote %llu\n"
+                                     "only wrote %lld\n"
                             ,p->vars_header_size
-                            ,s
+                            ,(int64_t)s
                             );
                 }
                 fd->offset = 0;
@@ -729,22 +717,14 @@ void adios_posix1_close (struct adios_file_struct * fd
             uint64_t index_start = fd->base_offset + fd->offset;
 
             // build index
-            adios_build_index_v1 (fd, &new_pg_root, &new_vars_root
-                                 ,&new_attrs_root
-                                 );
-            // if collective, gather the indexes from the rest and call
-            // adios_merge_index_v1 (&new_pg_root, &new_vars_root, pg, vars);
+            adios_build_index_v1 (fd, p->index);
             adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                 ,index_start, new_pg_root, new_vars_root
-                                 ,new_attrs_root
-                                 );
+                                 ,index_start, p->index);
             adios_write_version_v1 (&buffer, &buffer_size, &buffer_offset);
             adios_posix1_do_write (fd, method, buffer, buffer_offset);
 
             free (buffer);
 
-            adios_clear_index_v1 (new_pg_root, new_vars_root, new_attrs_root);
-
             break;
         }
 
@@ -765,9 +745,9 @@ void adios_posix1_close (struct adios_file_struct * fd
                 if (s != fd->vars_start)
                 {
                     fprintf (stderr, "POSIX1 method tried to write %llu, "
-                                     "only wrote %llu\n"
+                                     "only wrote %lld\n"
                             ,fd->vars_start
-                            ,s
+                            ,(int64_t)s
                             );
                 }
                 fd->offset = 0;
@@ -790,9 +770,9 @@ void adios_posix1_close (struct adios_file_struct * fd
                     if (s != fd->bytes_written)
                     {
                         fprintf (stderr, "POSIX1 method tried to write %llu, "
-                                         "only wrote %llu\n"
+                                         "only wrote %lld\n"
                                 ,fd->bytes_written
-                                ,s
+                                ,(int64_t)s
                                 );
                     }
                     fd->base_offset += s;
@@ -814,9 +794,9 @@ void adios_posix1_close (struct adios_file_struct * fd
                 if (s != p->vars_header_size)
                 {
                     fprintf (stderr, "POSIX1 method tried to write %llu, "
-                                     "only wrote %llu\n"
+                                     "only wrote %lld\n"
                             ,p->vars_header_size
-                            ,s
+                            ,(int64_t)s
                             );
                 }
                 fd->offset = 0;
@@ -828,19 +808,15 @@ void adios_posix1_close (struct adios_file_struct * fd
             uint64_t buffer_offset = 0;
             uint64_t index_start = fd->base_offset + fd->offset;
 
-            // build index
-            adios_build_index_v1 (fd, &new_pg_root, &new_vars_root
-                                 ,&new_attrs_root
-                                 );
+            // build index. Note: It merges with old indices already stored 
+            //                    in p->index in adios_posix_open's append case
+            adios_build_index_v1 (fd, p->index);
             // merge in old indicies
-            adios_merge_index_v1 (&p->old_pg_root, &p->old_vars_root
-                                 ,&p->old_attrs_root
-                                 ,new_pg_root, new_vars_root, new_attrs_root
-                                 );
+            //adios_merge_index_v1 (p->index, new_pg_root, 
+            //                      new_vars_root, new_attrs_root
+            //                     );
             adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
-                                 ,index_start, p->old_pg_root, p->old_vars_root
-                                 ,p->old_attrs_root
-                                 );
+                                 ,index_start, p->index);
             adios_write_version_v1 (&buffer, &buffer_size, &buffer_offset);
             adios_posix1_do_write (fd, method, buffer, buffer_offset);
 
@@ -872,15 +848,14 @@ void adios_posix1_close (struct adios_file_struct * fd
     }
 
     adios_posix_close_internal (&p->b);
-    adios_clear_index_v1 (p->old_pg_root, p->old_vars_root, p->old_attrs_root);
-    p->old_pg_root = 0;
-    p->old_vars_root = 0;
-    p->old_attrs_root = 0;
+    adios_clear_index_v1 (p->index);
 }
 
 void adios_posix1_finalize (int mype, struct adios_method_struct * method)
 {
-// nothing to do here
+    struct adios_POSIX1_data_struct * p = (struct adios_POSIX1_data_struct *)
+        method->method_data;
+    adios_free_index_v1 (p->index);
     if (adios_posix1_initialized)
         adios_posix1_initialized = 0;
 }
