@@ -16,8 +16,8 @@
 #include <stdint.h>
 #include <string.h>
 #include "mpi.h"
-#include "public/adios.h"
-#include "public/adios_read.h"
+#include "adios.h"
+#include "adios_read.h"
 
 #ifdef DMALLOC
 #include "dmalloc.h"
@@ -460,67 +460,67 @@ int write_file (char *fname)
     adios_write (fh, "offs4", &offs4);
     adios_write (fh, "offs5", &offs5);
     adios_write (fh, "offs6", &offs6);
-    adios_write (fh, "scalar_byte", &scalar_byte);
-    adios_write (fh, "scalar_short", &scalar_short);
-    adios_write (fh, "scalar_int", &scalar_int);
-    adios_write (fh, "scalar_long", &scalar_long);
-    adios_write (fh, "scalar_ubyte", &scalar_ubyte);
-    adios_write (fh, "scalar_ushort", &scalar_ushort);
-    adios_write (fh, "scalar_uint", &scalar_uint);
-    adios_write (fh, "scalar_ulong", &scalar_ulong);
-    adios_write (fh, "scalar_float", &scalar_float);
-    adios_write (fh, "scalar_double", &scalar_double);
-    adios_write (fh, "scalar_string", scalar_string);
-    adios_write (fh, "scalar_complex", &scalar_complex);
-    adios_write (fh, "scalar_double_complex", &scalar_double_complex);
-    adios_write (fh, "a1_byte", a1_byte);
-    adios_write (fh, "a1_short", a1_short);
-    adios_write (fh, "a1_int", a1_int);
-    adios_write (fh, "a1_long", a1_long);
-    adios_write (fh, "a1_ubyte", a1_ubyte);
-    adios_write (fh, "a1_ushort", a1_ushort);
-    adios_write (fh, "a1_uint", a1_uint);
-    adios_write (fh, "a1_ulong", a1_ulong);
-    adios_write (fh, "a1_float", a1_float);
-    adios_write (fh, "a1_double", a1_double);
-    adios_write (fh, "a1_complex", a1_complex);
-    adios_write (fh, "a1_double_complex", a1_double_complex);
-    adios_write (fh, "a2_byte", a2_byte);
-    adios_write (fh, "a2_short", a2_short);
-    adios_write (fh, "a2_int", a2_int);
-    adios_write (fh, "a2_long", a2_long);
-    adios_write (fh, "a2_ubyte", a2_ubyte);
-    adios_write (fh, "a2_ushort", a2_ushort);
-    adios_write (fh, "a2_uint", a2_uint);
-    adios_write (fh, "a2_ulong", a2_ulong);
-    adios_write (fh, "a2_float", a2_float);
-    adios_write (fh, "a2_double", a2_double);
-    adios_write (fh, "a2_complex", a2_complex);
-    adios_write (fh, "a2_double_complex", a2_double_complex);
-    adios_write (fh, "a3_byte", a3_byte);
-    adios_write (fh, "a3_short", a3_short);
-    adios_write (fh, "a3_int", a3_int);
-    adios_write (fh, "a3_long", a3_long);
-    adios_write (fh, "a3_ubyte", a3_ubyte);
-    adios_write (fh, "a3_ushort", a3_ushort);
-    adios_write (fh, "a3_uint", a3_uint);
-    adios_write (fh, "a3_ulong", a3_ulong);
-    adios_write (fh, "a3_float", a3_float);
-    adios_write (fh, "a3_double", a3_double);
-    adios_write (fh, "a3_complex", a3_complex);
-    adios_write (fh, "a3_double_complex", a3_double_complex);
-    adios_write (fh, "a6_byte", a6_byte);
-    adios_write (fh, "a6_short", a6_short);
-    adios_write (fh, "a6_int", a6_int);
-    adios_write (fh, "a6_long", a6_long);
-    adios_write (fh, "a6_ubyte", a6_ubyte);
-    adios_write (fh, "a6_ushort", a6_ushort);
-    adios_write (fh, "a6_uint", a6_uint);
-    adios_write (fh, "a6_ulong", a6_ulong);
-    adios_write (fh, "a6_float", a6_float);
-    adios_write (fh, "a6_double", a6_double);
-    adios_write (fh, "a6_complex", a6_complex);
-    adios_write (fh, "a6_double_complex", a6_double_complex);
+    adios_write (fh, "/scalars/scalar_byte", &scalar_byte);
+    adios_write (fh, "/scalars/scalar_short", &scalar_short);
+    adios_write (fh, "/scalars/scalar_int", &scalar_int);
+    adios_write (fh, "/scalars/scalar_long", &scalar_long);
+    adios_write (fh, "/scalars/scalar_ubyte", &scalar_ubyte);
+    adios_write (fh, "/scalars/scalar_ushort", &scalar_ushort);
+    adios_write (fh, "/scalars/scalar_uint", &scalar_uint);
+    adios_write (fh, "/scalars/scalar_ulong", &scalar_ulong);
+    adios_write (fh, "/scalars/scalar_float", &scalar_float);
+    adios_write (fh, "/scalars/scalar_double", &scalar_double);
+    adios_write (fh, "/scalars/scalar_string", scalar_string);
+    adios_write (fh, "/scalars/scalar_complex", &scalar_complex);
+    adios_write (fh, "/scalars/scalar_double_complex", &scalar_double_complex);
+    adios_write (fh, "/1D/a1_byte", a1_byte);
+    adios_write (fh, "/1D/a1_short", a1_short);
+    adios_write (fh, "/1D/a1_int", a1_int);
+    adios_write (fh, "/1D/a1_long", a1_long);
+    adios_write (fh, "/1D/a1_ubyte", a1_ubyte);
+    adios_write (fh, "/1D/a1_ushort", a1_ushort);
+    adios_write (fh, "/1D/a1_uint", a1_uint);
+    adios_write (fh, "/1D/a1_ulong", a1_ulong);
+    adios_write (fh, "/1D/a1_float", a1_float);
+    adios_write (fh, "/1D/a1_double", a1_double);
+    adios_write (fh, "/1D/a1_complex", a1_complex);
+    adios_write (fh, "/1D/a1_double_complex", a1_double_complex);
+    adios_write (fh, "/2D/a2_byte", a2_byte);
+    adios_write (fh, "/2D/a2_short", a2_short);
+    adios_write (fh, "/2D/a2_int", a2_int);
+    adios_write (fh, "/2D/a2_long", a2_long);
+    adios_write (fh, "/2D/a2_ubyte", a2_ubyte);
+    adios_write (fh, "/2D/a2_ushort", a2_ushort);
+    adios_write (fh, "/2D/a2_uint", a2_uint);
+    adios_write (fh, "/2D/a2_ulong", a2_ulong);
+    adios_write (fh, "/2D/a2_float", a2_float);
+    adios_write (fh, "/2D/a2_double", a2_double);
+    adios_write (fh, "/2D/a2_complex", a2_complex);
+    adios_write (fh, "/2D/a2_double_complex", a2_double_complex);
+    adios_write (fh, "/3D/a3_byte", a3_byte);
+    adios_write (fh, "/3D/a3_short", a3_short);
+    adios_write (fh, "/3D/a3_int", a3_int);
+    adios_write (fh, "/3D/a3_long", a3_long);
+    adios_write (fh, "/3D/a3_ubyte", a3_ubyte);
+    adios_write (fh, "/3D/a3_ushort", a3_ushort);
+    adios_write (fh, "/3D/a3_uint", a3_uint);
+    adios_write (fh, "/3D/a3_ulong", a3_ulong);
+    adios_write (fh, "/3D/a3_float", a3_float);
+    adios_write (fh, "/3D/a3_double", a3_double);
+    adios_write (fh, "/3D/a3_complex", a3_complex);
+    adios_write (fh, "/3D/a3_double_complex", a3_double_complex);
+    adios_write (fh, "/6D/a6_byte", a6_byte);
+    adios_write (fh, "/6D/a6_short", a6_short);
+    adios_write (fh, "/6D/a6_int", a6_int);
+    adios_write (fh, "/6D/a6_long", a6_long);
+    adios_write (fh, "/6D/a6_ubyte", a6_ubyte);
+    adios_write (fh, "/6D/a6_ushort", a6_ushort);
+    adios_write (fh, "/6D/a6_uint", a6_uint);
+    adios_write (fh, "/6D/a6_ulong", a6_ulong);
+    adios_write (fh, "/6D/a6_float", a6_float);
+    adios_write (fh, "/6D/a6_double", a6_double);
+    adios_write (fh, "/6D/a6_complex", a6_complex);
+    adios_write (fh, "/6D/a6_double_complex", a6_double_complex);
 
     adios_close (fh);
     MPI_Barrier (comm);
@@ -673,18 +673,18 @@ int read_file (char *fname)
     log ("  1D arrays... %s\n", fname);
     n = ldim1;
     sel = adios_selection_boundingbox (1, start, count); 
-    adios_schedule_read (f, sel, "1D/a1_byte", 0, 1, r1_byte);
-    adios_schedule_read (f, sel, "1D/a1_short", 0, 1, r1_short);
-    adios_schedule_read (f, sel, "1D/a1_int", 0, 1, r1_int);
-    adios_schedule_read (f, sel, "1D/a1_long", 0, 1, r1_long);
-    adios_schedule_read (f, sel, "1D/a1_ubyte", 0, 1, r1_ubyte);
-    adios_schedule_read (f, sel, "1D/a1_ushort", 0, 1, r1_ushort);
-    adios_schedule_read (f, sel, "1D/a1_uint", 0, 1, r1_uint);
-    adios_schedule_read (f, sel, "1D/a1_ulong", 0, 1, r1_ulong);
-    adios_schedule_read (f, sel, "1D/a1_float", 0, 1, r1_float);
-    adios_schedule_read (f, sel, "1D/a1_double", 0, 1, r1_double);
-    adios_schedule_read (f, sel, "1D/a1_complex", 0, 1, r1_complex);
-    adios_schedule_read (f, sel, "1D/a1_double_complex", 0, 1, r1_double_complex);
+    adios_schedule_read (f, sel, "/1D/a1_byte", 0, 1, r1_byte);
+    adios_schedule_read (f, sel, "/1D/a1_short", 0, 1, r1_short);
+    adios_schedule_read (f, sel, "/1D/a1_int", 0, 1, r1_int);
+    adios_schedule_read (f, sel, "/1D/a1_long", 0, 1, r1_long);
+    adios_schedule_read (f, sel, "/1D/a1_ubyte", 0, 1, r1_ubyte);
+    adios_schedule_read (f, sel, "/1D/a1_ushort", 0, 1, r1_ushort);
+    adios_schedule_read (f, sel, "/1D/a1_uint", 0, 1, r1_uint);
+    adios_schedule_read (f, sel, "/1D/a1_ulong", 0, 1, r1_ulong);
+    adios_schedule_read (f, sel, "/1D/a1_float", 0, 1, r1_float);
+    adios_schedule_read (f, sel, "/1D/a1_double", 0, 1, r1_double);
+    adios_schedule_read (f, sel, "/1D/a1_complex", 0, 1, r1_complex);
+    adios_schedule_read (f, sel, "/1D/a1_double_complex", 0, 1, r1_double_complex);
     adios_perform_reads (f, 1);
     adios_selection_delete (sel);
 
@@ -716,18 +716,18 @@ int read_file (char *fname)
     log ("  2D arrays... %s\n", fname);
     n = ldim1 * ldim2;
     sel = adios_selection_boundingbox (2, start, count); 
-    adios_schedule_read (f, sel, "2D/a2_byte", 0, 1, r2_byte);
-    adios_schedule_read (f, sel, "2D/a2_short", 0, 1, r2_short);
-    adios_schedule_read (f, sel, "2D/a2_int", 0, 1, r2_int);
-    adios_schedule_read (f, sel, "2D/a2_long", 0, 1, r2_long);
-    adios_schedule_read (f, sel, "2D/a2_ubyte", 0, 1, r2_ubyte);
-    adios_schedule_read (f, sel, "2D/a2_ushort", 0, 1, r2_ushort);
-    adios_schedule_read (f, sel, "2D/a2_uint", 0, 1, r2_uint);
-    adios_schedule_read (f, sel, "2D/a2_ulong", 0, 1, r2_ulong);
-    adios_schedule_read (f, sel, "2D/a2_float", 0, 1, r2_float);
-    adios_schedule_read (f, sel, "2D/a2_double", 0, 1, r2_double);
-    adios_schedule_read (f, sel, "2D/a2_complex", 0, 1, r2_complex);
-    adios_schedule_read (f, sel, "2D/a2_double_complex", 0, 1, r2_double_complex);
+    adios_schedule_read (f, sel, "/2D/a2_byte", 0, 1, r2_byte);
+    adios_schedule_read (f, sel, "/2D/a2_short", 0, 1, r2_short);
+    adios_schedule_read (f, sel, "/2D/a2_int", 0, 1, r2_int);
+    adios_schedule_read (f, sel, "/2D/a2_long", 0, 1, r2_long);
+    adios_schedule_read (f, sel, "/2D/a2_ubyte", 0, 1, r2_ubyte);
+    adios_schedule_read (f, sel, "/2D/a2_ushort", 0, 1, r2_ushort);
+    adios_schedule_read (f, sel, "/2D/a2_uint", 0, 1, r2_uint);
+    adios_schedule_read (f, sel, "/2D/a2_ulong", 0, 1, r2_ulong);
+    adios_schedule_read (f, sel, "/2D/a2_float", 0, 1, r2_float);
+    adios_schedule_read (f, sel, "/2D/a2_double", 0, 1, r2_double);
+    adios_schedule_read (f, sel, "/2D/a2_complex", 0, 1, r2_complex);
+    adios_schedule_read (f, sel, "/2D/a2_double_complex", 0, 1, r2_double_complex);
     adios_perform_reads (f, 1);
     adios_selection_delete (sel);
 
@@ -760,18 +760,18 @@ int read_file (char *fname)
     log ("  3D arrays... %s\n", fname);
     n = ldim1 * ldim2 * ldim3;
     sel = adios_selection_boundingbox (3, start, count); 
-    adios_schedule_read (f, sel, "3D/a3_byte", 0, 1, r3_byte);
-    adios_schedule_read (f, sel, "3D/a3_short", 0, 1, r3_short);
-    adios_schedule_read (f, sel, "3D/a3_int", 0, 1, r3_int);
-    adios_schedule_read (f, sel, "3D/a3_long", 0, 1, r3_long);
-    adios_schedule_read (f, sel, "3D/a3_ubyte", 0, 1, r3_ubyte);
-    adios_schedule_read (f, sel, "3D/a3_ushort", 0, 1, r3_ushort);
-    adios_schedule_read (f, sel, "3D/a3_uint", 0, 1, r3_uint);
-    adios_schedule_read (f, sel, "3D/a3_ulong", 0, 1, r3_ulong);
-    adios_schedule_read (f, sel, "3D/a3_float", 0, 1, r3_float);
-    adios_schedule_read (f, sel, "3D/a3_double", 0, 1, r3_double);
-    adios_schedule_read (f, sel, "3D/a3_complex", 0, 1, r3_complex);
-    adios_schedule_read (f, sel, "3D/a3_double_complex", 0, 1, r3_double_complex);
+    adios_schedule_read (f, sel, "/3D/a3_byte", 0, 1, r3_byte);
+    adios_schedule_read (f, sel, "/3D/a3_short", 0, 1, r3_short);
+    adios_schedule_read (f, sel, "/3D/a3_int", 0, 1, r3_int);
+    adios_schedule_read (f, sel, "/3D/a3_long", 0, 1, r3_long);
+    adios_schedule_read (f, sel, "/3D/a3_ubyte", 0, 1, r3_ubyte);
+    adios_schedule_read (f, sel, "/3D/a3_ushort", 0, 1, r3_ushort);
+    adios_schedule_read (f, sel, "/3D/a3_uint", 0, 1, r3_uint);
+    adios_schedule_read (f, sel, "/3D/a3_ulong", 0, 1, r3_ulong);
+    adios_schedule_read (f, sel, "/3D/a3_float", 0, 1, r3_float);
+    adios_schedule_read (f, sel, "/3D/a3_double", 0, 1, r3_double);
+    adios_schedule_read (f, sel, "/3D/a3_complex", 0, 1, r3_complex);
+    adios_schedule_read (f, sel, "/3D/a3_double_complex", 0, 1, r3_double_complex);
     adios_perform_reads (f, 1);
     adios_selection_delete (sel);
 
@@ -804,18 +804,18 @@ int read_file (char *fname)
     log ("  6D arrays... %s\n", fname);
     n = ldim1 * ldim2 * ldim3 * ldim4 * ldim5 * ldim6;
     sel = adios_selection_boundingbox (6, start, count); 
-    adios_schedule_read (f, sel, "6D/a6_byte", 0, 1, r6_byte);
-    adios_schedule_read (f, sel, "6D/a6_short", 0, 1, r6_short);
-    adios_schedule_read (f, sel, "6D/a6_int", 0, 1, r6_int);
-    adios_schedule_read (f, sel, "6D/a6_long", 0, 1, r6_long);
-    adios_schedule_read (f, sel, "6D/a6_ubyte", 0, 1, r6_ubyte);
-    adios_schedule_read (f, sel, "6D/a6_ushort", 0, 1, r6_ushort);
-    adios_schedule_read (f, sel, "6D/a6_uint", 0, 1, r6_uint);
-    adios_schedule_read (f, sel, "6D/a6_ulong", 0, 1, r6_ulong);
-    adios_schedule_read (f, sel, "6D/a6_float", 0, 1, r6_float);
-    adios_schedule_read (f, sel, "6D/a6_double", 0, 1, r6_double);
-    adios_schedule_read (f, sel, "6D/a6_complex", 0, 1, r6_complex);
-    adios_schedule_read (f, sel, "6D/a6_double_complex", 0, 1, r6_double_complex);
+    adios_schedule_read (f, sel, "/6D/a6_byte", 0, 1, r6_byte);
+    adios_schedule_read (f, sel, "/6D/a6_short", 0, 1, r6_short);
+    adios_schedule_read (f, sel, "/6D/a6_int", 0, 1, r6_int);
+    adios_schedule_read (f, sel, "/6D/a6_long", 0, 1, r6_long);
+    adios_schedule_read (f, sel, "/6D/a6_ubyte", 0, 1, r6_ubyte);
+    adios_schedule_read (f, sel, "/6D/a6_ushort", 0, 1, r6_ushort);
+    adios_schedule_read (f, sel, "/6D/a6_uint", 0, 1, r6_uint);
+    adios_schedule_read (f, sel, "/6D/a6_ulong", 0, 1, r6_ulong);
+    adios_schedule_read (f, sel, "/6D/a6_float", 0, 1, r6_float);
+    adios_schedule_read (f, sel, "/6D/a6_double", 0, 1, r6_double);
+    adios_schedule_read (f, sel, "/6D/a6_complex", 0, 1, r6_complex);
+    adios_schedule_read (f, sel, "/6D/a6_double_complex", 0, 1, r6_double_complex);
     adios_perform_reads (f, 1);
     adios_selection_delete (sel);
 
