@@ -24,8 +24,11 @@ cp $TRUNKDIR/examples/C/global-array/adios_global .
 cp $TRUNKDIR/examples/C/global-array/adios_read_global .
 cp $TRUNKDIR/examples/C/global-array/adios_global.xml .
 
+# Insert transform=X if requested by user
+add_transform_to_xmls
+
 echo "Run C adios_global"
-$MPIRUN $NP_MPIRUN $PROCS ./adios_global
+$MPIRUN $NP_MPIRUN $PROCS $EXEOPT ./adios_global
 EX=$?
 if [ ! -f adios_global.bp ]; then
     echo "ERROR: C version of adios_global failed. No BP file is created. Exit code=$EX"
@@ -33,16 +36,16 @@ if [ ! -f adios_global.bp ]; then
 fi
 
 echo "Check output with bpls"
-$TRUNKDIR/utils/bpls/bpls -la adios_global.bp -d -n 10  | grep -v endianness > c_bpls.txt
+$TRUNKDIR/utils/bpls/bpls -la adios_global.bp -d -n 10  | grep -v -e endianness -e 'file size' > c_bpls.txt
 diff -q c_bpls.txt $SRCDIR/reference/global_array_bpls.txt
 if [ $? != 0 ]; then
     echo "ERROR: C version of adios_global produced a file different from the reference."
-    echo "Compare \"bpls -la $PWD/adios_global.bp -d -n 10 | grep -v endianness\" to reference $SRCDIR/reference/global_array_bpls.txt"
+    echo "Compare \"bpls -la $PWD/adios_global.bp -d -n 10 | grep -v -e endianness -e 'file size'\" to reference $SRCDIR/reference/global_array_bpls.txt"
     exit 1
 fi
 
 echo "Run C adios_read_global"
-$MPIRUN $NP_MPIRUN $READPROCS ./adios_read_global | sort > c_read.txt
+$MPIRUN $NP_MPIRUN $READPROCS $EXEOPT ./adios_read_global | sort > c_read.txt
 EX=$?
 if [ $? != 0 ]; then
     echo "ERROR: C version of adios_read_global exited with $EX"
@@ -69,8 +72,11 @@ mv adios_global.bp adios_global_c.bp
 cp $TRUNKDIR/examples/Fortran/global-array/adios_global adios_global_f
 cp $TRUNKDIR/examples/Fortran/global-array/adios_global.xml .
 
+# Insert transform=X if requested by user
+add_transform_to_xmls
+
 echo "Run Fortran adios_global_f"
-$MPIRUN $NP_MPIRUN $PROCS ./adios_global_f
+$MPIRUN $NP_MPIRUN $PROCS $EXEOPT ./adios_global_f
 EX=$?
 if [ ! -f adios_global.bp ]; then
     echo "ERROR: Fortran version of adios_global failed. No BP file is created. Exit code=$EX"
@@ -78,11 +84,11 @@ if [ ! -f adios_global.bp ]; then
 fi
 
 echo "Check output with bpls"
-$TRUNKDIR/utils/bpls/bpls -la adios_global.bp -d -n 10 | grep -v endianness > f_bpls.txt
+$TRUNKDIR/utils/bpls/bpls -la adios_global.bp -d -n 10 | grep -v -e endianness -e 'file size' > f_bpls.txt
 diff -q f_bpls.txt $SRCDIR/reference/global_array_bpls.txt
 if [ $? != 0 ]; then
     echo "ERROR: Fortran version of adios_global produced a file different from the reference."
-    echo "Compare \"bpls -lav $PWD/adios_global.bp -d -n 10 | grep -v endianness\" to reference $SRCDIR/reference/global_array_bpls.txt"
+    echo "Compare \"bpls -lav $PWD/adios_global.bp -d -n 10 | grep -v -e endianness -e 'file size'\" to reference $SRCDIR/reference/global_array_bpls.txt"
     exit 1
 fi
 
