@@ -13,6 +13,8 @@
 #include "adios_transport_hooks.h"
 #include "adios_bp_v1.h"
 #include "bp_utils.h"
+#include "adios_transforms_common.h" // NCSU ALACRITY-ADIOS
+#include "adios_transforms_read.h" // NCSU ALACRITY-ADIOS
 //#include "adios_internals.h"
 
 #define DIVIDER "========================================================\n"
@@ -62,11 +64,7 @@ const char * value_to_string_ptr (enum ADIOS_DATATYPES type, void * data, uint64
 int main (int argc, char ** argv)
 {
     char * filename;
-    char * var;
-    int i = 0;
     int rc = 0;
-    uint64_t element_size = 0;
-    struct adios_bp_element_struct * element = NULL;
     struct dump_struct dump;
 
     if (argc < 2)
@@ -116,7 +114,6 @@ int main (int argc, char ** argv)
 
     struct adios_bp_buffer_struct_v1 * b = 0;
     uint32_t version = 0;
-    uint16_t flag = 0;
 
     b = malloc (sizeof (struct adios_bp_buffer_struct_v1));
     adios_buffer_struct_init (b);
@@ -131,6 +128,15 @@ int main (int argc, char ** argv)
 
     adios_posix_read_version (b);
     adios_parse_version (b, &version);
+    version = version & ADIOS_VERSION_NUM_MASK;
+    printf ("BP format version: %d\n", version);
+    if (version < 2)
+    {
+        fprintf (stderr, "bpdump: This version of bpdump can only dump BP format version 2. "
+                 "Use an older bpdump from adios 1.6 to dump this file.\n");
+        adios_posix_close_internal (b);
+        return -1;
+    }
 
     struct adios_index_process_group_struct_v1 * pg_root = 0;
     struct adios_index_process_group_struct_v1 * pg = 0;
@@ -159,7 +165,7 @@ int main (int argc, char ** argv)
     printf (DIVIDER);
     printf ("Vars Index:\n");
     adios_posix_read_vars_index (b);
-    adios_parse_vars_index_v1 (b, &vars_root);
+    adios_parse_vars_index_v1 (b, &vars_root, NULL, NULL);
     print_vars_index (vars_root);
 
     printf (DIVIDER);
@@ -222,9 +228,10 @@ int main (int argc, char ** argv)
             adios_parse_var_data_header_v1 (b, &var_header);
             print_var_header (&var_header);
 
-            if (   var_header.dims == 0
-                ||    dump.do_dump
-                   && !strcasecmp (dump.dump_var, var_header.name)
+            if ( var_header.dims == 0 || 
+                 ( dump.do_dump &&
+                   !strcasecmp (dump.dump_var, var_header.name)
+                 )
                )
             {
                 // add one for string null terminators
@@ -449,9 +456,8 @@ const char * value_to_string_ptr (enum ADIOS_DATATYPES type, void * data, uint64
 
         case adios_string:
         {
-            char * p = (char *) data;
+            //char * p = (char *) data;
             //sprintf (s, "%s", p [element]);
-            sprintf (s, "");
             fprintf (stderr, "arrays of strings not fully supported\n");
             break;
         }
@@ -473,6 +479,9 @@ const char * value_to_string_ptr (enum ADIOS_DATATYPES type, void * data, uint64
                     );
             break;
         }
+
+        case adios_unknown:
+            break;
     }
 
     return s;
@@ -482,7 +491,6 @@ void print_process_group_header (uint64_t num
                       ,struct adios_process_group_header_struct_v1 * pg_header
                       )
 {
-    int i;
     struct adios_method_info_struct_v1 * m;
 
     printf ("Process Group: %llu\n", num);
@@ -509,6 +517,30 @@ void print_vars_header (struct adios_vars_header_struct_v1 * vars_header)
     printf ("\tVars Count: %u\n", vars_header->count);
 }
 
+void print_characteristic_dims(struct adios_index_characteristic_dims_struct_v1 *dims) {
+    int j;
+
+    for (j = 0; j < dims->count; j++)
+    {
+        if (j != 0)
+            printf (",");
+        if (dims->dims [j * 3 + 1] != 0)
+        {
+            printf ("%llu:%llu:%llu"
+                   ,dims->dims [j * 3 + 0]
+                   ,dims->dims [j * 3 + 1]
+                   ,dims->dims [j * 3 + 2]
+                   );
+        }
+        else
+        {
+            printf ("%llu"
+                   ,dims->dims [j * 3 + 0]
+                   );
+        }
+    }
+}
+
 void print_var_header (struct adios_var_header_struct_v1 * var_header)
 {
     int i = 0;
@@ -641,30 +673,23 @@ void print_var_header (struct adios_var_header_struct_v1 * var_header)
     }
     if (var_header->characteristics.dims.count != 0)
     {
-        int j;
-
         printf ("\t\t\tDims (l:g:o): (");
-        for (j = 0; j < var_header->characteristics.dims.count; j++)
-        {
-            if (j != 0)
-                printf (",");
-            if (var_header->characteristics.dims.dims [j * 3 + 1] != 0)
-            {
-                printf ("%llu:%llu:%llu"
-                       ,var_header->characteristics.dims.dims [j * 3 + 0]
-                       ,var_header->characteristics.dims.dims [j * 3 + 1]
-                       ,var_header->characteristics.dims.dims [j * 3 + 2]
-                       );
-            }
-            else
-            {
-                printf ("%llu"
-                       ,var_header->characteristics.dims.dims [j * 3 + 0]
-                       );
-            }
+        print_characteristic_dims(&var_header->characteristics.dims);
+        printf(")");
         }
+
+    // NCSU ALACRITY-ADIOS - Adding printing of transform type
+    printf ("\t\t\tTransform-type(%hhu = %s)",
+            var_header->characteristics.transform.transform_type,
+            adios_transform_plugin_primary_xml_alias(var_header->characteristics.transform.transform_type));
+    if (var_header->characteristics.transform.transform_type != adios_transform_none) {
+        printf ("\t\t\tPre-transform-datatype(%s)", adios_type_to_string_int(var_header->characteristics.transform.pre_transform_type));
+        printf ("\t\t\tPre-transform-dims(l:g:o = ");
+        print_characteristic_dims(&var_header->characteristics.transform.pre_transform_dimensions);
         printf (")");
+        printf ("\t\t\tTransform-metadata-length(%hu)", var_header->characteristics.transform.transform_metadata_len);
     }
+
     printf ("\n");
 }
 
@@ -842,7 +867,6 @@ void print_var_payload (struct adios_var_header_struct_v1 * var_header
             uint64_t * position = 0;
             uint64_t * dims = 0;
             int i = 0;
-            int size_of_type = adios_get_type_size (var_header->type, "");
 
             while (d)
             {
@@ -994,7 +1018,7 @@ void print_vars_index (struct adios_index_var_struct_v1 * vars_root)
                    ,vars_root->var_name, vars_root->group_name, vars_root->id
                    );
 	}
-        char * typestr = adios_type_to_string_int (vars_root->type);
+        const char * typestr = adios_type_to_string_int (vars_root->type);
         printf ("\tDatatype: %s\n", typestr);
         //printf ("\tDatatype: %s\n", adios_type_to_string_int (vars_root->type));
         printf ("\tVars Characteristics: %llu\n"
@@ -1090,6 +1114,40 @@ void print_vars_index (struct adios_index_var_struct_v1 * vars_root)
                 }
                 printf (")");
             }
+
+            // NCSU ALACRITY-ADIOS - Print transform info
+            if (vars_root->characteristics[i].transform.transform_type != adios_transform_none) {
+                struct adios_index_characteristic_transform_struct *transform = &vars_root->characteristics[i].transform;
+                struct adios_index_characteristic_dims_struct_v1 *dims = &transform->pre_transform_dimensions;
+                int j;
+
+                printf ("\tTransform type: %s (ID = %hhu)", adios_transform_plugin_desc(transform->transform_type), transform->transform_type);
+                printf ("\tPre-transform datatype: %s", adios_type_to_string_int(transform->pre_transform_type));
+                printf ("\tPre-transform dims (l:g:o): (");
+                for (j = 0; j < dims->count; j++)
+                {
+                    if (j != 0)
+                        printf (",");
+                    if (  dims->dims [j * 3 + 1]
+                        != 0
+                       )
+                    {
+                        printf ("%llu:%llu:%llu"
+                         ,dims->dims [j * 3 + 0]
+                         ,dims->dims [j * 3 + 1]
+                         ,dims->dims [j * 3 + 2]
+                               );
+                    }
+                    else
+                    {
+                        printf ("%llu"
+                         ,dims->dims [j * 3 + 0]
+                               );
+                    }
+                }
+                printf (")");
+            }
+
             printf ("\n");
         }
 
