@@ -58,6 +58,7 @@ bool output_xml;
 bool use_regexp;           // use varmasks as regular expressions
 bool sortnames;            // sort names before listing
 bool listattrs;            // do list attributes too
+bool listmeshes;           // do list meshes too
 bool attrsonly;            // do list attributes only
 bool readattrs;            // also read all attributes and print
 bool longopt;              // -l is turned on
@@ -66,6 +67,7 @@ bool noindex;              // do no print array indices with data
 bool printByteAsChar;      // print 8 bit integer arrays as string
 bool plot;                 // dump histogram related information
 bool hidden_attrs;         // show hidden attrs in BP file
+bool show_decomp;          // show decomposition of arrays
 
 // other global variables
 char *prgname; /* argv[0] */
@@ -94,17 +96,19 @@ struct option options[] = {
     {"timestep",             no_argument,          NULL,    't'},
     {"attrs",                no_argument,          NULL,    'a'},
     {"attrsonly",            no_argument,          NULL,    'A'},
+    {"meshes",               no_argument,          NULL,    'm'},
     {"long",                 no_argument,          NULL,    'l'},
     {"string",               no_argument,          NULL,    'S'},
     {"columns",              required_argument,    NULL,    'n'}, 
     {"format",               required_argument,    NULL,    'f'}, 
     {"hidden_attrs",         no_argument,          &hidden_attrs,    true}, 
+    {"decomp",               no_argument,          NULL,    'D'},
     //    {"time",                 required_argument,    NULL,    't'}, 
     {NULL,                   0,                    NULL,    0}
 };
 
 
-static const char *optstring = "hvepyrtaAldSg:o:x:s:c:n:f:";
+static const char *optstring = "hvepyrtaAmldSDg:o:x:s:c:n:f:";
 
 // help function
 void display_help() {
@@ -119,6 +123,7 @@ void display_help() {
             "                               min/max values of arrays (no overhead to get them!)\n"
             "  --attrs     | -a           List/match attributes too\n"
             "  --attrsonly | -A           List attributes only\n"
+            "  --meshes    | -m           List meshes\n"
             /*
             "  --sort      | -r           Sort names before listing\n"
             */
@@ -144,6 +149,7 @@ void display_help() {
             "  --format    | -f \"str\"     Format string to use for one data item in print\n"
             "                               instead of the default. E.g. \"%%6.3f\"\n"
             "  --hidden_attrs             Show hidden ADIOS attributes in the file\n"
+            "  --decomp    | -D           Show decomposition of variables as layed out in file\n"
             /*
                "  --time    | -t N [M]      # print data for timesteps N..M only (or only N)\n"
                "                              default is to print all available timesteps\n"
@@ -173,8 +179,8 @@ int main( int argc, char *argv[] ) {
     ////prgname = strdup(argv[0]);
 
     /* other variables */
-    int c, last_c='_';
-    int last_opt = -1;
+    int c;
+    //int last_c = '_';
     /* Process the arguments */
     while ((c = getopt_long(argc, argv, optstring, options, NULL)) != -1) {
         switch (c) {
@@ -211,6 +217,9 @@ int main( int argc, char *argv[] ) {
                 longopt = true;
                 readattrs = true;
                 break;
+            case 'm':
+                listmeshes=true;
+                break;
             case 'n':
                 errno = 0;
                 tmp = strtol(optarg, (char **)NULL, 0);
@@ -244,6 +253,9 @@ int main( int argc, char *argv[] ) {
             case 'v':
                 verbose++;
                 break;
+            case 'D':
+                show_decomp = true;
+                break;
                 /*
                    case 't':
                    errno = 0;
@@ -284,7 +296,7 @@ int main( int argc, char *argv[] ) {
                 printf("Processing default: %c\n", c);
                 break;
         } /* end switch */
-        last_c = c;
+        //last_c = c;
     } /* end while */
 
     /* Check if we have a file defined */
@@ -355,6 +367,7 @@ void init_globals(void) {
     timestep             = false;
     sortnames            = false;
     listattrs            = false;
+    listmeshes           = false;
     attrsonly            = false;
     readattrs            = false;
     longopt              = false;
@@ -365,6 +378,7 @@ void init_globals(void) {
     hidden_attrs         = false;
     formatgiven          = false;
     printByteAsChar      = false;
+    show_decomp          = false;
     for (i=0; i<MAX_DIMS; i++) {
         istart[i]  = 0;
         icount[i]  = -1;  // read full var by default
@@ -373,8 +387,8 @@ void init_globals(void) {
 }
 
 
-#define PRINT_DIMS(str, v, n, loopvar) printf("%s = { ", str); \
-    for (loopvar=0; loopvar<n;loopvar++) printf("%lld ", v[loopvar]);    \
+#define PRINT_DIMS32(str, v, n, loopvar) printf("%s = { ", str); \
+    for (loopvar=0; loopvar<n;loopvar++) printf("%d ", v[loopvar]);    \
 printf("}")
 
 void printSettings(void) {
@@ -390,20 +404,22 @@ void printSettings(void) {
     printf("  output : %s\n", (outpath ? outpath : "stdout"));
 
     if (start != NULL) {
-        PRINT_DIMS("  start", istart, ndimsspecified,i); printf("\n");
+        PRINT_DIMS32("  start", istart, ndimsspecified,i); printf("\n");
     }
     if (count != NULL) {
-        PRINT_DIMS("  count", icount, ndimsspecified,i); printf("\n");
+        PRINT_DIMS32("  count", icount, ndimsspecified,i); printf("\n");
     }
 
     if (longopt)
         printf("      -l : show scalar values and min/max/avg of arrays\n");
     if (sortnames)
-        printf("      -t : sort names before listing\n");
+        printf("      -r : sort names before listing\n");
     if (attrsonly)
         printf("      -A : list attributes only\n");
     else if (listattrs)
         printf("      -a : list attributes too\n");
+    else if (listmeshes)
+        printf("      -m : list meshes too\n");
     if (dump)
         printf("      -d : dump matching variables and attributes\n");
     if (use_regexp)
@@ -412,6 +428,8 @@ void printSettings(void) {
         printf("      -f : dump using printf format \"%s\"\n", format);
     if (output_xml)
         printf("      -x : output data in XML format\n");
+    if (show_decomp)
+        printf("      -D : show decomposition of variables in the file\n");
     if (hidden_attrs)
         printf("         : show hidden attributes in the file\n");
 }
@@ -424,9 +442,8 @@ void printSettings(void) {
 
 void print_file_size(uint64_t size)
 {
-    static const int  sn=7;
     static const char *sm[]={"bytes", "KB", "MB", "GB", "TB", "PB", "EB"};
-    uint64_t s = size, r;
+    uint64_t s = size, r=0;
     int idx = 0;
     while ( s/1024 > 0 ) {
         r = s%1024; 
@@ -436,12 +453,15 @@ void print_file_size(uint64_t size)
     if (r > 511)
         s++;
     printf ("  file size:     %lld %s\n", s, sm[idx]); 
-
 }
 
-// prototypes
-int print_data_hist(ADIOS_VARINFO * vi, char * varname);
-int print_data_characteristics(void * min, void * max, double * avg, double * std_dev, enum ADIOS_DATATYPES adiosvartype, bool allowformat);
+
+static inline int ndigits (int n) 
+{
+    static char digitstr[32];
+    return snprintf (digitstr, 32, "%d", n);
+}
+
 
 int     nVarsMatched=0;
 
@@ -451,9 +471,7 @@ int doList_group (ADIOS_FILE *fp)
     ADIOS_VARINFO **vis; 
     enum ADIOS_DATATYPES vartype;
     int     i, j, n;             // loop vars
-    int     status;
     int     attrsize;                       // info about one attribute
-    int     mpi_comm_dummy=0;
     bool    matches;
     int     len, maxlen, maxtypelen;
     int     retval;
@@ -569,7 +587,7 @@ int doList_group (ADIOS_FILE *fp)
                 }
 
                 if (longopt || plot) {
-                    status = adios_inq_var_stat (fp, vi, timestep && timed, 0);
+                    adios_inq_var_stat (fp, vi, timestep && timed, show_decomp);
                 }
 
                 if (plot && vi->statistics && vi->statistics->histogram) {
@@ -582,27 +600,21 @@ int doList_group (ADIOS_FILE *fp)
                     if(timestep == false || timed == false ) {
 
                         fprintf(outf," = ");
-                        print_data(vi->statistics->min, 0, vartype, false); 
-
-                        fprintf(outf,"/ ");
-                        print_data(vi->statistics->max, 0, vartype, false); 
-
                         if(vartype == adios_complex || vartype == adios_double_complex) {
-
+                            // force printing (double,double) here
+                            print_data(vi->statistics->min, 0, adios_double_complex, false); 
+                            fprintf(outf,"/ ");
+                            print_data(vi->statistics->max, 0, adios_double_complex, false); 
                             fprintf(outf,"/ ");
                             print_data(vi->statistics->avg, 0, adios_double_complex, false);
-                        } else {
-
-                            fprintf(outf,"/ ");
-                            print_data(vi->statistics->avg, 0, adios_double, false);
-                        }
-
-                        if(vartype == adios_complex || vartype == adios_double_complex) {
-
                             fprintf(outf,"/ ");
                             print_data(vi->statistics->std_dev, 0, adios_double_complex, false);
                         } else {
-
+                            print_data(vi->statistics->min, 0, vartype, false); 
+                            fprintf(outf,"/ ");
+                            print_data(vi->statistics->max, 0, vartype, false); 
+                            fprintf(outf,"/ ");
+                            print_data(vi->statistics->avg, 0, adios_double, false);
                             fprintf(outf,"/ ");
                             print_data(vi->statistics->std_dev, 0, adios_double, false);
                         }
@@ -675,6 +687,12 @@ int doList_group (ADIOS_FILE *fp)
                     }
                 } // longopt && vi->statistics 
                 fprintf(outf,"\n");
+
+                if (show_decomp) {
+                    adios_inq_var_blockinfo (fp, vi);
+                    print_decomp(vi);
+                }
+
             } else {
                 // scalar
                 fprintf(outf,"  scalar");
@@ -684,6 +702,11 @@ int doList_group (ADIOS_FILE *fp)
                     matches = false; // already printed
                 }
                 fprintf(outf,"\n");
+
+                if (show_decomp) {
+                    adios_inq_var_blockinfo (fp, vi);
+                    print_decomp(vi);
+                }
             }
         }
 
@@ -713,21 +736,167 @@ int doList_group (ADIOS_FILE *fp)
     return 0;
 }                
 
+#define PRINT_ARRAY(str, ndim, dims, loopvar, format) \
+    fprintf(outf,"%s",str); \
+    if (ndim > 0) { \
+        fprintf(outf,"{"#format, dims[0]); \
+        for (loopvar=1; loopvar < ndim; loopvar++) { \
+            fprintf(outf,", "#format, dims[loopvar]); \
+        } \
+        fprintf(outf,"}\n"); \
+    } else { \
+        fprintf(outf,"empty\n"); \
+    }
 
-int doList(const char *path) {
+
+void printMeshes (ADIOS_FILE  *fp)
+{
+    int meshid,i,j;     // loop vars
+    ADIOS_FILE *meshfp = NULL;
+    int     mpi_comm_dummy=0;
+    if (fp->nmeshes==0) {
+        fprintf(outf, "Mesh info: There are no meshes defined in this file\n");
+        return;
+    }
+    fprintf(outf, "Mesh info: \n");
+    for (meshid=0; meshid < fp->nmeshes; meshid++) {
+        fprintf(outf, "  %s\n", fp->mesh_namelist[meshid]);
+        ADIOS_MESH *mi = adios_inq_mesh_byid (fp, meshid);
+        if (mi) {
+            if (meshid != mi->id)
+                fprintf(outf, "  bpls warning: meshid (=%d) != inquired mesh id (%d)\n", 
+                        meshid, mi->id);
+            if (strcmp (fp->mesh_namelist[meshid], mi->name))
+                fprintf(outf, "  bpls warning: mesh name in list (=\"%s\") != "
+                        "inquired mesh name (\"%s\")\n", 
+                        fp->mesh_namelist[meshid], mi->name);
+            if (mi->file_name) {
+               meshfp = adios_read_open_file(mi->file_name, ADIOS_READ_METHOD_BP, mpi_comm_dummy );
+               adios_complete_meshinfo (fp, meshfp, mi); 
+            }
+            fprintf(outf, "    type:         ");
+            switch (mi->type) {
+                case ADIOS_MESH_UNIFORM:
+                    fprintf(outf, "uniform\n");
+                    PRINT_ARRAY("    dimensions:   ", 
+                                 mi->uniform->num_dimensions, 
+                                 mi->uniform->dimensions, 
+                                 j, %lld)
+                    if (mi->uniform->origins) {
+                        PRINT_ARRAY("    origins:      ", 
+                                     mi->uniform->num_dimensions, 
+                                     mi->uniform->origins, 
+                                     j, %g) 
+                    }
+                    if (mi->uniform->spacings) {
+                        PRINT_ARRAY ("    spacings:     ", 
+                                     mi->uniform->num_dimensions, 
+                                     mi->uniform->spacings,
+                                     j, %g)
+                    }
+                    if (mi->uniform->maximums) {
+                        PRINT_ARRAY ("    maximums:     ", 
+                                     mi->uniform->num_dimensions, 
+                                     mi->uniform->maximums,
+                                     j, %g)
+                    }
+                    break;
+
+                case ADIOS_MESH_RECTILINEAR:
+                    fprintf(outf, "rectilinear\n");
+                    PRINT_ARRAY("    dimensions:   ", 
+                                mi->rectilinear->num_dimensions, 
+                                mi->rectilinear->dimensions, 
+                                j, %lld)
+                    if (mi->rectilinear->use_single_var) {
+                        fprintf(outf, "    coordinates:  single-var: \"%s\"\n", 
+                                mi->rectilinear->coordinates[0]);
+                    } else {
+                        fprintf(outf, "    coordinates:  multi-var: \"%s\"", 
+                                mi->rectilinear->coordinates[0]);
+                        for (i=1; i < mi->rectilinear->num_dimensions; i++) { 
+                            fprintf(outf,", \"%s\"", mi->rectilinear->coordinates[i]); 
+                        }
+                        fprintf(outf, "\n");
+                    }
+                    break;
+
+                case ADIOS_MESH_STRUCTURED:
+                    fprintf(outf, "structured\n");
+                    PRINT_ARRAY("    dimensions:   ", 
+                                mi->structured->num_dimensions, 
+                                mi->structured->dimensions, 
+                                j, %lld);
+                    if (mi->structured->use_single_var) {
+                        fprintf(outf, "    points:       single-var: \"%s\"\n", 
+                                mi->structured->points[0]);
+                    } else {
+                        fprintf(outf, "    points:       multi-var: \"%s\"", 
+                                mi->structured->points[0]);
+                        for (i=1; i < mi->structured->num_dimensions; i++) { 
+                            fprintf(outf,", \"%s\"", mi->structured->points[i]); 
+                        }
+                        fprintf(outf, "\n");
+                    }
+                    fprintf(outf, "    nspaces:      %d\n", mi->structured->nspaces);
+                    break;
+
+                case ADIOS_MESH_UNSTRUCTURED:
+                    fprintf(outf, "unstructured\n");
+                    if (mi->unstructured->nvar_points <= 1) {
+                        fprintf(outf, "    npoints:      %lld\n", mi->unstructured->npoints);
+                        fprintf(outf, "    points:       single-var: \"%s\"\n", 
+                                mi->unstructured->points[0]);
+                    } else {
+                        fprintf(outf, "    points:       multi-var: \"%s\"", 
+                                mi->unstructured->points[0]);
+                        for (i=1; i < mi->unstructured->nvar_points; i++) { 
+                            fprintf(outf,", \"%s\"", mi->unstructured->points[i]); 
+                        }
+                        fprintf(outf, "\n");
+                    }
+                    fprintf(outf, "    ncsets:       %d\n", mi->unstructured->ncsets);
+                    for (i=0; i < mi->unstructured->ncsets; i++) { 
+                        fprintf(outf, "    cell set %d:\n", i);
+                        fprintf(outf, "      cell type:  %d\n", mi->unstructured->ctypes[i]);
+                        fprintf(outf, "      ncells:     %llu\n", mi->unstructured->ccounts[i]);
+                        fprintf(outf, "      cells var:  \"%s\"\n", mi->unstructured->cdata[i]);
+                    }
+                    fprintf(outf, "    nspaces:      %d\n", mi->unstructured->nspaces);
+                    break;
+
+                default:
+                    fprintf(outf, "undefined\n");
+
+            } 
+            fprintf(outf, "    time varying: %s\n", (mi->time_varying ? "yes" : "no") );
+            adios_free_meshinfo (mi);
+            if (meshfp) 
+                adios_read_close(meshfp);
+
+        }
+    }
+    fprintf(outf, "\n");
+}
+
+int doList(const char *path) 
+{
     ADIOS_FILE  *fp;
-    int     grpid;     // loop vars
+    int     grpid;
     int     status;
     int     mpi_comm_dummy=0;
     int     nGroupsMatched=0;
     int     nGroups; // number of groups
     char  **group_namelist;
     char    init_params[128];
+    int     adios_verbose=2;
 
     if (verbose>1) printf("\nADIOS BP open: read header info from %s\n", path);
 
     // initialize BP reader
-    strcpy (init_params, "verbose=2");
+    if (verbose>1) adios_verbose = 3; // print info lines
+    if (verbose>2) adios_verbose = 4; // print debug lines
+    sprintf (init_params, "verbose=%d", adios_verbose);
     if (hidden_attrs)
         strcat (init_params, ";show_hidden_attrs");
     status = adios_read_init_method (ADIOS_READ_METHOD_BP, mpi_comm_dummy, init_params);
@@ -739,7 +908,7 @@ int doList(const char *path) {
     // open the BP file
     fp = adios_read_open_file (path, ADIOS_READ_METHOD_BP, mpi_comm_dummy); 
     if (fp == NULL) {
-        fprintf(stderr, "Error: %s\n", adios_errmsg());
+        //fprintf(stderr, "Error: %s\n", adios_errmsg());
         bpexit(7, 0);
     }
 
@@ -755,6 +924,7 @@ int doList(const char *path) {
         printf ("  of groups:     %d\n", nGroups);
         printf ("  of variables:  %d\n", fp->nvars);
         printf ("  of attributes: %d\n", fp->nattrs);
+        printf ("  of meshes:     %d\n", fp->nmeshes);
         printf ("  time steps:    %d - %d\n", fp->current_step, fp->last_step);
         print_file_size(fp->file_size);
         printf ("  bp version:    %d\n", fp->version);
@@ -764,6 +934,10 @@ int doList(const char *path) {
         printf ("\n");
     }
 
+    // Print out the meshes in the file
+    if (listmeshes) {
+        printMeshes(fp);
+    }
 
     if (grpmask) {
         // each group has to be handled separately
@@ -785,7 +959,7 @@ int doList(const char *path) {
     } else {
         doList_group (fp);
     }
-
+    
     if (grpmask != NULL && nGroupsMatched == 0) {
         fprintf(stderr, "\nError: None of the groups matched the group mask you provided: %s\n", grpmask);
         return 4;
@@ -850,9 +1024,10 @@ int print_data_hist(ADIOS_VARINFO * vi, char * varname)
     strcat(xtics, ")\n");
 
     fprintf(out_plot, "start = -0.5\npos(x) = start + x * 1\nset boxwidth 1\nset style fill solid border 5#5lt6#6\n");
-    fprintf(out_plot, xtics);
+    fputs(xtics, out_plot);
     fprintf(out_plot, "plot '%s' using 3 smooth frequency w boxes\n", hist_file);
     fprintf(out_plot, "pause -1 'Press Enter to quit'\n");
+    return 0;
 }
 
 int cmpstringp(const void *p1, const void *p2)
@@ -963,6 +1138,9 @@ int getTypeInfo( enum ADIOS_DATATYPES adiosvartype, int* elemsize)
     return 0;
 }
 
+#define PRINT_DIMS64(str, v, n, loopvar) printf("%s = { ", str); \
+    for (loopvar=0; loopvar<n;loopvar++) printf("%lld ", v[loopvar]);    \
+printf("}")
 /** Read data of a variable and print 
  * Return: 0: ok, != 0 on error
  */
@@ -984,6 +1162,7 @@ int readVar(ADIOS_FILE *fp, ADIOS_VARINFO *vi, const char * name, bool timed)
     int  status;            
     bool incdim;            // used in incremental reading in
     ADIOS_SELECTION * sel;  // boundnig box to read
+    int ndigits_dims[32];        // # of digits (to print) of each dimension 
 
     if (getTypeInfo(vi->type, &elemsize)) {
         fprintf(stderr, "Adios type %d (%s) not supported in bpls. var=%s\n", 
@@ -1087,9 +1266,12 @@ int readVar(ADIOS_FILE *fp, ADIOS_VARINFO *vi, const char * name, bool timed)
 
 
     // init s and c
+    // and calculate ndigits_dims
     for (j=0; j<tdims; j++) {
         s[j]=start_t[j];
         c[j]=readn[j];
+
+        ndigits_dims[j] = ndigits (start_t[j]+count_t[j]-1); // -1: dim=100 results in 2 digits (0..99)
     }
 
     // read until read all 'nelems' elements
@@ -1103,8 +1285,8 @@ int readVar(ADIOS_FILE *fp, ADIOS_VARINFO *vi, const char * name, bool timed)
 
         if (verbose>2) {
             printf("adios_read_var name=%s ", name);
-            PRINT_DIMS("  start", s, tdims, j); 
-            PRINT_DIMS("  count", c, tdims, j); 
+            PRINT_DIMS64("  start", s, tdims, j); 
+            PRINT_DIMS64("  count", c, tdims, j); 
             printf("  read %d elems\n", actualreadn);
         }
 
@@ -1132,7 +1314,7 @@ int readVar(ADIOS_FILE *fp, ADIOS_VARINFO *vi, const char * name, bool timed)
         //if (verbose>2) printf("  read %lld bytes\n", bytes_read);
 
         // print slice
-        print_dataset(data, vi->type, s, c, tdims); 
+        print_dataset(data, vi->type, s, c, tdims, ndigits_dims); 
 
         // prepare for next read
         sum += actualreadn;
@@ -1285,7 +1467,6 @@ int print_data_as_string(void * data, int maxlen, enum ADIOS_DATATYPES adiosvart
 {
     char *str = (char *)data;
     int len = maxlen;
-    bool cstring = false;
     switch(adiosvartype) {
         case adios_unsigned_byte:
         case adios_byte:
@@ -1424,8 +1605,7 @@ int print_data_characteristics(void * min, void * max, double * avg, double * st
             break;
 
         case adios_long_double:
-            //fprintf(outf,(f ? format : "%g "), ((double *) data)[item]);
-            fprintf(outf,(f ? format : "????????"));
+            fprintf(outf,"????????");
             break;
 
             // TO DO
@@ -1438,6 +1618,8 @@ int print_data_characteristics(void * min, void * max, double * avg, double * st
                fprintf(outf,(f ? format : "(%g,i%g)" ), ((double *) data)[2*item], ((double *) data)[2*item+1]);
                break;
              */
+        default:
+            break;
     } // end switch
     return 0;
 }
@@ -1449,7 +1631,7 @@ int print_data(void *data, int item, enum ADIOS_DATATYPES adiosvartype, bool all
         fprintf(outf, "null ");
         return 0;
     }
-    // print next data item into vstr
+    // print next data item 
     switch(adiosvartype) {
         case adios_unsigned_byte:
             fprintf(outf,(f ? format : "%hhu "), ((unsigned char *) data)[item]);
@@ -1492,8 +1674,8 @@ int print_data(void *data, int item, enum ADIOS_DATATYPES adiosvartype, bool all
 
 
         case adios_long_double:
-            //fprintf(outf,(f ? format : "%g "), ((double *) data)[item]);
-            fprintf(outf,(f ? format : "????????"));
+            fprintf(outf,(f ? format : "%Lg "), ((long double *) data)[item]);
+            //fprintf(outf,(f ? format : "????????"));
             break;
 
 
@@ -1504,15 +1686,18 @@ int print_data(void *data, int item, enum ADIOS_DATATYPES adiosvartype, bool all
         case adios_double_complex:
             fprintf(outf,(f ? format : "(%g,i%g)" ), ((double *) data)[2*item], ((double *) data)[2*item+1]);
             break;
+
+        default:
+            break;
     } // end switch
     return 0;
 }
 
 int print_dataset(void *data, enum ADIOS_DATATYPES adiosvartype, 
-        uint64_t *s, uint64_t *c, int tdims)
+        uint64_t *s, uint64_t *c, int tdims, int *ndigits)
 {
     int i,item, steps;
-    char idxstr[128], vstr[128], buf[16];
+    char idxstr[128], buf[16];
     uint64_t ids[MAX_DIMS];  // current indices
     bool roll;
 
@@ -1531,9 +1716,9 @@ int print_dataset(void *data, enum ADIOS_DATATYPES adiosvartype,
         idxstr[0] = '\0'; // empty idx string
         if (nextcol == 0) {
             if (!noindex && tdims > 0) {
-                sprintf(idxstr,"    (%lld",ids[0]);
+                sprintf(idxstr,"    (%*lld",ndigits[0], ids[0]);
                 for (i=1; i<tdims; i++) {
-                    sprintf(buf,",%lld",ids[i]);
+                    sprintf(buf,",%*lld",ndigits[i],ids[i]);
                     strcat(idxstr, buf);
                 }
                 strcat(idxstr,")    ");
@@ -1589,6 +1774,105 @@ void print_endline(void)
 }
 
 
+void print_decomp(ADIOS_VARINFO *vi)
+{
+    /* Print block info */
+    int i,j,k;
+    int ndigits_nsteps = ndigits (vi->nsteps-1);
+    if (vi->ndim == 0) 
+    {
+        // scalars
+        for (i=0; i < vi->nsteps; i++) {
+            fprintf(outf, "        step %*d: ", ndigits_nsteps, i);
+            fprintf(outf, "%d instances available\n", vi->nblocks[i]);
+        }
+        return;
+    } 
+    else 
+    {
+        // arrays
+        int ndigits_nblocks;
+        int ndigits_dims[32];
+        int blockid = 0;
+        for (k=0; k < vi->ndim; k++) {
+            // get digit lengths for each dimension
+            ndigits_dims[k] = ndigits (vi->dims[k]-1);
+        }
+
+        for (i=0; i < vi->nsteps; i++) {
+            fprintf(outf, "        step %*d: ", ndigits_nsteps, i);
+            fprintf(outf,"\n");
+            ndigits_nblocks = ndigits (vi->nblocks[i]-1);
+            for (j=0; j < vi->nblocks[i]; j++) {
+                fprintf(outf,"          block %*d: [", ndigits_nblocks, j);
+                for (k=0; k < vi->ndim; k++) {
+                    fprintf(outf, "%*lld:%*lld", 
+                            ndigits_dims[k],
+                            vi->blockinfo[j].start[k],
+                            ndigits_dims[k],
+                            vi->blockinfo[j].start[k] + vi->blockinfo[j].count[k]-1);
+                    if (k < vi->ndim-1)
+                        fprintf(outf, ", ");
+                }
+                fprintf(outf, "]");
+
+                /* Print per-block statistics if available */
+                if (longopt && vi->statistics->blocks) {
+                    fprintf(outf," = ");
+                    if (vi->statistics->blocks->mins) {
+                        if(vi->type == adios_complex || vi->type == adios_double_complex) {
+                            print_data(vi->statistics->blocks->mins[blockid], 0, adios_double_complex, false); 
+                        } else {
+                            print_data(vi->statistics->blocks->mins[blockid], 0, vi->type, false); 
+                        }
+                    } else {
+                        fprintf(outf,"N/A ");
+                    }
+
+                    fprintf(outf,"/ ");
+                    if (vi->statistics->blocks->maxs) {
+                        if(vi->type == adios_complex || vi->type == adios_double_complex) {
+                            print_data(vi->statistics->blocks->maxs[blockid], 0, adios_double_complex, false); 
+                        } else {
+                            print_data(vi->statistics->blocks->maxs[blockid], 0, vi->type, false); 
+                        }
+                    } else {
+                        fprintf(outf,"N/A ");
+                    }
+
+                    fprintf(outf,"/ ");
+                    if (vi->statistics->blocks->avgs) {
+                        if(vi->type == adios_complex || vi->type == adios_double_complex) {
+                            print_data (vi->statistics->blocks->avgs[blockid], 0, 
+                                        adios_double_complex, false);
+                        } else {
+                            print_data (vi->statistics->blocks->avgs[blockid], 0, 
+                                        adios_double, false);
+                        }
+                    } else {
+                        fprintf(outf,"N/A ");
+                    }
+
+                    fprintf(outf,"/ ");
+                    if (vi->statistics->blocks->avgs) {
+                        if(vi->type == adios_complex || vi->type == adios_double_complex) {
+                            print_data (vi->statistics->blocks->std_devs[blockid], 0, 
+                                        adios_double_complex, false);
+                        } else {
+                            print_data (vi->statistics->blocks->std_devs[blockid], 0, 
+                                        adios_double, false);
+                        }
+                    } else {
+                        fprintf(outf,"N/A ");
+                    }
+
+                }
+                fprintf(outf, "\n");
+                blockid++;
+            }
+        }
+    }
+}
 
 // parse a string "0, 3; 027" into an integer array
 // of [0,3,27] 
