@@ -49,8 +49,9 @@
 
 #define DIVIDER "========================================================\n"
 
-
+#ifndef bool
 typedef int bool;
+#endif
 #define true 1
 #define false 0
 
@@ -93,15 +94,14 @@ void display_help() {
 /** Main */
 int main( int argc, char *argv[] ) {
     int excode;
-    int from      = -1;           // split from  'from'. -1=last
-    int to        = -1;           // split until 'to'. -1=last
     char *filein  = NULL;
     char *fileout = NULL;       
 
     prgname = strdup(argv[0]);
 
     /* other variables */
-    int c, last_c='_';
+    int c;
+    //int last_c='_';
     int idx = 1;
     /* Process the arguments */
     while ((c = getopt_long(argc, argv, optstring, options, NULL)) != -1) {
@@ -128,7 +128,7 @@ int main( int argc, char *argv[] ) {
             exit (1);
             break;
         } /* end switch */
-        last_c = c;
+        //last_c = c;
         idx++;
     } /* end while */
 
@@ -203,7 +203,7 @@ int read_indexes(char *filename, bool input) {
     
     // read and parse variable index
     adios_posix_read_vars_index (bp);
-    adios_parse_vars_index_v1 (bp, &vars_root);
+    adios_parse_vars_index_v1 (bp, &vars_root, NULL, NULL);
 
     // read and parse attribute index
     adios_posix_read_attributes_index (bp);
@@ -338,7 +338,7 @@ void recalc_offsets(void) {
     struct adios_index_process_group_struct_v1 * pg = in_pg_root;
     struct adios_index_var_struct_v1       * vg = in_vars_root;
     struct adios_index_attribute_struct_v1 * ag = in_attrs_root;
-    int i, start, count;
+    int i;
 
     // process group offsets
     if (verbose) printf("Recalc offsets for new process groups\n");
@@ -350,8 +350,6 @@ void recalc_offsets(void) {
     // variable characteristics offsets
     if (verbose) printf("Recalc offsets for new variable characteristics\n");
     while (vg) {
-        start = 0;
-        count = 0;
         for (i=0; i < vg->characteristics_count; i++) {
             vg->characteristics[i].offset += out_bp->pg_index_offset;
         }
@@ -361,8 +359,6 @@ void recalc_offsets(void) {
     // attribute characteristics offsets
     if (verbose) printf("Recalc offsets for new attribute characteristics\n");
     while (ag) {
-        start = 0;
-        count = 0;
         for (i=0; i < ag->characteristics_count; i++) {
             ag->characteristics[i].offset += out_bp->pg_index_offset;
         }
@@ -390,12 +386,20 @@ int recover(int f) {
         return 1;
     }
 
+    struct adios_index_struct_v1 * idx = adios_alloc_index_v1(0);
+    idx->pg_root = out_pg_root;
+    idx->vars_root = out_vars_root;
+    idx->attrs_root = out_attrs_root;
+
     // write old index into a buffer
     adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset, index_start, 
-                          out_pg_root, out_vars_root, out_attrs_root);
+                          idx);
     if (verbose>1) fprintf(stderr, "  original index size %llu 0x%llx\n", buffer_offset, buffer_offset);
     adios_write_version_v1 (&buffer, &buffer_size, &buffer_offset);
 
+    adios_clear_index_v1 (idx);
+    adios_free_index_v1 (idx);
+
     // write index buffer out
     if (verbose>1) fprintf(stderr, "  write %llu 0x%llx bytes of indexes...\n", buffer_offset, buffer_offset);
     bytes_written = write (f, buffer, buffer_offset);
@@ -405,7 +409,7 @@ int recover(int f) {
         return 2;
     }
     if (bytes_written != buffer_offset) {
-        fprintf(stderr, "  Error: could not write complete original index into output file: %llu bytes from %llu\n"
+        fprintf(stderr, "  Error: could not write complete original index into output file: %zu bytes from %llu\n"
                 "Index in output file will be damaged.\n",
                 bytes_written, buffer_offset);
         return 2;
@@ -495,13 +499,18 @@ int append_in_to_out( const char *fileout, const char *filein) {
     uint64_t buffer_offset = 0;
     uint64_t index_start =  in_bp->pg_index_offset + out_bp->pg_index_offset;
 
+    struct adios_index_struct_v1 * idx = adios_alloc_index_v1(0);
+    idx->pg_root = out_pg_root;
+    idx->vars_root = out_vars_root;
+    idx->attrs_root = out_attrs_root;
+
     if (verbose>1) printf("  index starts at %llu (0x%llx)\n", index_start, index_start);
 
     // merge in old indicies
-    adios_merge_index_v1 (&out_pg_root, &out_vars_root, &out_attrs_root,
+    adios_merge_index_v1 (idx,
                           in_pg_root, in_vars_root, in_attrs_root);
     adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset, index_start, 
-                          out_pg_root, out_vars_root, out_attrs_root);
+                          idx);
     if (verbose>1) printf("  index size %llu 0x%llx\n", buffer_offset, buffer_offset);
     adios_write_version_v1 (&buffer, &buffer_size, &buffer_offset);
 
@@ -509,21 +518,19 @@ int append_in_to_out( const char *fileout, const char *filein) {
     if (verbose>1) printf("  write %llu 0x%llx bytes of indexes into %s\n", buffer_offset, buffer_offset, fileout);
     bytes_written = write (f, buffer, buffer_offset);
 
-    if (verbose>1) printf("  written %llu 0x%llx bytes of indexes into %s\n", bytes_written, bytes_written, fileout);
+    if (verbose>1) printf("  written %zu 0x%zx bytes of indexes into %s\n", bytes_written, bytes_written, fileout);
 
     // clean up
     free(buffer);
     close(f);
+    adios_clear_index_v1 (idx);
+    adios_free_index_v1 (idx);
 
     return 0;
 }
 
 int bpappend(char *filein, char *fileout) {
     int excode = 0;
-    uint32_t from, to;
-    uint32_t maxtime = 1;   // at least there is time=1 (single group) in a bp file
-    struct adios_bp_buffer_struct_v1 * b = 0;
-
     // open output file if exists, read and parse indexes 
     excode = read_indexes( fileout, false );
     if (excode == 128) {
